/* $Id: VLTIfileUtil.h,v 1.2 2001/11/26 12:51:04 csabet Exp $ */
#ifndef _VLTIFILEUTIL_H
#define _VLTIFILEUTIL_H
/*----------------------------------------------------------------------- */
/* FITS file utility routines                                             */
/*----------------------------------------------------------------------- */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "VLTItable.h"

int append_tables(char *infile, char *outfile);
/*----------------------------------------------------------------------- */
/*  Append all tables from one file onto another.                         */
/*  This is done by copying all records following the primary HDU from    */
/*  the input to the output.                                              */
/*   Inputs:                                                              */
/*      infile    input file name                                         */
/*      outfile   output file name                                        */
/*   Returns:                                                             */
/*      0 on success, else error                                          */
/*----------------------------------------------------------------------- */

void copy_table(char *table_type, fitsfile *infptr, fitsfile *outfptr, 
		int *ierr);
/*----------------------------------------------------------------------- */
/*  Copy a specified table from one open FITS file to another             */
/*   Inputs:                                                              */
/*      table_type table type name                                        */
/*      infptr     input file cfitsio structure                           */
/*      outfptr    output file  cfitsio structure                         */
/*   Outputs:                                                             */
/*      ierr       Error code, 0 = OK, else failed, checked on input      */
/*----------------------------------------------------------------------- */

void copy_header_keywords(fitsfile *infptr, fitsfile *outfptr, 
			  int nkeys, char** keys, int *ierr);
/*----------------------------------------------------------------------- */
/*  Copy all keywords in a specified list from one header to another.     */
/*  ESO Heirarchal keywords and specified list are copied.                */
/*   Inputs:                                                              */
/*      infptr     input file cfitsio structure                           */
/*      outfptr    output file  cfitsio structure                         */
/*      nkeys      number of entries in keys                              */
/*      keys       array of keywords to copy                              */
/*   Outputs:                                                             */
/*      ierr       Error code, 0 = OK, else failed, checked on input      */
/*----------------------------------------------------------------------- */

/* int default_fits_read_key(fitsfile *infptr, int datatype, */
void default_fits_read_key(fitsfile *infptr, int datatype, /* It is never int csabet 26.Jun.01 */
			  char *keyname, void* value, char *comment, 
			  void* deflt, int *status);
/*----------------------------------------------------------------------- */
/*  Attempt to read a given keyword but if it is not present return a     */
/*  default value.                                                        */
/*   Inputs:                                                              */
/*      infptr     input file cfitsio structure                           */
/*      datatype   cfitsio data type code                                 */
/*      keyname    keyword                                                */
/*      deflt      pointer to default value                               */
/*      nkeys      number of entries in keys                              */
/*      keys       array of keywords to copy                              */
/*   Outputs:                                                             */
/*      value      pointer to value                                       */
/*      comment    comment on keyword                                     */
/*      status     Error code, 0 = OK, else failed, checked on input      */
/*----------------------------------------------------------------------- */

/* int default_fits_read_key_str(fitsfile *infptr, */
void default_fits_read_key_str(fitsfile *infptr, /* It is never required to be int csabet 26.Jun.01 */
			      char *keyname, void* value, char *comment, 
			      void* deflt, int *status);
/*----------------------------------------------------------------------- */
/*  Attempt to read a string keyword but if it is not present return a    */
/*  default value.                                                        */
/*   Inputs:                                                              */
/*      infptr     input file cfitsio structure                           */
/*      keyname    keyword                                                */
/*      deflt      pointer to default value                               */
/*      nkeys      number of entries in keys                              */
/*      keys       array of keywords to copy                              */
/*   Outputs:                                                             */
/*      value      pointer to value                                       */
/*      comment    comment on keyword                                     */
/*      status     Error code, 0 = OK, else failed, checked on input      */
/*----------------------------------------------------------------------- */

/* int default_fits_read_key_dbl(fitsfile *infptr, */
void default_fits_read_key_dbl(fitsfile *infptr, /* It is never required to be int */
			      char *keyname, void* value, char *comment, 
			      void* deflt, int *status);
/*----------------------------------------------------------------------- */
/*  Attempt to read a double keyword but if it is not present return a    */
/*  default value.                                                        */
/*   Inputs:                                                              */
/*      infptr     input file cfitsio structure                           */
/*      keyname    keyword                                                */
/*      deflt      pointer to default value                               */
/*      nkeys      number of entries in keys                              */
/*      keys       array of keywords to copy                              */
/*   Outputs:                                                             */
/*      value      pointer to value                                       */
/*      comment    comment on keyword                                     */
/*      status     Error code, 0 = OK, else failed, checked on input      */
/*----------------------------------------------------------------------- */

/* int default_fits_read_key_flt(fitsfile *infptr, */
void default_fits_read_key_flt(fitsfile *infptr, /* csabet 26.Jun.01 */
			      char *keyname, void* value, char *comment, 
			      void* deflt, int *status);
/*----------------------------------------------------------------------- */
/*  Attempt to read a float keyword but if it is not present return a     */
/*  default value.                                                        */
/*   Inputs:                                                              */
/*      infptr     input file cfitsio structure                           */
/*      keyname    keyword                                                */
/*      deflt      pointer to default value                               */
/*      nkeys      number of entries in keys                              */
/*      keys       array of keywords to copy                              */
/*   Outputs:                                                             */
/*      value      pointer to value                                       */
/*      comment    comment on keyword                                     */
/*      status     Error code, 0 = OK, else failed, checked on input      */
/*----------------------------------------------------------------------- */

/* int default_fits_read_key_lng(fitsfile *infptr, */
void default_fits_read_key_lng(fitsfile *infptr, /* It is never typed csabet 26.Jun.01 */
			      char *keyname, void* value, char *comment, 
			      void* deflt, int *status);
/*----------------------------------------------------------------------- */
/*  Attempt to read a long keyword but if it is not present return a      */
/*  default value.                                                        */
/*   Inputs:                                                              */
/*      infptr     input file cfitsio structure                           */
/*      keyname    keyword                                                */
/*      deflt      pointer to default value                               */
/*      nkeys      number of entries in keys                              */
/*      keys       array of keywords to copy                              */
/*   Outputs:                                                             */
/*      value      pointer to value                                       */
/*      comment    comment on keyword                                     */
/*      status     Error code, 0 = OK, else failed, checked on input      */
/*----------------------------------------------------------------------- */

/*  Return a NaN  */
float give_float_NaN();
/*----------------------------------------------------------------------- */
/*  return a float NaN (not a number)                                     */
/*  uses a word with all bits on                                          */
/*----------------------------------------------------------------------- */

#endif /* end VLTIFILEUTIL_H */
