/* $Id: history.c,v 1.2 2001/11/26 12:49:53 csabet Exp $ */
/*----------------------------------------------------------------------- */
/* Routines for history keywords in FITS headers                          */
/*----------------------------------------------------------------------- */
#include "history.h"
#include <string.h>

void copy_history (fitsfile *infptr, fitsfile *outfptr, int *ierr)
/*----------------------------------------------------------------------- */
/*   Copies history records from one open FITS unit to another            */
/*   Inputs                                                               */
/*      infptr   cfitsio pointer to open input FITS file                  */
/*      outfptr   cfitsio pointer to open output FITS file                */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*----------------------------------------------------------------------- */
{
  int  i, number, morekey;
  char hiline[FLEN_CARD];

  if (*ierr!=0) return;  /* existing error condition? */
  if (infptr==NULL)  return; /* sanity check */
  if (outfptr==NULL) return; /* sanity check */


  /* how many are there? */
  fits_get_hdrspace (infptr, &number, &morekey, ierr);
  if (*ierr!=0) return;

  /* loop over input keywords copying history to output until END */
  for (i=1; i<=number; i++) {
    fits_read_record(infptr, i, hiline, ierr);
    if (*ierr!=0) break;
    if (!strncmp(hiline, "END     ", 8)) break; /* done? */
    if (!strncmp(hiline, "HISTORY ", 8)) /* found one - write it */
      fits_write_record(outfptr, hiline, ierr);
  } /* end loop copying history keywords */
} /* end copy_history */
  
void history_time (char *timestr)
/*----------------------------------------------------------------------- */
/*   Gets the current date/time string suitable for history               */
/*   Input/Output:                                                        */
/*      timestr      character large enouthg (>30) for the time string    */
/*----------------------------------------------------------------------- */
{
  char *tt;
  time_t t;
  t = time(NULL);  /* current time */
  tt = ctime(&t);  /* convert to string */
  strncpy (timestr, tt, strlen(tt)-1); /* strip linefeed */
  timestr[strlen(tt)-1] = 0;           /* now terminate */
} /* end history_time */
