/*******************************************************************/
/* VLTI VINCI DATA REDUCTION SOFTWARE (VNDRS)                      */
/*                                                                 */
/* FILE: vndrs_ASCII.c                                             */
/*                                                                 */
/* ASCII outputs related routines                                   */
/*                                                                 */
/* Written by P. Kervella - 2002				   */
/*******************************************************************/ 

#include "vndrs.h"
#include <math.h>
#include "vndrs_Globalvar.h"



/*
    Complete data processing report. pkervell 22-Oct-02.
*/

void vndrs_complete_report(double mu1_fft_avg, double mu1_wl_avg,
                           double mu1_fft_avg_bs, double mu1_wl_avg_bs,
                           double mu1_fft_avg_noncentered, double mu1_wl_avg_noncentered,
                           double mu1_fft_avg_mil, double mu1_wl_avg_mil,
                           double unweighted_mu1_fft, double unweighted_mu1_wl,
                           double avg_background_fourier, double avg_background_wavelets,
                           double background_fourier, double background_wavelets,
                           double slope_background_fft, double slope_background_wl,
                           double Fringe_speed, double Frame_rate, long N_frames_cut, long N_frames_precut,
                           double Ci2_mu1_fft_avg_min, double Ci2_mu1_fft_avg_max,
                           double Ci2_mu1_wl_avg_min, double Ci2_mu1_wl_avg_max,
                           double Ci3_mu1_fft_avg_min, double Ci3_mu1_fft_avg_max,
                           double Ci3_mu1_wl_avg_min, double Ci3_mu1_wl_avg_max,
                           double jack_fft, double jack_wl,
                           double simple_error_fft, double simple_error_wl,
                           double Wavenum_interval,
                           vector_double *Wavenum_limits_fft, vector_double *Wavenum_limits_wl,
                           double Wavenum_min_lowband_fft, double Wavenum_max_lowband_fft,
                           double Wavenum_min_lowband_wl, double Wavenum_max_lowband_wl,
                           double Wavenum_min_hi_band_fft, double Wavenum_max_hi_band_fft,
                           double Wavenum_min_hi_band_wl, double Wavenum_max_hi_band_wl,
                           double wavenum_fft_min, double wavenum_fft_max,
                           double wavenum_wl_min, double wavenum_wl_max,
                           double skew_mean_fft, double skew_mean_wl,
                           double skew_median_fft, double skew_median_wl,
                           double kurtosis_mean_fft, double kurtosis_mean_wl,
                           double kurtosis_median_fft, double kurtosis_median_wl,
                           double Wavenum_theo, double weighted_peak_pos, double weighted_peak_pos_wl,
                           double weighted_peak_pos_fft_centered, double weighted_peak_pos_wl_centered,
                           vector_double *Off_offsets_Mean, vector_double *Off_offsets_Stddev,
                           vector_double *A_Mean, vector_double *B_Mean,
                           vector_double *On_source_Mean,
                           double on_src_bg_I1, double on_src_bg_I2, double on_src_bg_PA, double on_src_bg_PB,
                           double SNR_I1, double SNR_I2, double SNR_PA, double SNR_PB,
                           double avg_photom, double average_unbalance,
                           double kappa1A, double kappa1B, double kappa2A, double kappa2B,
                           double kappa1A_err, double kappa1B_err, double kappa2A_err, double kappa2B_err,
                           double relative_kappa_norm_var,
                           vector_double *splitting_ratio_A, vector_double *splitting_ratio_B,
                           vector_double *off_source_spectrum_slopes, vector_double *number_of_pickups,
                           vector_double *gains, vector_double *RONs, long flag_remove,
                           long N_scans_processed, long N_scans_on,
                           vector_double *removed_indices, long numberOfScansRejected, long n_sig_reject,
                           long scans_rejected_0, long scans_rejected_1, long scans_rejected_2,
                           long scans_rejected_3, long scans_rejected_4,
                           long scans_rejected_5, long scans_rejected_6,
                           long scans_rejected_7, long scans_rejected_8,
                           double mu1_fft_stdev_avg, double Ci_mu1_fft_avg_min, double Ci_mu1_fft_avg_max,
                           double mu1_wl_stdev_avg, double Ci_mu1_wl_avg_min, double Ci_mu1_wl_avg_max,
                           double mu1_fft_med, double Ci_mu1_fft_med_min, double Ci_mu1_fft_med_max,
                           double mu1_wl_med, double Ci_mu1_wl_med_min, double Ci_mu1_wl_med_max,
                           double total_weight_batch, double time_batch, double baseline_batch,
                           double azimuth_batch, double u_batch, double v_batch, int Autotest_case,
                           vector_double *histogram_scale_fft, vector_double *histogram_fft,
                           vector_double *histogram_scale_wl, vector_double *histogram_wl,
                           double peak_pos_mean, double peak_pos_sigma, double peak_width_avg, double peak_width_sigma,
                           double altitude_batch_obj, double azimuth_batch_obj,
                           double wavenum_min_halfmax_fft, double wavenum_max_halfmax_fft,
                           double wavenum_min_halfmax_wl, double wavenum_max_halfmax_wl,
                           int ionic_in /* added pkervell 13-feb-03 */,
                           int select_scans /* added pkervell 21-jan-04 */,
                           int *warnings_flag, int *reject_flag /*out*/)
{
    long I, ind;
    int buffer_warnings, buffer_reject;
    long index_freq_low_fft, index_freq_high_fft, index_freq_low_wl, index_freq_high_wl;
    
       	printf("\n\n");
        printf("-----------------------------------------------------------------------------------------------------------------------\n"); 
        printf("                                                   DATA QUALITY CONTROL\n");
        printf("-----------------------------------------------------------------------------------------------------------------------\n"); 
	printf("                                       Fourier based                    Wavelets based\n\n"); 
        printf(" *** Mu^2 values consistency:\n");
        printf("     Final Mu^2 from avg spectrum:      % 6.3e                        % 6.3e\n", mu1_fft_avg, mu1_wl_avg);
        printf("     BS Mu^2 in specified band:         % 6.3e                        % 6.3e\n", mu1_fft_avg_bs, mu1_wl_avg_bs);
        printf("     Mu^2 power inside measured WN:     % 6.3e                        % 6.3e\n", mu1_fft_avg_mil, mu1_wl_avg_mil);
        printf("     Unweighted Mu^2:                   % 6.3e                        % 6.3e\n", unweighted_mu1_fft, unweighted_mu1_wl);
        printf("     Non-recentered peak Mu^2:          % 6.3e                        % 6.3e\n\n",
                mu1_fft_avg_noncentered, mu1_wl_avg_noncentered);

        printf(" *** Background characteristics:\n");
        printf("     Background removed inside band:    % 6.3e                        % 6.3e\n", avg_background_fourier, avg_background_wavelets);
        printf("     Second order residual BG:          % 6.3e                        % 6.3e\n", background_fourier, background_wavelets);
        printf("     BG average slope (ADU^2/Hz^2):     % 6.3e                        % 6.3e (0.0 if off-src debias on)\n\n",
        slope_background_fft/(Frame_rate/N_frames_cut), slope_background_wl/(Frame_rate/N_frames_cut));
        printf(" *** Error bars:\n");
        printf("     Avg 1sig BS error (68.4%% conf.):   % 6.3e                        % 6.3e\n", 
        (Ci_mu1_fft_avg_min+Ci_mu1_fft_avg_max)/2.0, (Ci_mu1_wl_avg_min+Ci_mu1_wl_avg_max)/2.0);
        printf("     Avg 95%% conf. BS error:            % 6.3e                        % 6.3e\n", 
        (Ci2_mu1_fft_avg_min+Ci2_mu1_fft_avg_max)/2.0, (Ci2_mu1_wl_avg_min+Ci2_mu1_wl_avg_max)/2.0);
        printf("     Avg 99%% conf. BS error:            % 6.3e                        % 6.3e\n", 
        (Ci3_mu1_fft_avg_min+Ci3_mu1_fft_avg_max)/2.0, (Ci3_mu1_wl_avg_min+Ci3_mu1_wl_avg_max)/2.0);
        printf("     Jackknife avg 1sig:                % 6.3e                        % 6.3e\n", jack_fft, jack_wl);
        printf("     Simple error sig/sqrt(n):          % 6.3e                        % 6.3e\n", simple_error_fft, simple_error_wl);
        printf("     Relative final avg error bar:       %7.2f%%                          %7.2f%%\n\n",
                        100.0*(mu1_fft_stdev_avg)/(mu1_fft_avg),
                        100.0*(mu1_wl_stdev_avg)/(mu1_wl_avg));

        printf(" *** Integration limits:\n");
        printf("     FINAL FFT wn limits (cm^-1):        %7.2f (%5.3f mic)               %7.2f (%5.3f mic)\n",
                    Wavenum_limits_fft->data[0]/100.0, 1000000.0 /Wavenum_limits_fft->data[0],
                    Wavenum_limits_fft->data[1]/100.0, 1000000.0 /Wavenum_limits_fft->data[1]);
        printf("     FINAL Wavelets wn limits:           %7.2f (%5.3f mic)               %7.2f (%5.3f mic)\n",
                    Wavenum_limits_wl->data[0]/100.0, 1000000.0/Wavenum_limits_wl->data[0],
                    Wavenum_limits_wl->data[1]/100.0, 1000000.0/Wavenum_limits_wl->data[1]);
        printf("     Min wavenum at half power:          %7.2f (%5.3f mic)               %7.2f (%5.3f mic)\n",
            wavenum_min_halfmax_fft / 100.0, 1000000.0 /wavenum_min_halfmax_fft,
            wavenum_min_halfmax_wl / 100.0, 1000000.0 /wavenum_min_halfmax_wl);
        printf("     Max wavenum at half power:          %7.2f (%5.3f mic)               %7.2f (%5.3f mic)\n",
            wavenum_max_halfmax_fft / 100.0, 1000000.0 /wavenum_max_halfmax_fft,
            wavenum_max_halfmax_wl / 100.0, 1000000.0 /wavenum_max_halfmax_wl);
        printf("     Min wavenum at BG level:            %7.2f (%5.3f mic)               %7.2f (%5.3f mic)\n",
                    wavenum_fft_min/100.0, 1000000.0 /wavenum_fft_min,
                    wavenum_wl_min/100.0, 1000000.0 /wavenum_wl_min);
        printf("     Max wavenum at BG level:            %7.2f (%5.3f mic)               %7.2f (%5.3f mic)\n",
                    wavenum_fft_max/100.0, 1000000.0 /wavenum_fft_max,
                    wavenum_wl_max/100.0, 1000000.0 /wavenum_wl_max);
        printf("     Relative   (BS-final)/final:       %+6.3e                        %+6.3e\n",
                        (mu1_fft_avg_bs-mu1_fft_avg)/(1.0*mu1_fft_avg), (mu1_wl_avg_bs-mu1_wl_avg)/(1.0*mu1_wl_avg));
        printf("     Relative (meas-final)/final:       %+6.3e                        %+6.3e\n",
                        (mu1_fft_avg_mil-mu1_fft_avg)/(1.0*mu1_fft_avg), (mu1_wl_avg_mil-mu1_wl_avg)/(1.0*mu1_wl_avg));
        printf("     Rel. (unweight-final)/final:       %+6.3e                        %+6.3e\n",
                        (unweighted_mu1_fft-mu1_fft_avg)/(1.0*mu1_fft_avg), (unweighted_mu1_wl-mu1_wl_avg)/(1.0*mu1_wl_avg));
        printf("     Relative final avg error bar:      % 6.3e                        % 6.3e\n",
                        (mu1_fft_stdev_avg)/(mu1_fft_avg),
                        (mu1_wl_stdev_avg)/(mu1_wl_avg));
        printf("     Wavenumber interval (cm^-1/pix):    %7.4f\n\n", Wavenum_interval/100.0);

        printf(" *** Lateral bands:\n");
        printf("     Low band min. wavenumber:           %7.2f                           %7.2f\n",
                    Wavenum_min_lowband_fft/100.0, Wavenum_min_lowband_wl/100.0);
        printf("     Low band max. wavenumber:           %7.2f                           %7.2f\n",
                    Wavenum_max_lowband_fft/100.0, Wavenum_max_lowband_wl/100.0);
        printf("     High band min. wavenumber:          %7.2f                           %7.2f\n",
                    Wavenum_min_hi_band_fft/100.0, Wavenum_min_hi_band_wl/100.0);
        printf("     High band max. wavenumber:          %7.2f                           %7.2f\n\n",
                    Wavenum_max_hi_band_fft/100.0, Wavenum_max_hi_band_wl/100.0);

        printf(" *** Fringes power peak:\n");
        printf("     Final avg wavelength:                 %5.3f +/- %5.3f microns (%7.2f cm^-1, dubious if < 100 scans)\n",
                    1000000.0 / (weighted_peak_pos * Wavenum_theo),
                    fabs(1000000.0 / (weighted_peak_pos * Wavenum_theo - peak_pos_sigma * Wavenum_interval / sqrt(N_scans_processed))
                    - 1000000.0 / (weighted_peak_pos * Wavenum_theo)),
                    weighted_peak_pos * Wavenum_theo/100.0);
        printf("     Theoretical wavelength:               %5.3f microns (%7.2f cm^-1)\n",
                    1000000.0 / Wavenum_theo, Wavenum_theo/100.0);
        printf("     Raw wavelength from wl peak pos.      %5.3f \n", 
                    1000000.0 / (peak_pos_mean * Wavenum_interval));
        printf("     Weighted PSD rel. peak pos.:       % 6.3e                        % 6.3e\n", 
                    weighted_peak_pos, weighted_peak_pos_wl);
        printf("     Weighted PSD centered peak pos.:   % 6.3e                        % 6.3e\n",
                    weighted_peak_pos_fft_centered, weighted_peak_pos_wl_centered);
        printf("     Weighted PSD wavenum (cm^-1):       %7.2f                           %7.2f\n",
                    weighted_peak_pos * Wavenum_theo/100.0, weighted_peak_pos_wl * Wavenum_theo/100.0);
        printf("     Weighted PSD wavelength (mu):         %5.3f                           %5.3f\n",
                    1000000.0 / (weighted_peak_pos * Wavenum_theo), 1000000.0 / (weighted_peak_pos_wl * Wavenum_theo));        
        printf("     Min wavenum at half power:         % 6.3e (%5.3f mic)            % 6.3e (%5.3f mic)\n",
            wavenum_min_halfmax_fft / 100.0, 1000000.0 /wavenum_min_halfmax_fft,
            wavenum_min_halfmax_wl / 100.0, 1000000.0 /wavenum_min_halfmax_wl);
        printf("     Max wavenum at half power:         % 6.3e (%5.3f mic)            % 6.3e (%5.3f mic)\n",
            wavenum_max_halfmax_fft / 100.0, 1000000.0 /wavenum_max_halfmax_fft,
            wavenum_max_halfmax_wl / 100.0, 1000000.0 /wavenum_max_halfmax_wl);
        printf("     Peak pos rms jitter (wl):           %7.2f cm^-1 (%5.2f%%)\n",
            peak_pos_sigma * Wavenum_interval / 100.0,
            100.0 * peak_pos_sigma * Wavenum_interval / (weighted_peak_pos * Wavenum_theo));
        printf("     Peak fwhm avg (wl):                 %7.2f cm^-1 (R = %3.2f)\n",
            peak_width_avg * Wavenum_interval / 100.0, (weighted_peak_pos * Wavenum_theo)/(peak_width_avg * Wavenum_interval));
        printf("     Peak fwhm rms jitter (wl):          %7.2f cm^-1 (%5.2f%%)\n\n",
            peak_width_sigma * Wavenum_interval / 100.0, 100.0 * peak_width_sigma / peak_width_avg);

        printf(" *** Histogram: histogram scale (%%), FFT, and WL number of values:\n   ");
        printf(" ");
        for(I=0;I<histogram_scale_fft->dimension;I++)
            printf("% 6.2f  ", 100.0*histogram_scale_fft->data[I]);
        printf("\n    ");
        for(I=0;I<histogram_fft->dimension;I++)
            printf("% 6ld  ", (long)histogram_fft->data[I]);
        printf("\n    ");
        for(I=0;I<histogram_wl->dimension;I++)
            printf("% 6ld  ", (long)histogram_wl->data[I]);
        printf("\n\n");

        printf(" *** Histogram properties:\n");
        printf("     Mu^2 avg skewness coef.:           %+6.3e                        %+6.3e\n",
        skew_mean_fft, skew_mean_wl);
        printf("     Mu^2 avg kurtosis:                 %+6.3e                        %+6.3e (>0 = peaked, <0 = flat)\n",
        kurtosis_mean_fft, kurtosis_mean_wl);
        printf("     Mu^2 med skew (no weighting):      %+6.3e                        %+6.3e\n",
        skew_median_fft, skew_median_wl);
        printf("     Mu^2 med kurtosis:                 %+6.3e                        %+6.3e\n\n",
        kurtosis_median_fft, kurtosis_median_wl);
        
                  
        printf(" *** Photometry:\n");
        printf("     Raw avg OFFSRC background (ADUs): I1 = %7.2f     I2 = %7.2f     PA = %7.2f     PB = %7.2f \n",
        Off_offsets_Mean->data[0],
        Off_offsets_Mean->data[1],
        Off_offsets_Mean->data[2],
        Off_offsets_Mean->data[3]);
        printf("     Avg ON-SRC background (ADUs):     I1 = %7.2f     I2 = %7.2f     PA = %7.2f     PB = %7.2f \n",
        on_src_bg_I1, on_src_bg_I2, on_src_bg_PA, on_src_bg_PB);
        printf("     Raw avg beam A flux (ADUs):       I1 = %7.2f     I2 = %7.2f     PA = %7.2f     PB = %7.2f \n",
        A_Mean->data[0],
        A_Mean->data[1],
        A_Mean->data[2],
        A_Mean->data[3]);
        printf("     Raw avg beam B flux (ADUs):       I1 = %7.2f     I2 = %7.2f     PA = %7.2f     PB = %7.2f \n",
        B_Mean->data[0],
        B_Mean->data[1],
        B_Mean->data[2],
        B_Mean->data[3]);
        printf("     Raw avg On-source flux (ADUs):    I1 = %7.2f     I2 = %7.2f     PA = %7.2f     PB = %7.2f \n",
        On_source_Mean->data[0],
        On_source_Mean->data[1],
        On_source_Mean->data[2],
        On_source_Mean->data[3]);
        printf("     Cal avg On-source flux (ADUs):    I1 = %7.2f     I2 = %7.2f     PA = %7.2f     PB = %7.2f \n",
        On_source_Mean->data[0]-Off_offsets_Mean->data[0],
        On_source_Mean->data[1]-Off_offsets_Mean->data[1],
        On_source_Mean->data[2]-Off_offsets_Mean->data[2],
        On_source_Mean->data[3]-Off_offsets_Mean->data[3]);
        printf("     Off-source RMS noise (ADUs):      I1 = %7.2f     I2 = %7.2f     PA = %7.2f     PB = %7.2f \n",
        Off_offsets_Stddev->data[0],
        Off_offsets_Stddev->data[1],
        Off_offsets_Stddev->data[2],
        Off_offsets_Stddev->data[3]);
        printf("     On-source SNRs :                  I1 = %7.2f     I2 = %7.2f     PA = %7.2f     PB = %7.2f \n",
        SNR_I1, SNR_I2, SNR_PA, SNR_PB);
        printf("     Average photom on fringes (ADU):    %.2f\n", avg_photom);
        printf("     Average photom balance:             %.2f%%\n\n", average_unbalance*100.0);

        printf(" *** Transfer matrix:\n");
        printf("     Kappa matrix :                    K1A = %6.4f     K1B = %6.4f     K2A = %6.4f     K2B = %6.4f \n",
        kappa1A, kappa1B, kappa2A, kappa2B);
        printf("     Kappa matrix errors :             K1A = %6.4f     K1B = %6.4f     K2A = %6.4f     K2B = %6.4f \n",
        kappa1A_err, kappa1B_err, kappa2A_err, kappa2B_err);
        printf("     Relative error on kappa norm:       %4.2f%%\n\n", 100.0*sqrt(relative_kappa_norm_var));

        printf(" *** Beam combiner splitting ratios:\n");
        printf("     Beware : these values are indicative only and can be severly biased by output misalignment !\n");
        printf("     Beam A only splitting ratios:     I1 =   %4.1f%%     I2 =   %4.1f%%     PA =   %4.1f%%     PB =   %4.1f%%\n",
        100.0*splitting_ratio_A->data[0], 100.0*splitting_ratio_A->data[1],
        100.0*splitting_ratio_A->data[2], 100.0*splitting_ratio_A->data[3]);
        printf("     Beam B only splitting ratios:     I1 =   %4.1f%%     I2 =   %4.1f%%     PA =   %4.1f%%     PB =   %4.1f%%\n\n",
        100.0*splitting_ratio_B->data[0], 100.0*splitting_ratio_B->data[1],
        100.0*splitting_ratio_B->data[2], 100.0*splitting_ratio_B->data[3]);
                
        printf(" *** Camera noise quality and photosites health:\n");
        printf("     Offsrc PSD slopes (ADU^2/Hz^-2):   % 6.3e       % 6.3e       % 6.3e       % 6.3e\n",
        off_source_spectrum_slopes->data[0]/(Frame_rate/N_frames_precut),
        off_source_spectrum_slopes->data[1]/(Frame_rate/N_frames_precut),
        off_source_spectrum_slopes->data[2]/(Frame_rate/N_frames_precut),
        off_source_spectrum_slopes->data[3]/(Frame_rate/N_frames_precut));
        if(vndrs_sum_scan(number_of_pickups) > 0)
        {
            printf("     Pickups/contamination detected. \n");
            printf("     Number of pickups:                  %d                %d                %d                %d\n",
            (int)number_of_pickups->data[0],
            (int)number_of_pickups->data[1],
            (int)number_of_pickups->data[2],
            (int)number_of_pickups->data[3]);
        } else
        printf("     No pickup detected. \n");
        printf("     Gains and RONs values are indicative, and valid only for strong flux targets (>50 with good balance).\n");
        printf("     Gains (e-/ADU, +/- 2.0):                %05.2f            %05.2f            %05.2f            %05.2f\n",
            gains->data[0], gains->data[1], gains->data[2], gains->data[3]);
        printf("     Readout noises (e-, +/- 4.0):           %05.2f            %05.2f            %05.2f            %05.2f\n\n",
            RONs->data[0], RONs->data[1], RONs->data[2], RONs->data[3]);
        
        printf(" *** Position of object in sky: (invalid for Autotest)\n");
        printf("     Avg altitude (deg):                    % 5.2f\n", altitude_batch_obj);
        printf("     Avg azimuth (deg, S=0, cw):            % 6.2f\n\n", azimuth_batch_obj);

        if(flag_remove > 0)
        {
            printf(" *** %ld scan(s) removed (# in series, threshold = %d sigmas around med):\n", (long)flag_remove, (int)n_sig_reject); 
            ind = 0;
            printf("    ");
            for(I=0;I<(long)(flag_remove);I++)
            {
                printf(" % 4ld", (long)removed_indices->data[I]);            
                ind++;
                if(ind >= 10)
                {
                    printf("\n    "); /* To avoid having all numbers on one line... */
                    ind = 0;
                }
            }
        printf("\n\n");          
        }
        
        if(numberOfScansRejected > 0)
        {
            printf(" *** Scan defects (in percents of the number of defective scans):\n");
            if(select_scans == 1)
                printf("     Wavelets based scan selection ACTIVATED.\n");
            else
                printf("     Wavelets based scan selection IGNORED.\n");
            printf("     Photometry too low:                                % 5ld (%2.2f %%)\n",
                scans_rejected_0, 100.0*scans_rejected_0/(numberOfScansRejected));
            printf("     Fringe packet too wide or narrow in time domain:   % 5ld (%2.2f %%)\n",
                scans_rejected_3, 100.0*scans_rejected_3/(numberOfScansRejected));
            printf("     Fringe peak not at correct freq position:          % 5ld (%2.2f %%)\n",
                scans_rejected_4, 100.0*scans_rejected_4/(numberOfScansRejected));
            printf("     Fringe peak too wide or narrow in freq domain:     % 5ld (%2.2f %%)\n",
                scans_rejected_5, 100.0*scans_rejected_5/(numberOfScansRejected));
            printf("     Large OPD jump before scan:                        % 5ld (%2.2f %%)\n",
                scans_rejected_7, 100.0*scans_rejected_7/(numberOfScansRejected));
            printf("     Fringes could not be centered (at edge):           % 5ld (%2.2f %%)\n",
                scans_rejected_8, 100.0*scans_rejected_8/(numberOfScansRejected));
            printf("     Visibility outliers (%1d sigmas):                    % 5ld (%2.2f %%)\n\n",
                (int)n_sig_reject, (long)flag_remove, 100.0*flag_remove/(numberOfScansRejected));
        }
        
        printf(" *** Total number of scans:\n");
	printf("     Scans processed:                    % 5ld  (%4.1f%%)\n",
        N_scans_processed, 100.0*(N_scans_processed)/(1.0*N_scans_on)); 
	printf("     Scans not processed:                % 5ld  (%4.1f%%)\n\n",
        numberOfScansRejected, 100.0*(numberOfScansRejected)/(1.0*N_scans_on)); 


	printf("-----------------------------------------------------------------------------------------------------------------------\n"); 
	printf("   BATCH VALUES                      Fourier based                                        Wavelets based               \n"); 
	printf("                        Val         sig      -error     +error                Val         sig     -error     +error    \n"); 
	printf("-----------------------------------------------------------------------------------------------------------------------\n"); 
	printf(" Avg final Mu^2 :    % 6.3e    %4.1e    %4.1e    %4.1e          % 6.3e    %4.1e    %4.1e    %4.1e\n",
	       mu1_fft_avg, mu1_fft_stdev_avg, Ci_mu1_fft_avg_min, Ci_mu1_fft_avg_max,
               mu1_wl_avg, mu1_wl_stdev_avg, Ci_mu1_wl_avg_min, Ci_mu1_wl_avg_max); 
	printf(" Med final Mu^2:     % 6.3e               %4.1e    %4.1e          % 6.3e               %4.1e    %4.1e\n\n",
	       mu1_fft_med, Ci_mu1_fft_med_min, Ci_mu1_fft_med_max, mu1_wl_med, Ci_mu1_wl_med_min, Ci_mu1_wl_med_max); 
	printf(" Final error bars are 1 sigma (68.4%% confidence), no weighting for median.\n\n");
	printf(" Total weight of batch:           % 6.3e\n", total_weight_batch); 
	printf(" Average batch Julian date:        %.6f\n", time_batch); 
	printf(" Average baseline (m):             %.5f\n", baseline_batch); 
	printf(" Average BL azimuth (degs,S=0,cw): %.3f\n", azimuth_batch); 
	printf(" Average (u,v) (m):                %+.5f , %+.5f\n\n", u_batch, v_batch);  
	



       	sprintf(vndrs_protocol.text, "#\n");
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "#-----------------------------------------------------------------------------------------------------------------------\n"); 
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "#                                                   DATA QUALITY CONTROL\n");
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "#-----------------------------------------------------------------------------------------------------------------------\n"); 
        vndrs_runtime_protocol();
	sprintf(vndrs_protocol.text, "#                                   Fourier based                    Wavelets based\n"); 
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# *** Absolute Mu^2 values consistency:\n");
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Final Mu^2 from avg spectrum:      % 6.3e                        % 6.3e\n", mu1_fft_avg, mu1_wl_avg);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# BS Mu^2 in specified band:         % 6.3e                        % 6.3e\n", mu1_fft_avg_bs, mu1_wl_avg_bs);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Mu^2 power inside measured WN:     % 6.3e                        % 6.3e\n", mu1_fft_avg_mil, mu1_wl_avg_mil);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Unweighted Mu^2:                   % 6.3e                        % 6.3e\n", unweighted_mu1_fft, unweighted_mu1_wl);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Non-recentered peak Mu^2:          % 6.3e                        % 6.3e\n",
        mu1_fft_avg_noncentered, mu1_wl_avg_noncentered);
        vndrs_runtime_protocol();

        sprintf(vndrs_protocol.text, "# *** Background characteristics:\n");
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Background removed inside band:    % 6.3e                        % 6.3e\n",
        avg_background_fourier, avg_background_wavelets);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Second order residual BG:          % 6.3e                        % 6.3e\n",
        background_fourier, background_wavelets);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# BG average slope (ADU^2/Hz^2):     % 6.3e                        % 6.3e (0 if debias on)\n", 	
        slope_background_fft/(Frame_rate/N_frames_cut), slope_background_wl/(Frame_rate/N_frames_cut));
        vndrs_runtime_protocol();

        sprintf(vndrs_protocol.text, "# *** Error bars:\n");
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Avg 1sig BS error (68.4pc conf.):  % 6.3e                        % 6.3e\n", 
        (Ci_mu1_fft_avg_min+Ci_mu1_fft_avg_max)/2.0, (Ci_mu1_wl_avg_min+Ci_mu1_wl_avg_max)/2.0);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Avg 95pc conf. BS error:           % 6.3e                        % 6.3e\n", 
        (Ci2_mu1_fft_avg_min+Ci2_mu1_fft_avg_max)/2.0, (Ci2_mu1_wl_avg_min+Ci2_mu1_wl_avg_max)/2.0);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Avg 99pc conf. BS error:           % 6.3e                        % 6.3e\n", 
        (Ci3_mu1_fft_avg_min+Ci3_mu1_fft_avg_max)/2.0, (Ci3_mu1_wl_avg_min+Ci3_mu1_wl_avg_max)/2.0);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Jackknife avg 1sig:                % 6.3e                        % 6.3e\n", jack_fft, jack_wl);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Simple error sig/sqrt(n):          % 6.3e                        % 6.3e\n", simple_error_fft, simple_error_wl);
        sprintf(vndrs_protocol.text, "# Relative final avg error bar:       %7.2fpc                         %7.2fpc\n\n",
                        100.0*(mu1_fft_stdev_avg)/(mu1_fft_avg),
                        100.0*(mu1_wl_stdev_avg)/(mu1_wl_avg));
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# *** Integration limits:\n");
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# FINAL FFT wn limits (cm^-1):        %7.2f (%5.3f mic)               %7.2f (%5.3f mic)\n",
                    Wavenum_limits_fft->data[0]/100.0, 1000000.0 /Wavenum_limits_fft->data[0],
                    Wavenum_limits_fft->data[1]/100.0, 1000000.0 /Wavenum_limits_fft->data[1]);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# FINAL Wavelets wn limits:           %7.2f (%5.3f mic)               %7.2f (%5.3f mic)\n",
                    Wavenum_limits_wl->data[0]/100.0, 1000000.0/Wavenum_limits_wl->data[0],
                    Wavenum_limits_wl->data[1]/100.0, 1000000.0/Wavenum_limits_wl->data[1]);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Min wavenum at half power:          %7.2f (%5.3f mic)               %7.2f (%5.3f mic)\n",
            wavenum_min_halfmax_fft / 100.0, 1000000.0 /wavenum_min_halfmax_fft,
            wavenum_min_halfmax_wl / 100.0, 1000000.0 /wavenum_min_halfmax_wl);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Max wavenum at half power:          %7.2f (%5.3f mic)               %7.2f (%5.3f mic)\n",
            wavenum_max_halfmax_fft / 100.0, 1000000.0 /wavenum_max_halfmax_fft,
            wavenum_max_halfmax_wl / 100.0, 1000000.0 /wavenum_max_halfmax_wl);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Min wavenum at BG level:            %7.2f (%5.3f mic)               %7.2f (%5.3f mic)\n",
                    wavenum_fft_min/100.0, 1000000.0 /wavenum_fft_min,
                    wavenum_wl_min/100.0, 1000000.0 /wavenum_wl_min);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Max wavenum at BG level:            %7.2f (%5.3f mic)               %7.2f (%5.3f mic)\n",
                    wavenum_fft_max/100.0, 1000000.0 /wavenum_fft_max,
                    wavenum_wl_max/100.0, 1000000.0 /wavenum_wl_max);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Relative   (BS-final)/final:       %+6.3e                        %+6.3e\n",
                        (mu1_fft_avg_bs-mu1_fft_avg)/(1.0*mu1_fft_avg), (mu1_wl_avg_bs-mu1_wl_avg)/(1.0*mu1_wl_avg));
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Relative (meas-final)/final:       %+6.3e                        %+6.3e\n",
                        (mu1_fft_avg_mil-mu1_fft_avg)/(1.0*mu1_fft_avg), (mu1_wl_avg_mil-mu1_wl_avg)/(1.0*mu1_wl_avg));
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Rel. (unweight-final)/final:       %+6.3e                        %+6.3e\n",
                        (unweighted_mu1_fft-mu1_fft_avg)/(1.0*mu1_fft_avg), (unweighted_mu1_wl-mu1_wl_avg)/(1.0*mu1_wl_avg));
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Relative final avg error bar:      % 6.3e                        % 6.3e\n",
                        (Ci_mu1_fft_avg_min + Ci_mu1_fft_avg_max)/(2.0*mu1_fft_avg),
                        (Ci_mu1_wl_avg_min + Ci_mu1_wl_avg_max)/(2.0*mu1_wl_avg));
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Wavenumber interval (cm^-1/pix):    %7.4f\n", Wavenum_interval/100.0);
        vndrs_runtime_protocol();

        sprintf(vndrs_protocol.text, "# *** Lateral bands:\n");
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Low band min. wavenumber:           %7.2f                           %7.2f\n",
                    Wavenum_min_lowband_fft/100.0, Wavenum_min_lowband_wl/100.0);
        vndrs_runtime_protocol(); 
        sprintf(vndrs_protocol.text, "# Low band max. wavenumber:           %7.2f                           %7.2f\n",
                    Wavenum_max_lowband_fft/100.0, Wavenum_max_lowband_wl/100.0);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# High band min. wavenumber:          %7.2f                           %7.2f\n",
                    Wavenum_min_hi_band_fft/100.0, Wavenum_min_hi_band_wl/100.0);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# High band max. wavenumber:          %7.2f                           %7.2f\n",
                    Wavenum_max_hi_band_fft/100.0, Wavenum_max_hi_band_wl/100.0);
        vndrs_runtime_protocol();

        sprintf(vndrs_protocol.text, "# *** Fringes power peak position and jitter:\n");
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Final avg wavelength:                 %5.3f +/- %5.3f microns (%7.2f cm^-1, dubious if < 100 scans)\n",
                    1000000.0 / (weighted_peak_pos * Wavenum_theo),
                    fabs(1000000.0 / (weighted_peak_pos * Wavenum_theo - peak_pos_sigma * Wavenum_interval / sqrt(N_scans_processed))
                    - 1000000.0 / (weighted_peak_pos * Wavenum_theo)),
                    weighted_peak_pos * Wavenum_theo/100.0);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Theoretical wavelength:               %5.3f microns (%7.2f cm^-1)\n",
                    1000000.0 / Wavenum_theo, Wavenum_theo/100.0);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Raw wavelength from wl peak pos.      %5.3f \n", 
                    1000000.0 / (peak_pos_mean * Wavenum_interval));
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Weighted PSD rel. peak pos.:       % 6.3e                        % 6.3e\n", 
                    weighted_peak_pos, weighted_peak_pos_wl);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Weighted PSD centered peak pos.:   % 6.3e                        % 6.3e\n",
                    weighted_peak_pos_fft_centered, weighted_peak_pos_wl_centered);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Weighted PSD wavenum (cm^-1):       %7.2f                           %7.2f\n",
         weighted_peak_pos * Wavenum_theo/100.0, weighted_peak_pos_wl * Wavenum_theo/100.0);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Weighted PSD wavelength (mu):         %5.3f                           %5.3f\n",
         1000000.0 / (weighted_peak_pos * Wavenum_theo), 1000000.0 / (weighted_peak_pos_wl * Wavenum_theo));        
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Min wavenum at half power:         % 6.3e (%5.3f mic)            % 6.3e (%5.3f mic)\n",
            wavenum_min_halfmax_fft / 100.0, 1000000.0 /wavenum_min_halfmax_fft,
            wavenum_min_halfmax_wl / 100.0, 1000000.0 /wavenum_min_halfmax_wl);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Max wavenum at half power:         % 6.3e (%5.3f mic)            % 6.3e (%5.3f mic)\n",
            wavenum_max_halfmax_fft / 100.0, 1000000.0 /wavenum_max_halfmax_fft,
            wavenum_max_halfmax_wl / 100.0, 1000000.0 /wavenum_max_halfmax_wl);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Peak pos rms jitter (wl):           %7.2f cm^-1 (%.2f%%)\n",
            peak_pos_sigma * Wavenum_interval / 100.0, 100.0 * peak_pos_sigma * Wavenum_interval / (weighted_peak_pos * Wavenum_theo));
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Peak fwhm avg (wl):                 %7.2f cm^-1 (R = %.2f)\n",
            peak_width_avg * Wavenum_interval / 100.0, (weighted_peak_pos * Wavenum_theo)/(peak_width_avg * Wavenum_interval));
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Peak fwhm rms jitter (wl):          %7.2f cm^-1 (%.2f%%)\n\n",
            peak_width_sigma * Wavenum_interval / 100.0, 100.0 * peak_width_sigma / peak_width_avg);
        vndrs_runtime_protocol();

        sprintf(vndrs_protocol.text, "# *** Histogram: histogram scale (%%), FFT, and WL number of values:\n   ");
        vndrs_runtime_protocol();
        for(I=0;I<histogram_scale_fft->dimension;I++)
        {
            sprintf(vndrs_protocol.text, "% 6.2f  ", 100.0*histogram_scale_fft->data[I]);
            vndrs_runtime_protocol();
        }
        sprintf(vndrs_protocol.text, "\n");
        vndrs_runtime_protocol();
        for(I=0;I<histogram_fft->dimension;I++)
        {
            sprintf(vndrs_protocol.text, "% 6ld  ", (long)histogram_fft->data[I]);
            vndrs_runtime_protocol();
        }
        sprintf(vndrs_protocol.text, "\n");
        vndrs_runtime_protocol();
        for(I=0;I<histogram_wl->dimension;I++)
        {
            sprintf(vndrs_protocol.text, "% 6ld  ", (long)histogram_wl->data[I]);
            vndrs_runtime_protocol();
        }
        sprintf(vndrs_protocol.text, "\n");
        vndrs_runtime_protocol();

        sprintf(vndrs_protocol.text, "# *** Histogram properties:\n");
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Mu^2 skewness coef.:               %+6.3e                        %+6.3e (ok if abs < 0.5)\n",
        skew_mean_fft, skew_mean_wl);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Mu^2 avg kurtosis:                 %+6.3e                        %+6.3e (>0 = peaked, <0 = flat)\n",
        kurtosis_mean_fft, kurtosis_mean_wl);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Mu^2 med skew (no weighting):      %+6.3e                        %+6.3e\n",
        skew_median_fft, skew_median_wl);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Mu^2 med kurtosis:                 %+6.3e                        %+6.3e\n",
        kurtosis_median_fft, kurtosis_median_wl);
        vndrs_runtime_protocol();

        sprintf(vndrs_protocol.text, "# *** Photometry:\n");
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Raw avg OFFSRC background (ADUs): I1 = %7.2f  I2 = %7.2f  PA = %7.2f  PB = %7.2f \n",
        Off_offsets_Mean->data[0],
        Off_offsets_Mean->data[1],
        Off_offsets_Mean->data[2],
        Off_offsets_Mean->data[3]);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Avg ON-SRC background (ADUs):     I1 = %7.2f     I2 = %7.2f     PA = %7.2f     PB = %7.2f \n",
        on_src_bg_I1, on_src_bg_I2, on_src_bg_PA, on_src_bg_PB);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Raw avg beam A flux (ADUs):       I1 = %7.2f  I2 = %7.2f  PA = %7.2f  PB = %7.2f \n",
        A_Mean->data[0],
        A_Mean->data[1],
        A_Mean->data[2],
        A_Mean->data[3]);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Raw avg beam B flux (ADUs):       I1 = %7.2f  I2 = %7.2f  PA = %7.2f  PB = %7.2f \n",
        B_Mean->data[0],
        B_Mean->data[1],
        B_Mean->data[2],
        B_Mean->data[3]);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Raw avg On-source flux (ADUs):    I1 = %7.2f  I2 = %7.2f  PA = %7.2f  PB = %7.2f \n",
        On_source_Mean->data[0],
        On_source_Mean->data[1],
        On_source_Mean->data[2],
        On_source_Mean->data[3]);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Cal avg On-source flux (ADUs):    I1 = %7.2f  I2 = %7.2f  PA = %7.2f  PB = %7.2f \n",
        On_source_Mean->data[0]-Off_offsets_Mean->data[0],
        On_source_Mean->data[1]-Off_offsets_Mean->data[1],
        On_source_Mean->data[2]-Off_offsets_Mean->data[2],
        On_source_Mean->data[3]-Off_offsets_Mean->data[3]);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Off-source RMS noise (ADUs):      I1 = %7.2f  I2 = %7.2f  PA = %7.2f  PB = %7.2f \n",
        Off_offsets_Stddev->data[0],
        Off_offsets_Stddev->data[1],
        Off_offsets_Stddev->data[2],
        Off_offsets_Stddev->data[3]);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# On-source SNRs :                  I1 = %7.2f  I2 = %7.2f  PA = %7.2f  PB = %7.2f \n",
        SNR_I1, SNR_I2, SNR_PA, SNR_PB);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Average photom on fringes (ADU):    %.2f\n", avg_photom);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Average photom balance:             %.2fpc\n", average_unbalance*100.0);
        vndrs_runtime_protocol();

        sprintf(vndrs_protocol.text, "# *** Transfer matrix:\n");
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Kappa matrix :                    K1A = %6.4f  K1B = %6.4f  K2A = %6.4f  K2B = %6.4f \n",
        kappa1A, kappa1B, kappa2A, kappa2B);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Kappa matrix errors :             K1A = %6.4f  K1B = %6.4f  K2A = %6.4f  K2B = %6.4f \n",
        kappa1A_err, kappa1B_err, kappa2A_err, kappa2B_err);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Relative error on kappa norm:       %.5f\n", sqrt(relative_kappa_norm_var));
        vndrs_runtime_protocol();
        
        sprintf(vndrs_protocol.text, "# *** Beam combiner splitting ratios:\n");
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Beware : these values are indicative only and can be severly biased by output misalignment !\n");
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Beam A only splitting ratios:     I1 = %6.2fpc  I2 = %6.2fpc  PA = %6.2fpc  PB = %6.2fpc\n",
        100.0*splitting_ratio_A->data[0], 100.0*splitting_ratio_A->data[1],
        100.0*splitting_ratio_A->data[2], 100.0*splitting_ratio_A->data[3]);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Beam B only splitting ratios:     I1 = %6.2fpc  I2 = %6.2fpc  PA = %6.2fpc  PB = %6.2fpc\n",
        100.0*splitting_ratio_B->data[0], 100.0*splitting_ratio_B->data[1],
        100.0*splitting_ratio_B->data[2], 100.0*splitting_ratio_B->data[3]);
        vndrs_runtime_protocol();
                
        sprintf(vndrs_protocol.text, "# *** Camera noise quality and photosites health:\n");
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Offsrc PSD slopes (ADU^2/Hz^-2):   % 6.3e       % 6.3e       % 6.3e       % 6.3e\n",
        off_source_spectrum_slopes->data[0]/(Frame_rate/N_frames_precut),
        off_source_spectrum_slopes->data[1]/(Frame_rate/N_frames_precut),
        off_source_spectrum_slopes->data[2]/(Frame_rate/N_frames_precut),
        off_source_spectrum_slopes->data[3]/(Frame_rate/N_frames_precut));
        vndrs_runtime_protocol();
        if(vndrs_sum_scan(number_of_pickups) > 0)
        {
            sprintf(vndrs_protocol.text, "# Pickups/contamination detected. \n");
            vndrs_runtime_protocol();
            sprintf(vndrs_protocol.text, "# Number of pickups:                  %d                %d                %d                %d\n",
            (int)number_of_pickups->data[0],
            (int)number_of_pickups->data[1],
            (int)number_of_pickups->data[2],
            (int)number_of_pickups->data[3]);
            vndrs_runtime_protocol();
        }
        else
        {
        sprintf(vndrs_protocol.text, "# No pickup detected.\n");
        vndrs_runtime_protocol();
        }
        sprintf(vndrs_protocol.text, "# Gains and RONs values are valid only for photometry >50 and good balance of flux.\n");
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Gains (e-/ADU, +/- 2.0): %5.2f   %5.2f  %5.2f  %5.2f\n",
            gains->data[0], gains->data[1], gains->data[2], gains->data[3]);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# RONs (e-, +/- 4.0):      %5.2f   %5.2f  %5.2f  %5.2f\n",
            RONs->data[0], RONs->data[1], RONs->data[2], RONs->data[3]);
        vndrs_runtime_protocol();

        sprintf(vndrs_protocol.text, "# *** Position of object in sky: (invalid for Autotest)\n");
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Avg altitude (deg):                     % 5.2f\n", altitude_batch_obj);
        vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Avg azimuth (deg, S=0, cw):             % 6.2f\n", azimuth_batch_obj);
        vndrs_runtime_protocol();
        
        if(flag_remove > 0)
        {
            sprintf(vndrs_protocol.text, "# %d scan(s) removed (# in series, threshold = %d sigmas around med):\n", (int)flag_remove, (int)n_sig_reject); 
            vndrs_runtime_protocol();
            ind = 0;
            for(I=0;I<(long)(flag_remove);I++)
            {
                sprintf(vndrs_protocol.text, " % 4ld", (long)removed_indices->data[I]);            
                vndrs_runtime_protocol();
                ind++;
                if(ind >= 10)
                {
                    sprintf(vndrs_protocol.text, "\n"); /* To avoid having all numbers on one line... */
                    vndrs_runtime_protocol();
                   ind = 0;
                }
            }
            sprintf(vndrs_protocol.text, "\n#\n");          
            vndrs_runtime_protocol();
        }
        
        if(numberOfScansRejected > 0)
        {
            sprintf(vndrs_protocol.text, "# Interferogram defects (in percents of the number of defective scans):\n");
            vndrs_runtime_protocol();
            if(select_scans == 1)
                {
                sprintf(vndrs_protocol.text, "# Wavelets based scan selection ACTIVATED.\n");
                vndrs_runtime_protocol();
                }
            else
                {
                sprintf(vndrs_protocol.text, "# Wavelets based scan selection IGNORED.\n");
                vndrs_runtime_protocol();
                }
            sprintf(vndrs_protocol.text, "# Photometry too low:                                % 5ld (%2.2f pc)\n",
                scans_rejected_0, 100.0*scans_rejected_0/(numberOfScansRejected));
            vndrs_runtime_protocol();
            sprintf(vndrs_protocol.text, "# Fringe packet too wide or narrow in time domain:   % 5ld (%2.2f pc)\n",
                scans_rejected_3, 100.0*scans_rejected_3/(numberOfScansRejected));
            vndrs_runtime_protocol();
            sprintf(vndrs_protocol.text, "# Fringe peak not at correct freq position:          % 5ld (%2.2f pc)\n",
                scans_rejected_4, 100.0*scans_rejected_4/(numberOfScansRejected));
            vndrs_runtime_protocol();
            sprintf(vndrs_protocol.text, "# Fringe peak too wide or narrow in freq domain:     % 5ld (%2.2f pc)\n",
                scans_rejected_5, 100.0*scans_rejected_5/(numberOfScansRejected));
            vndrs_runtime_protocol();
            sprintf(vndrs_protocol.text, "# Large OPD jump before scan:                        % 5ld (%2.2f pc)\n",
                scans_rejected_7, 100.0*scans_rejected_7/(numberOfScansRejected));
            vndrs_runtime_protocol();
            sprintf(vndrs_protocol.text, "# Fringes could not be centered (at edge):           % 5ld (%2.2f pc)\n",
                scans_rejected_8, 100.0*scans_rejected_8/(numberOfScansRejected));
            vndrs_runtime_protocol();
            sprintf(vndrs_protocol.text, "# Visibility outliers (%1d sigmas):                    % 5ld (%2.2fpc)\n",
                    (int)n_sig_reject, (long)flag_remove, 100.0*flag_remove/(numberOfScansRejected));
            vndrs_runtime_protocol();
        }
        
	sprintf(vndrs_protocol.text, "# Scans processed:                    % 5ld  (%4.1fpc)\n",
        N_scans_processed, 100.0*(N_scans_processed)/(1.0*N_scans_on)); 
        vndrs_runtime_protocol();
	sprintf(vndrs_protocol.text, "# Scans not processed:                % 5ld  (%4.1fpc)\n",
        numberOfScansRejected, 100.0*(numberOfScansRejected)/(1.0*N_scans_on)); 
        vndrs_runtime_protocol();


	sprintf(vndrs_protocol.text, "#-----------------------------------------------------------------------------------------------------------------------\n"); 
        vndrs_runtime_protocol();
	sprintf(vndrs_protocol.text, "#   BATCH VALUES                      Fourier based                                        Wavelets based               \n"); 
        vndrs_runtime_protocol();
	sprintf(vndrs_protocol.text, "#                        Val         sig      -error     +error                Val         sig     -error     +error    \n"); 
        vndrs_runtime_protocol();
	sprintf(vndrs_protocol.text, "#-----------------------------------------------------------------------------------------------------------------------\n"); 
        vndrs_runtime_protocol();
	sprintf(vndrs_protocol.text, "# Avg final Mu^2 :    % 6.3e    %4.1e    %4.1e    %4.1e          % 6.3e    %4.1e    %4.1e    %4.1e\n",
	       mu1_fft_avg, mu1_fft_stdev_avg, Ci_mu1_fft_avg_min, Ci_mu1_fft_avg_max,
               mu1_wl_avg, mu1_wl_stdev_avg, Ci_mu1_wl_avg_min, Ci_mu1_wl_avg_max); 
        vndrs_runtime_protocol();
	sprintf(vndrs_protocol.text, "# Med final Mu^2:     % 6.3e               %4.1e    %4.1e          % 6.3e               %4.1e    %4.1e\n",
	       mu1_fft_med, Ci_mu1_fft_med_min, Ci_mu1_fft_med_max, mu1_wl_med, Ci_mu1_wl_med_min, Ci_mu1_wl_med_max); 
        vndrs_runtime_protocol();
	sprintf(vndrs_protocol.text, "# Final error bars are 1 sigma (68pc confidence), no weighting for median.\n");
        vndrs_runtime_protocol();
	sprintf(vndrs_protocol.text, "# Total weight of batch:           % 6.3e\n", total_weight_batch); 
        vndrs_runtime_protocol();
	sprintf(vndrs_protocol.text, "# Average batch Julian date:        %.6f\n", time_batch); 
        vndrs_runtime_protocol();
	sprintf(vndrs_protocol.text, "# Average baseline (m):             %.5f\n", baseline_batch); 
        vndrs_runtime_protocol();
	sprintf(vndrs_protocol.text, "# Average BL azimuth (degs,S=0,cw): %.3f\n", azimuth_batch); 
        vndrs_runtime_protocol();
	sprintf(vndrs_protocol.text, "# Average (u,v) (m):                %+.5f , %+.5f\n", u_batch, v_batch);  
        vndrs_runtime_protocol();
	sprintf(vndrs_protocol.text, "#\n");  
        vndrs_runtime_protocol();


    /* New: warnings about the data quality - pkervell 16-Oct-02 */
    printf(" DATA QUALITY WARNING(S) (* = problem, *** = reject data):  \n");
    sprintf(vndrs_protocol.text,"# DATA QUALITY WARNING(S)  (* = problem, *** = reject data):\n");
    vndrs_runtime_protocol();
    
    buffer_warnings = 0;

    vndrs_warnings(N_scans_processed, N_scans_on, SNR_PA, SNR_PB, avg_photom,
    mu1_fft_avg_mil, mu1_fft_avg, mu1_wl_avg_mil, mu1_wl_avg,
    skew_mean_fft/*10*/, skew_mean_wl, weighted_peak_pos, Wavenum_theo, average_unbalance,
    number_of_pickups, off_source_spectrum_slopes, Frame_rate, N_frames_precut,
    slope_background_fft, slope_background_wl/*20*/, N_frames_cut,
    mu1_fft_med, mu1_fft_stdev_avg, mu1_wl_med, mu1_wl_stdev_avg,
    gains, RONs, kappa1A, kappa1B, kappa2A/*30*/, kappa2B, Autotest_case,
    splitting_ratio_A, splitting_ratio_B, ionic_in, &buffer_warnings, &buffer_reject);
    
    *warnings_flag = buffer_warnings;
    *reject_flag = buffer_reject;

    /*---*/

    index_freq_low_fft  = (long)(Wavenum_min_lowband_fft/fabs(Frame_rate / Fringe_speed / N_frames_cut));
    index_freq_high_fft = (long)(Wavenum_max_lowband_fft/fabs(Frame_rate / Fringe_speed / N_frames_cut));
    sprintf(vndrs_protocol.text,"# Minimum index in FFT PSD lower band: %ld\n", index_freq_low_fft);
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text,"# Maximum index in FFT PSD lower band: %ld\n", index_freq_high_fft);
    vndrs_runtime_protocol();
    /* added pkervell - 10-Aug-02 (in plane to Hawaii :-)) */
    index_freq_low_fft  = (long)(Wavenum_limits_fft->data[0]/fabs(Frame_rate / Fringe_speed / N_frames_cut));
    index_freq_high_fft = (long)(Wavenum_limits_fft->data[1]/fabs(Frame_rate / Fringe_speed / N_frames_cut));
    sprintf(vndrs_protocol.text,"# Minimum index in FFT PSD for peak integration: %ld\n", index_freq_low_fft);
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text,"# Maximum index in FFT PSD for peak integration: %ld\n", index_freq_high_fft);
    vndrs_runtime_protocol();
    /* printf(" Minimum index in PSD for peak integration: %ld\n", index_freq_low);
    printf(" Maximum index in PSD for peak integration: %ld\n", index_freq_high);*/
    index_freq_low_fft = (long)(Wavenum_min_hi_band_fft/fabs(Frame_rate / Fringe_speed / N_frames_cut));
    index_freq_high_fft = (long)(Wavenum_max_hi_band_fft/fabs(Frame_rate / Fringe_speed / N_frames_cut));
    sprintf(vndrs_protocol.text,"# Minimum index in FFT PSD upper band: %ld\n", index_freq_low_fft);
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text,"# Maximum index in FFT PSD upper band: %ld\n", index_freq_high_fft);
    vndrs_runtime_protocol();
    
    index_freq_low_wl  = (long)(Wavenum_min_lowband_wl/fabs(Frame_rate / Fringe_speed / N_frames_cut));
    index_freq_high_wl = (long)(Wavenum_max_lowband_wl/fabs(Frame_rate / Fringe_speed / N_frames_cut));
    sprintf(vndrs_protocol.text,"# Minimum index in WL PSD lower band: %ld\n", index_freq_low_wl);
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text,"# Maximum index in WL PSD lower band: %ld\n", index_freq_high_wl);
    vndrs_runtime_protocol();
    /* added pkervell - 10-Aug-02 (in plane to Hawaii :-)) */
    index_freq_low_wl  = (long)(Wavenum_limits_wl->data[0]/fabs(Frame_rate / Fringe_speed / N_frames_cut));
    index_freq_high_wl = (long)(Wavenum_limits_wl->data[1]/fabs(Frame_rate / Fringe_speed / N_frames_cut));
    sprintf(vndrs_protocol.text,"# Minimum index in WL PSD for peak integration: %ld\n", index_freq_low_wl);
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text,"# Maximum index in WL PSD for peak integration: %ld\n", index_freq_high_wl);
    vndrs_runtime_protocol();
    /* printf(" Minimum index in PSD for peak integration: %ld\n", index_freq_low);
    printf(" Maximum index in PSD for peak integration: %ld\n", index_freq_high);*/
    index_freq_low_wl = (long)(Wavenum_min_hi_band_wl/fabs(Frame_rate / Fringe_speed / N_frames_cut));
    index_freq_high_wl = (long)(Wavenum_max_hi_band_wl/fabs(Frame_rate / Fringe_speed / N_frames_cut));
    sprintf(vndrs_protocol.text,"# Minimum index in WL PSD upper band: %ld\n", index_freq_low_wl);
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text,"# Maximum index in WL PSD upper band: %ld\n", index_freq_high_wl);
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text,"# Wavenumber interval: %f\n", Wavenum_interval);
    vndrs_runtime_protocol();
}


/*
    Display and saving of the photometric values of the batch.
    pkervell 22-Oct-02
*/

void vndrs_photometry_info(vector_double *Off_offsets_Stddev,
                           vector_double *Off_offsets_Mean,
                           vector_double *A_Mean,
                           vector_double *B_Mean,
                           vector_double *On_source_Mean,
                           double SNR_I1, double SNR_I2,
                           double SNR_PA, double SNR_PB)
{
    printf("  Raw avg OFFSRC background (ADUs): I1 = %7.2f  I2 = %7.2f  PA = %7.2f  PB = %7.2f \n",
     Off_offsets_Mean->data[0],
     Off_offsets_Mean->data[1],
     Off_offsets_Mean->data[2],
     Off_offsets_Mean->data[3]);
    sprintf(vndrs_protocol.text,"# Average OFF-SOURCE offset (ADUs, order: I1, I2, PA, PB):\n  %7.2f  %7.2f  %7.2f  %7.2f\n",
     Off_offsets_Mean->data[0],
     Off_offsets_Mean->data[1],
     Off_offsets_Mean->data[2],
     Off_offsets_Mean->data[3]);
    vndrs_runtime_protocol();

    printf("  Raw avg beam A flux (ADUs):       I1 = %7.2f  I2 = %7.2f  PA = %7.2f  PB = %7.2f \n",
     A_Mean->data[0],
     A_Mean->data[1],
     A_Mean->data[2],
     A_Mean->data[3]);
    sprintf(vndrs_protocol.text,"# Raw avg BEAM A flux (I1, I2, PA, PB):\n  %7.2f  %7.2f  %7.2f  %7.2f\n",
     A_Mean->data[0],
     A_Mean->data[1],
     A_Mean->data[2],
     A_Mean->data[3]);
    vndrs_runtime_protocol();
    
    printf("  Raw avg beam B flux (ADUs):       I1 = %7.2f  I2 = %7.2f  PA = %7.2f  PB = %7.2f \n",
     B_Mean->data[0],
     B_Mean->data[1],
     B_Mean->data[2],
     B_Mean->data[3]);
    sprintf(vndrs_protocol.text,"# Raw avg BEAM B flux (I1, I2, PA, PB):\n  %7.2f  %7.2f  %7.2f  %7.2f\n",
     B_Mean->data[0],
     B_Mean->data[1],
     B_Mean->data[2],
     B_Mean->data[3]);
    vndrs_runtime_protocol();

    printf("  Raw avg On-source flux (ADUs):    I1 = %7.2f  I2 = %7.2f  PA = %7.2f  PB = %7.2f \n",
     On_source_Mean->data[0],
     On_source_Mean->data[1],
     On_source_Mean->data[2],
     On_source_Mean->data[3]);
    sprintf(vndrs_protocol.text,"# Raw avg ON-SOURCE flux (I1, I2, PA, PB):\n  %7.2f  %7.2f  %7.2f  %7.2f\n",
     On_source_Mean->data[0],
     On_source_Mean->data[1],
     On_source_Mean->data[2],
     On_source_Mean->data[3]);
    vndrs_runtime_protocol();

    printf("  Cal avg On-source flux (ADUs):    I1 = %7.2f  I2 = %7.2f  PA = %7.2f  PB = %7.2f \n",
     On_source_Mean->data[0]-Off_offsets_Mean->data[0],
     On_source_Mean->data[1]-Off_offsets_Mean->data[1],
     On_source_Mean->data[2]-Off_offsets_Mean->data[2],
     On_source_Mean->data[3]-Off_offsets_Mean->data[3]);
    sprintf(vndrs_protocol.text,"# Cal avg ON-SOURCE flux (I1, I2, PA, PB):\n  %7.2f  %7.2f  %7.2f  %7.2f\n",
     On_source_Mean->data[0]-Off_offsets_Mean->data[0],
     On_source_Mean->data[1]-Off_offsets_Mean->data[1],
     On_source_Mean->data[2]-Off_offsets_Mean->data[2],
     On_source_Mean->data[3]-Off_offsets_Mean->data[3]);
    vndrs_runtime_protocol();

    printf("  Off-source RMS noise (ADUs):      I1 = %7.2f  I2 = %7.2f  PA = %7.2f  PB = %7.2f \n\n",
     Off_offsets_Stddev->data[0],
     Off_offsets_Stddev->data[1],
     Off_offsets_Stddev->data[2],
     Off_offsets_Stddev->data[3]);
    sprintf(vndrs_protocol.text,"# OFF-SOURCE RMS noise (I1, I2, PA, PB):\n  %7.2f  %7.2f  %7.2f  %7.2f\n",
     Off_offsets_Stddev->data[0],
     Off_offsets_Stddev->data[1],
     Off_offsets_Stddev->data[2],
     Off_offsets_Stddev->data[3]);
    vndrs_runtime_protocol();

    printf("  On-source SNRs :                  I1 = %7.2f  I2 = %7.2f  PA = %7.2f  PB = %7.2f \n", SNR_I1, SNR_I2, SNR_PA, SNR_PB);
    sprintf(vndrs_protocol.text,"# Average ON-SOURCE SNRs (I1, I2, PA, PB):\n  %7.2f  %7.2f  %7.2f  %7.2f\n", SNR_I1, SNR_I2, SNR_PA, SNR_PB);
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text,"# \n");
    vndrs_runtime_protocol();
}


/*
    Verification of the processed data, emission of warnings - pkervell 20-Oct-02
*/

void vndrs_warnings(long N_scans_processed, long N_scans_on, double SNR_PA, double SNR_PB, double avg_photom,
    double mu1_fft_avg_mil, double mu1_fft_avg, double mu1_wl_avg_mil, double mu1_wl_avg,
    double skew_mean_fft, double skew_mean_wl, double weighted_peak_pos, double Wavenum_theo, double average_unbalance,
    vector_double *number_of_pickups, vector_double *off_source_spectrum_slopes, double Frame_rate, double N_frames_precut,
    double slope_background_fft, double slope_background_wl, double N_frames_cut,
    double mu1_fft_med, double mu1_fft_stdev_avg, double mu1_wl_med, double mu1_wl_stdev_avg,
    vector_double *gains, vector_double *RONs, double kappa1A, double kappa1B, double kappa2A, double kappa2B,
    int Autotest_case, vector_double *splitting_ratio_A, vector_double *splitting_ratio_B,
    int ionic_in,
    int *warnings_flag, int *reject_flag) /* out */
{
    *warnings_flag = 0;
    *reject_flag = 0;
    
    /* if(splitting_ratio_A->data[0] > 0.8 || splitting_ratio_A->data[1] > 0.8 ||
       splitting_ratio_A->data[0] < 0.2 || splitting_ratio_A->data[1] < 0.2)
    {
        *warnings_flag = *warnings_flag + 1;
        printf("    * Beam combiner splitting ratios for channel A are defective.\n");
        sprintf(vndrs_protocol.text,"# * Beam combiner splitting ratios for channel A are defective.\n");
        vndrs_runtime_protocol();
    }*/ /* we check only the interferometric channels, as the photometric channels are sometimes inverted.
    Also, they are symmetric to these values. */
    /*if(splitting_ratio_B->data[0] > 0.8 || splitting_ratio_B->data[1] > 0.8 ||
       splitting_ratio_B->data[0] < 0.2 || splitting_ratio_B->data[1] < 0.2)
    {
        *warnings_flag = *warnings_flag + 1;
        printf("    * Beam combiner splitting ratios for channel B are defective.\n");
        sprintf(vndrs_protocol.text,"# * Beam combiner splitting ratios for channel B are defective.\n");
        vndrs_runtime_protocol();
    } removed - pkervell 09-Nov-02 - the mode beam1x1 does not allow a proper estimation of the splitting ratios */
    if(Autotest_case == 1)
    {
        *warnings_flag = *warnings_flag + 1;
        printf("    * Kappa matrix computed using off-source backgrounds. May cause a bias in final Mu^2.\n");
        sprintf(vndrs_protocol.text,"# * Kappa matrix computed using off-source backgrounds. May cause a bias in final Mu^2.\n");
        vndrs_runtime_protocol();
    }
    if(N_scans_processed < 10)
    {
        *reject_flag = *reject_flag + 1;
        printf("  *** Less than 10 interferograms were processed: batch values unreliable.\n");
        sprintf(vndrs_protocol.text,"# *** Less than 10 interferograms were processed: batch values unreliable.\n");
        vndrs_runtime_protocol();
    }
    if((1.0*(N_scans_processed)/N_scans_on) < 0.05)
    {
        *reject_flag = *reject_flag + 1;
        printf("  *** Less than 5 percents of the scans were processed: beware of false fringe detections.\n");
        sprintf(vndrs_protocol.text,"# *** Less than 5 percents of the scans were processed: beware of false fringe detections.\n");
        vndrs_runtime_protocol();
    }
    if(SNR_PA < 1.0)
    {
        *reject_flag = *reject_flag + 1;
        printf("  *** Beam PA avg SNR below 1.0: calibration may be unstable.\n");
        sprintf(vndrs_protocol.text,"# *** Beam PA SNR below 1.0: calibration may be unstable.\n");
        vndrs_runtime_protocol();
    }
    if(SNR_PB < 1.0)
    {
        *reject_flag = *reject_flag + 1;
        printf("  *** Beam PB avg SNR below 1.0: calibration may be unstable.\n");
        sprintf(vndrs_protocol.text,"# *** Beam PB SNR below 1.0: calibration may be unstable.\n");
        vndrs_runtime_protocol();
    }
    if(avg_photom < 5.0)
    {
        *warnings_flag = *warnings_flag + 1;
        printf("    * Average photometry on fringes below 5 ADUs: calibration may be unstable.\n");
        sprintf(vndrs_protocol.text,"# * Average photometry on fringes below 5 ADUs: calibration may be unstable.\n");
        vndrs_runtime_protocol();
    }
    if((1.0*(mu1_fft_avg_mil-mu1_fft_avg)/mu1_fft_avg)>0.03)
    {
        *warnings_flag = *warnings_flag + 1;
        printf("    * %5.2f percents of the Mu^2 fft total power out of the integration band (%4.2f sigmas).\n",
            (100.0*(mu1_fft_avg_mil-mu1_fft_avg)/mu1_fft_avg), (mu1_fft_avg_mil-mu1_fft_avg)/mu1_fft_stdev_avg);
        sprintf(vndrs_protocol.text,"# * %5.2f percents of the Mu^2 fft total power out of the integration band (%4.2f sigmas).\n",
            (100.0*(mu1_fft_avg_mil-mu1_fft_avg)/mu1_fft_avg), (mu1_fft_avg_mil-mu1_fft_avg)/mu1_fft_stdev_avg);
        vndrs_runtime_protocol();
    }
    if((1.0*(mu1_wl_avg_mil-mu1_wl_avg)/mu1_wl_avg)>0.03)
    {
        *warnings_flag = *warnings_flag + 1;
        printf("    * %5.2f percents of the Mu^2 wl total power out of the integration band (%4.2f sigmas).\n",
            (100.0*(mu1_wl_avg_mil-mu1_wl_avg)/mu1_wl_avg), (mu1_wl_avg_mil-mu1_wl_avg)/mu1_wl_stdev_avg);
        sprintf(vndrs_protocol.text,"# * %5.2f percents of the Mu^2 wl power out of the integration band (%4.2f sigmas).\n",
            (100.0*(mu1_wl_avg_mil-mu1_wl_avg)/mu1_wl_avg), (mu1_wl_avg_mil-mu1_wl_avg)/mu1_wl_stdev_avg);
        vndrs_runtime_protocol();
    }
    if(skew_mean_fft > 0.5) /* No test on the median skew, that does not mean much, and that is highly unstable */
    {
        *reject_flag = *reject_flag + 1;
        printf("  *** Skewness coefficient for Fourier based values is above 0.5: average Mu^2 probably biased.\n");
        sprintf(vndrs_protocol.text,"# *** Skewness coefficient for Fourier based values is above 0.5: average Mu^2 probably biased.\n");
        vndrs_runtime_protocol();
    }
    if(skew_mean_wl > 0.5)
    {
        *reject_flag = *reject_flag + 1;
        printf("  *** Skewness coefficient for wavelets based values is above 0.5: average Mu^2 probably biased.\n");
        sprintf(vndrs_protocol.text,"# *** Skewness coefficient for wavelets based values is above 0.5: average Mu^2 probably biased.\n");
        vndrs_runtime_protocol();
    }
    if( fabs(weighted_peak_pos-1.0) > 0.05 )
    {
        *warnings_flag = *warnings_flag + 1;
        printf("    * Position of fft peak is off by more than 5 percents of the assumed central wavenumber (%7.2f cm^-1).\n",
            Wavenum_theo/100.0);
        sprintf(vndrs_protocol.text,"# * Position of fft peak is off by more than 5 percents of the assumed central wavenumber (%7.2f cm^-1).\n",
            Wavenum_theo/100.0);
        vndrs_runtime_protocol();
    } /* No check on wavelets, as the position of the peak depends on the shape of the chosen wave */
    if( average_unbalance < 0.1 )
    {
        *warnings_flag = *warnings_flag + 1;
        printf("    * Strong average unbalance between photometric beams (Blc = %5.2f%%), results may be biased.\n", average_unbalance*100.0);
        sprintf(vndrs_protocol.text,"# * Strong average unbalance between photometric beams (Blc = %5.2fpc), results may be biased.\n",
            average_unbalance*100.0);
        vndrs_runtime_protocol();
    }
/*    if(vndrs_sum_scan(number_of_pickups) > 0) - removed for distribution version 3.0 - pkervell - 02-Nov-03
    {
        *warnings_flag = *warnings_flag + 1;
        printf("    * Pickups or photometric contamination detected in off-source power spectra.\n");
        sprintf(vndrs_protocol.text,"# * Pickups or photometric contamination detected in off-source power spectra.\n");
        vndrs_runtime_protocol();
    }
    if( off_source_spectrum_slopes->data[0]/(Frame_rate/N_frames_precut)>2e-6 ||
        off_source_spectrum_slopes->data[1]/(Frame_rate/N_frames_precut)>2e-6 ||
        off_source_spectrum_slopes->data[2]/(Frame_rate/N_frames_precut)>2e-6 ||
        off_source_spectrum_slopes->data[3]/(Frame_rate/N_frames_precut)>2e-6)
    {
        *warnings_flag = *warnings_flag + 1;
        printf("    * Slope(s) of off-source background PSD larger than usual. Check power spectra.\n");
        sprintf(vndrs_protocol.text,"# * Slope(s) of off-source background PSD larger than usual. Check power spectra.\n");
        vndrs_runtime_protocol();
    }*/
    if( off_source_spectrum_slopes->data[0]/(Frame_rate/N_frames_precut)<-1e-6 ||
        off_source_spectrum_slopes->data[1]/(Frame_rate/N_frames_precut)<-1e-6 ||
        off_source_spectrum_slopes->data[2]/(Frame_rate/N_frames_precut)<-1e-6 ||
        off_source_spectrum_slopes->data[3]/(Frame_rate/N_frames_precut)<-1e-6)
    {
        *warnings_flag = *warnings_flag + 1;
        printf("    * Slope(s) of off-source background PSD are negative. Off-source probably contaminated by star light.\n");
        sprintf(vndrs_protocol.text,"# * Slope(s) of off-source background PSDs are negative. Off-source probably contaminated by star light.\n");
        vndrs_runtime_protocol();
    }
    if( fabs(slope_background_fft/(Frame_rate/N_frames_cut)) > 1e-6)
    {
        *warnings_flag = *warnings_flag + 1;
        printf("    * Slope of fft subtracted background too large. Average in high band subtracted instead.\n");
        sprintf(vndrs_protocol.text,"# * Slope of fft subtracted background too large. Average in high band subtracted instead.\n");
        vndrs_runtime_protocol();
    }
    if( fabs(slope_background_wl/(Frame_rate/N_frames_cut)) > 1e-6)
    {
        *warnings_flag = *warnings_flag + 1;
        printf("    * Slope of wl subtracted background too large. Average in high band subtracted instead.\n");
        sprintf(vndrs_protocol.text,"# * Slope of wl subtracted background too large. Average in high band subtracted instead.\n");
        vndrs_runtime_protocol();
    }
    if( fabs(mu1_fft_avg - mu1_fft_med) > 5.0*mu1_fft_stdev_avg && mu1_fft_stdev_avg>0.0)
    /* We report only if there is more than 5 sigs difference */
    {
        *warnings_flag = *warnings_flag + 1;
        printf("    * Average and median fft values are separated by %4.2f sigmas. Average values possibly biased.\n",
        fabs(mu1_fft_avg - mu1_fft_med)/mu1_fft_stdev_avg);
        sprintf(vndrs_protocol.text,"# * Average and median fft values are separated by %4.2f sigmas. Average values possibly biased.\n",
        fabs(mu1_fft_avg - mu1_fft_med)/mu1_fft_stdev_avg);
        vndrs_runtime_protocol();
    }
    if( fabs(mu1_wl_avg - mu1_wl_med) > 5.0*mu1_wl_stdev_avg && mu1_wl_stdev_avg>0.0)
    {
        *warnings_flag = *warnings_flag + 1;
        printf("    * Average and median wl values are separated by %4.2f sigmas. Average values possibly biased.\n",
        fabs(mu1_wl_avg - mu1_wl_med)/mu1_wl_stdev_avg);
        sprintf(vndrs_protocol.text,"# * Average and median wl values are separated by %4.2f sigmas. Average values possibly biased.\n",
        fabs(mu1_wl_avg - mu1_wl_med)/mu1_wl_stdev_avg);
        vndrs_runtime_protocol();
    }
    
    /* Tests below removed 13-Feb-03 - pkervell: not reliable */
    /* if((gains->data[0] < 3.0 || gains->data[0] > 10.0 ||
    gains->data[1] < 3.0 || gains->data[1] > 10.0 ||
    gains->data[2] < 3.0 || gains->data[2] > 10.0 ||
    gains->data[3] < 3.0 || gains->data[3] > 10.0)
    && avg_photom > 20.0 && average_unbalance > 0.20)
    {
        *warnings_flag = *warnings_flag + 1;
        printf("    * LISA computed gain values are outside specified limits (3 to 10 e-/ADU).\n");
        sprintf(vndrs_protocol.text,"# * LISA computed gain values are outside specified limits (3 to 10 e-/ADU).\n");
        vndrs_runtime_protocol();
    }*/
    /* if((RONs->data[0] < 5.0 || RONs->data[0] > 20.0 ||
    RONs->data[1] < 5.0 || RONs->data[1] > 20.0 ||
    RONs->data[2] < 5.0 || RONs->data[2] > 20.0 ||
    RONs->data[3] < 5.0 || RONs->data[3] > 20.0)
    && avg_photom > 20.0 && average_unbalance > 0.20)
    {
        *warnings_flag = *warnings_flag + 1;
        printf("    * LISA computed readout noise values are outside specified limits (5 to 20 e-).\n");
        sprintf(vndrs_protocol.text,"# * LISA computed readout noise values are outside specified limits (5 to 20 e-).\n");
        vndrs_runtime_protocol();
    }*/
    if((kappa1A < 0.1 | kappa1B < 0.1) || ((kappa2A < 0.1 | kappa2B < 0.1) && (ionic_in == 0)))
    {
        *warnings_flag = *warnings_flag + 1;
        printf("    * Some of the Kappa coefficients are below 10 percents. MONA splitting ratios may be off-limits.\n");
        sprintf(vndrs_protocol.text,"# * Some of the Kappa coefficients are below 10 percents. MONA splitting ratios may be off-limits.\n");
        vndrs_runtime_protocol();
    }
    if(*warnings_flag == 0 && *reject_flag == 0)
    {
        printf(" None.\n");
        sprintf(vndrs_protocol.text,"# None.\n");
        vndrs_runtime_protocol();
    }
    printf("-----------------------------------------------------------------------------------------------------------------------\n"); 
    printf("\n"); 
    
	sprintf(vndrs_protocol.text, "#-----------------------------------------------------------------------------------------------------------------------\n");
	vndrs_runtime_protocol();
}



/* added pkervell 09-Sep-02 to clean the main routine code */

void vndrs_display_and_write_header(void)
{
printf("\n-------------------------------------------------------------------------------------------------------------------------------------\n");
printf(" Scan |  OrigQL |  Pos(t) |  Phase  |   OPD   | FWHM(t)|  Pos(f) |  FWHM(f)| FP | FF | Photom  | Blc  | Weight  |       Mu^2 WL     |\n");
printf("-------------------------------------------------------------------------------------------------------------------------------------\n");
printf("  #   |    mic  |   mic   |   Deg   |   mic   |   mic  |    Hz   |    Hz   |    |    |   SNR   |      |         |     Pow      BG   |\n");
printf("-------------------------------------------------------------------------------------------------------------------------------------\n");
 
 
    sprintf(vndrs_protocol.text, "#---------------------------------------------------------------------------------------------------------------------------------------------------------------\n");
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# Scan Julian_date Altitude_object Azimuth_obj Baseline Azimuth_baseline PeakQL  OrigQL  Pos(t) Phase Total_OPD FWHM(t) Pos(f) FWHM(f) Peak_Strehl FP FF  BG_PA  BG_PB Ph_I1  Ph_I2 Ph_PA Ph_PB Photom Balance Weight Mu2FFT Mu2WL BG_WL\n");
    vndrs_runtime_protocol();

}



/* Writes the data in log file - pkervell - 24-Sep-02 */
void vndrs_write_power_spectra_to_file(long dim_pow,
                                       vector_double *Freq_scale_avg_spectra,
                                       full_scan *Avg_power_spectra_on,
                                       full_scan *Avg_power_spectra_off,
                                       vector_double *WA_mean,
                                       vector_double *WB_mean,
                                       vector_double *Freq_scale_fringes,
                                       vector_double *Pow_spec_avg_I1,
                                       vector_double *Phase_avg_I1,
                                       vector_double *Pow_spec_avg_I1_no_centering,
                                       vector_double *Phase_avg_I1_no_centering,
                                       vector_double *Pow_spec_avg_I1_off,
                                       vector_double *Background_PSD,
                                       vector_double *Pow_spec_avg_I1_corrected,
                                       vector_double *WL_spec_avg_I1_vector_corrected,
                                       vector_double *WL_spec_avg_I1_vector_no_centering,
                                       vector_double *WL_spectrum_vector_off,
                                       vector_double *Background_PSD_wl_vector)
{
    long I;
    vector_double *Pow_spec_avg_I1_smooth;
    long dim_average_pows;
    
    dim_average_pows = (long)(Avg_power_spectra_off->I1->dimension/2);
    
    sprintf(vndrs_protocol.text, "#\n");
    vndrs_runtime_protocol();
    /* Output of the four average off-source spectra (the QL is not output) */
    sprintf(vndrs_protocol.text, "# Average OFF-SOURCE power spectra:\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 1) FREQUENCY scale (Hz)\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 2) I1off\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 3) I2off\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 4) PAoff\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 5) PBoff\n");  
    vndrs_runtime_protocol();
    for(I=0; I<dim_average_pows;I++) /* only half of the power spectrum needs to be saved to log file */
    {
        sprintf(vndrs_protocol.text, "%4.3e ", Freq_scale_avg_spectra->data[I]);
        vndrs_runtime_protocol();
    }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();
    for(I=0; I<dim_average_pows;I++) /* only half of the power spectrum needs to be saved to log file */
    {
        sprintf(vndrs_protocol.text, "%4.3e ", Avg_power_spectra_off->I1->data[I]);
        vndrs_runtime_protocol();
    }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();
    for(I=0; I<dim_average_pows;I++) /* only half of the power spectrum needs to be saved to log file */
    {
        sprintf(vndrs_protocol.text, "%4.3e ", Avg_power_spectra_off->I2->data[I]);
        vndrs_runtime_protocol();
    }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();
    for(I=0; I<dim_average_pows;I++) /* only half of the power spectrum needs to be saved to log file */
    {
        sprintf(vndrs_protocol.text, "%4.3e ", Avg_power_spectra_off->PA->data[I]);
        vndrs_runtime_protocol();
    }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();
    for(I=0; I<dim_average_pows;I++) /* only half of the power spectrum needs to be saved to log file */
    {
        sprintf(vndrs_protocol.text, "%4.3e ", Avg_power_spectra_off->PB->data[I]);
        vndrs_runtime_protocol();
    }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();

    /* -------- */
    sprintf(vndrs_protocol.text, "# Average ON-SOURCE power spectra:\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 1) FREQUENCY scale (Hz)\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 2) I1on\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 3) I2on\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 4) PAon\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 5) PBon\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 6) Quicklook\n");  
    vndrs_runtime_protocol();
    
    for(I=0; I<dim_average_pows;I++) /* only half of the power spectrum needs to be saved to log file */
    {
        sprintf(vndrs_protocol.text, "%4.3e ", Freq_scale_avg_spectra->data[I]);
        vndrs_runtime_protocol();
    }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();    
    for(I=0; I<dim_average_pows;I++) /* only half of the power spectrum needs to be saved to log file */
    {
        sprintf(vndrs_protocol.text, "%4.3e ", Avg_power_spectra_on->I1->data[I]);
        vndrs_runtime_protocol();
    }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();
    for(I=0; I<dim_average_pows;I++) /* only half of the power spectrum needs to be saved to log file */
    {
        sprintf(vndrs_protocol.text, "%4.3e ", Avg_power_spectra_on->I2->data[I]);
        vndrs_runtime_protocol();
    }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();
    for(I=0; I<dim_average_pows;I++) /* only half of the power spectrum needs to be saved to log file */
    {
        sprintf(vndrs_protocol.text, "%4.3e ", Avg_power_spectra_on->PA->data[I]);
        vndrs_runtime_protocol();
    }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();
    for(I=0; I<dim_average_pows;I++) /* only half of the power spectrum needs to be saved to log file */
    {
        sprintf(vndrs_protocol.text, "%4.3e ", Avg_power_spectra_on->PB->data[I]);
        vndrs_runtime_protocol();
    }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();
    for(I=0; I<dim_average_pows;I++) /* only half of the power spectrum needs to be saved to log file */
    {
        sprintf(vndrs_protocol.text, "%4.3e ", Avg_power_spectra_on->QL->data[I]);
        vndrs_runtime_protocol();
    }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();


    /* -------- */
    
    /* Output of the Wiener filters WA_mean and WB_mean in the log file */
    sprintf(vndrs_protocol.text, "# Average Wiener filters for PA and PB:\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 1) Wavenumber scale (in cm^-1)\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 2) Wiener filter for beam A\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 3) Wiener filter for beam B\n");  
    vndrs_runtime_protocol();
    for(I=0; I<dim_pow;I++)
    {
        sprintf(vndrs_protocol.text, "%4.3e ", Freq_scale_fringes->data[I]);
        vndrs_runtime_protocol();
    }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();
    for(I=0; I<dim_pow;I++) /* only half of the filter needs to be saved to log file */
    {
        sprintf(vndrs_protocol.text, "%4.3e ", WA_mean->data[I]);
        vndrs_runtime_protocol();
    }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();
    for(I=0; I<dim_pow;I++) /* only half of the filter needs to be saved to log file */
    {
        sprintf(vndrs_protocol.text, "%4.3e ", WB_mean->data[I]);
        vndrs_runtime_protocol();
    }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();


    /* -------- */
            

    sprintf(vndrs_protocol.text, "# Fourier power spectra of processed scans:\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 1) WAVENUMBER scale (in cm^-1)\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 2) Average Fourier PSD of I1 - I2, fringe peak recentered (with residual photon noise):\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 3) Average non-recentered Fourier PSD of calibrated I1 - I2 (with photon noise):\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 4) Average Fourier power spectrum of removed noise from off-source\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 5) Model Fourier power spectrum of removed background\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 6) Average Fourier power spectrum of calibrated and BG corrected I1 - I2\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 7) Average smoothed Fourier power spectrum of calibrated and BG corrected I1 - I2\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 8) Average phase of recentered I1 - I2:\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 9) Average phase of non-recentered I1 - I2:\n");  
    vndrs_runtime_protocol();
    for(I=0; I<dim_pow;I++)
    {
        sprintf(vndrs_protocol.text, "%4.3e ", Freq_scale_fringes->data[I]);
        vndrs_runtime_protocol();
    }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();
    for(I=0; I<dim_pow;I++)
        {
        sprintf(vndrs_protocol.text, "%4.3e ", Pow_spec_avg_I1->data[I]);
        vndrs_runtime_protocol();
        }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();

    for(I=0; I<dim_pow;I++)
        {
        sprintf(vndrs_protocol.text, "%4.3e ", Pow_spec_avg_I1_no_centering->data[I]);
        vndrs_runtime_protocol();
        }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();

    for(I=0; I<dim_pow;I++)
        {
        sprintf(vndrs_protocol.text, "%4.3e ", Pow_spec_avg_I1_off->data[I]);
        vndrs_runtime_protocol();
        }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();
    
    for(I=0; I<dim_pow;I++)
        {
        sprintf(vndrs_protocol.text, "%4.3e ", Background_PSD->data[I]);
        vndrs_runtime_protocol();
        }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();

    for(I=0; I<dim_pow;I++)
        {
        sprintf(vndrs_protocol.text, "%4.3e ", Pow_spec_avg_I1_corrected->data[I]);
        vndrs_runtime_protocol();
        }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();

        
    Pow_spec_avg_I1_smooth = vndrs_vector_smooth(Pow_spec_avg_I1_corrected, 5);
    for(I=0; I<dim_pow;I++)
        {
        sprintf(vndrs_protocol.text, "%4.3e ", Pow_spec_avg_I1_smooth->data[I]);
        vndrs_runtime_protocol();
        }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();
    vndrs_free_vector_double(Pow_spec_avg_I1_smooth);

    for(I=0; I<dim_pow;I++)
        {
        sprintf(vndrs_protocol.text, "%4.3e ", Phase_avg_I1->data[I]);
        vndrs_runtime_protocol();
        }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();

    for(I=0; I<dim_pow;I++)
        {
        sprintf(vndrs_protocol.text, "%4.3e ", Phase_avg_I1_no_centering->data[I]);
        vndrs_runtime_protocol();
        }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();
    

    /* -------- */
    sprintf(vndrs_protocol.text, "# Wavelets power spectra of processed scans:\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 1) WAVENUMBER scale (in cm^-1)\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 2) Average Wavelets power spectrum of fully corrected I1 - I2\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 3) Average Wavelets power spectrum of non recentered fringes wavelets PSD I1 - I2\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 4) Average Wavelets power spectrum of removed noise from off-source\n");  
    vndrs_runtime_protocol();
    sprintf(vndrs_protocol.text, "# 5) Average Wavelets power spectrum of I1 - I2 model background\n");  
    vndrs_runtime_protocol();
    
    for(I=0; I<dim_pow;I++)
    {
        sprintf(vndrs_protocol.text, "%4.3e ", Freq_scale_fringes->data[I]);
        vndrs_runtime_protocol();
    }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();
    
    for(I=0; I<dim_pow;I++)
    {
        sprintf(vndrs_protocol.text, "%4.3e ", WL_spec_avg_I1_vector_corrected->data[I]);
        vndrs_runtime_protocol();
    }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();

    for(I=0; I<dim_pow;I++)
    {
        sprintf(vndrs_protocol.text, "%4.3e ", WL_spec_avg_I1_vector_no_centering->data[I]);
        vndrs_runtime_protocol();
    }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();
    
    for(I=0; I<dim_pow;I++)
    {
        sprintf(vndrs_protocol.text, "%4.3e ", WL_spectrum_vector_off->data[I]);
        vndrs_runtime_protocol();
    }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();
    for(I=0; I<dim_pow;I++)
    {
        sprintf(vndrs_protocol.text, "%4.3e ", Background_PSD_wl_vector->data[I]);
        vndrs_runtime_protocol();
    }
    sprintf(vndrs_protocol.text, "\n");
    vndrs_runtime_protocol();

}




