/* $Id: vndrs_FITS.h,v 1.1 2001/07/24 09:42:14 vltip Exp $ */
#ifndef _VNDRS_FITS_H
#define _VNDRS_FITS_H
/*----------------------------------------------------------------------- */
/* VINCI FITS binary tables routines                                      */
/* W. D. Cotton  March, 2001                                              */ 
/*----------------------------------------------------------------------- */
#include "vndrs.h"
#include "vndrs_OPD.h"
#include "VLTItable.h"
#include "fringe_a.h"
#include "batch.h"
#include "source_table.h"
#include "array_geometry.h"
#include "optical_train.h"
#include "vndrs_OPD.h"
/* public functions */

int vndrs_make_data_file (char *infile,
			  char *outfile,
			  fitsfile **infptr, 
			  fitsfile **outfptr, 
			  int *ierr,
			  double shape,
			  double threshold,
			  double apod,
			  double blockout
			  );

/*----------------------------------------------------------------------- */
/*   Create/init FITS file for VINCI processed data                       */
/*   If the output file doesn't exist it is created and initialized.      */
/*   Input file is assumed an ESO VLTI/VINCI file.                        */
/*   Inputs:                                                              */
/*      infile    input (on-source) file name                             */
/*      outfile   output file name                                        */
/*   Output:                                                              */
/*      infptr    cfitsio pointer to open input FITS file                 */
/*      outfptr   cfitsio pointer to open output FITS file                */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      0=> file previously existed, 1=> was just created                 */
/*----------------------------------------------------------------------- */

fringe_a_table_def* vndrs_make_data_table (char *infile,
					   char *outfile,
					   fitsfile **outfptr, 
					   int *tel1, int *tel2, 
					   int *source, int *batch_id,
					   int *ierr,
					   double shape,
					   double threshold,
					   double apod,
					   double blockout
					   );
/*----------------------------------------------------------------------- */
/*   Create/init FITS fringe_a table                                      */
/*   If the output file doesn't exist it is created and initialized with  */
/*   a source and array geometry table and the fringe_a table created     */
/*   Input file is assumed an ESO VINCI archive file                      */
/*   Inputs:                                                              */
/*      infile    input (on-source) file name                             */
/*      outfile   output file name                                        */
/*   Output:                                                              */
/*      outfptr   cfitsio pointer to open output FITS file                */
/*      tel1,2    telescope numbers                                       */
/*      source    source number                                           */
/*      batch_id  batch number in the batch table                         */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*       fringe_a_table_def structure to use to write table               */
/*       NULL on failure                                                  */
/*----------------------------------------------------------------------- */

void vndrs_write_data_table (fitsfile *outfptr, fringe_a_table_def *datadef, 
			     batch *Batch_on, opd_data *opd,
			     int tel1, int tel2, int source, int batch_id,
			     char *file_on, char *file_off, 
			     char *file_a, char *file_b, 
                             matrix_double *Matrix, double shape,
			     int *ierr);
/*----------------------------------------------------------------------- */
/*   Write data to FITS fringe_a table                                    */
/*   Inputs:                                                              */
/*      outfptr   cfitsio pointer to open output FITS file                */
/*      datadef   output table definition structure                       */
/*      batch_on  processed on source batch,  scan_data member is used    */
/*      opd       batch and scan values of OPD and rate                   */
/*      tel1,2    telescope numbers                                       */
/*      source    source number                                           */
/*      batch_id  batch number in the batch table                         */
/*      file_on   on source raw FITS file name                            */
/*      file_off  off source raw FITS file name                           */
/*      file_a    Port A only raw FITS file name                          */
/*      file_b    Port B only raw FITS file name                          */
/*      Matrix    Kappa matrix.                                           */
/*      shape     shape factor (e.g. 0.000013)                            */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*----------------------------------------------------------------------- */

void vndrs_vlti_geometry(fitsfile *infptr, fitsfile *outfptr, 
			 char *date_obs, int mount, 
			 int *tel1, int *tel2, int *ierr);
/*----------------------------------------------------------------------- */
/*   Save Array Geometry for all of VLTI array if it doesn't exist        */
/*   Copies data from input file.                                         */
/*   If an ARRAY_GEOMETRY table exists nothing is done                    */
/*   Returns telescope numbers used                                       */
/*   Inputs:                                                              */
/*      infptr    cfitsio pointer to open FITS file for header info       */
/*      outfptr   cfitsio pointer to open FITS file for table             */
/*      date_obs  Date (yyyy-mm-dd) observations began                    */
/*      mount     mount type 0 = alt-az, 3=siderostat                     */
/*   Output:                                                              */
/*      tel1,2    telescope numbers                                       */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Return:                                                              */
/*      Source number for the source                                      */
/*----------------------------------------------------------------------- */

void vndrs_vlti_optical_train(fitsfile *infptr, fitsfile *outfptr, 
			      int tel1, int tel2, int *ierr);
/*----------------------------------------------------------------------- */
/*   Save Optical train for current VLTI configuration if it doesn't exist*/
/*   Copies data from input file.                                         */
/*   If an OPTICAL_TRAN table exists nothing is done.                     */
/*   Inputs:                                                              */
/*      infptr    cfitsio pointer to open FITS file for header info       */
/*      outfptr   cfitsio pointer to open FITS file for table             */
/*      tel1,2    telescope numbers                                       */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Return:                                                              */
/*      Source number for the source                                      */
/*----------------------------------------------------------------------- */

int vndrs_vlti_source(fitsfile *infptr, fitsfile *outfptr, int *ierr);
/*----------------------------------------------------------------------- */
/*   Save source info in Source table.                                    */
/*   Copies data from input file.                                         */
/*   Inputs:                                                              */
/*      infptr      cfitsio pointer to open FITS file for header info     */
/*      outfptr     cfitsio pointer to open FITS file for source table    */
/*                  if null on entry open file                            */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Return:                                                              */
/*      Source number for the source                                      */
/*----------------------------------------------------------------------- */

int vndrs_vlti_batch(fitsfile *infptr, fitsfile *outfptr, int *ierr);
/*----------------------------------------------------------------------- */
/*   Save batch info in Batch table.                                      */
/*   Copies data from input file.                                         */
/*   Inputs:                                                              */
/*      infptr      cfitsio pointer to open FITS file for header info     */
/*      outfptr     cfitsio pointer to open FITS file for batch table     */
/*                  if null on entry open file                            */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Return:                                                              */
/*      Batch number                                                      */
/*----------------------------------------------------------------------- */

#endif /* end VNDRS_FITS_H */
