/*******************************************************************/
/* VLTI VINCI DATA REDUCTION SOFTWARE (VNDRS)                      */
/*                                                                 */
/* FILE: vndrs_IO.c                                               */
/*                                                                 */
/* Contains the routines for IO procedures                         */
/*                                                                 */
/* Written by Pascal Ballester, Markus Schoeller & Rainer Wilhelm  */
/* ESO, 2000-2001                                                  */
/*******************************************************************/

#include "vndrs.h"
#include "vndrs_Globalvar.h"
#include <stdio.h>
#include <ctype.h>	/* csabet 09-Aug-2001. To avoid implicit declaration of function `toupper' */ 

/* Configuration of the Export Format keyword list */

        /* Do not edit before this line */ 
  int    export_mode = 1; /* behavior on missing key: mode=-1: aborts, mode=0: warns, mode=1: continue */ 
  int    export_dbx  = 0;  /* no debug:dbx=0, debug mode: dbx=1 */

  int    export_int_default = 0;                                /* The integer default value */
  double export_dbl_default = 0.;                               /* The double precision default value */ 
  char   export_log_default = 'F';                             /* The logical default value */ 
  char   export_str_default[MAX_STRING_LENGTH] = {"UNKNOWN"};  /* The char. string default value */ 

  char*  export_keylist[MAX_STRING_LENGTH] = {

    /* Each element of the list has the syntax <type>:<fits> with 
       <type> a one character value type (D=double precision, I=integer, 
       C=character string, L=logical, #=comment line), and
       <fits> is the name of the FITS keyword, or for comments a free string 
       Examples: 
        "D:HIERARCH ESO DET QL FRINGESPD",
        "I:HIERARCH ESO DET QL OBSMODE",
        "C:TELESCOP",
	"L:HIERARCH ESO INS LAMP1 ST",
        "#:This is a comment line",
    */ 



    /* First the list of disabled keywords not yet in the FITS header */ 
       /* 
	*/

    /* Second the list of enabled keywords already present in the FITS header */ 

        "C:TELESCOP",
        "C:INSTRUME",
        "D:RA",
        "D:DEC",
        "D:EQUINOX",
        "C:RADECSYS",
        "D:MJD-OBS",

        "C:ESO OBS NAME",
        "C:ESO TPL ID",
        "C:ESO TPL START",

        "C:ESO DPR CATG",
        "C:ESO DPR TECH",

        "C:ESO INS MODE",
        "C:ESO INS FILT1 NAME",
	/* add all hierachical keywords as of 14 Mar 01 - W. Cotton */
        "C:HIERARCH ESO COU GUID STATUS",
        "D:HIERARCH ESO DEL DLT1 OPL END",
        "D:HIERARCH ESO DEL DLT1 OPL START",
        "C:HIERARCH ESO DEL FT STATUS",
        "D:HIERARCH ESO DEL OPD OFFSET",
        "C:HIERARCH ESO DEL REF NAME",
	"D:HIERARCH ESO DEL REF OPL",
	"C:HIERARCH ESO DET CHIP ID",
	"C:HIERARCH ESO DET CHIP NAME",
	"I:HIERARCH ESO DET CHIP NX",
	"I:HIERARCH ESO DET CHIP NY",
	"I:HIERARCH ESO DET CHOP FREQ",
	"C:HIERARCH ESO DET CON OPMODE",
	"I:HIERARCH ESO DET DACQ MODE",
	"C:HIERARCH ESO DET DID",
	"D:HIERARCH ESO DET DIT",
	"D:HIERARCH ESO DET DITDELAY",
	"I:HIERARCH ESO DET IRACE ADC1 DELAY",
	"I:HIERARCH ESO DET IRACE ADC1 ENABLE",
	"I:HIERARCH ESO DET IRACE ADC1 FILTER1",
	"I:HIERARCH ESO DET IRACE ADC1 FILTER2",
	"I:HIERARCH ESO DET IRACE ADC1 HEADER",
	"C:HIERARCH ESO DET IRACE ADC1 NAME",
	"C:HIERARCH ESO DET IRACE SEQCONT",
	"I:HIERARCH ESO DET NC BATCH",
	"I:HIERARCH ESO DET NC NSCAN",
	"I:HIERARCH ESO DET NCORRS",
	"C:HIERARCH ESO DET NCORRS NAME",
	"I:HIERARCH ESO DET NDIT",
	"I:HIERARCH ESO DET NDITSKIP",
	"I:HIERARCH ESO DET QL BEAMI1",
	"I:HIERARCH ESO DET QL BEAMI2",
	"I:HIERARCH ESO DET QL DBLCHK",
	"I:HIERARCH ESO DET QL DETLEVEL",
	"D:HIERARCH ESO DET QL FRAMERATE",
	"D:HIERARCH ESO DET QL FRINGESPD",
	"D:HIERARCH ESO DET QL GAINHW",
	"I:HIERARCH ESO DET QL INSMODE",
	"D:HIERARCH ESO DET QL MAXWAVEK",
	"D:HIERARCH ESO DET QL MAXWAVEOF",
	"D:HIERARCH ESO DET QL MINWAVEK",
	"D:HIERARCH ESO DET QL MINWAVEOFF",
	"I:HIERARCH ESO DET QL OBSMODE",
	"I:HIERARCH ESO DET QL OPDST",
	"I:HIERARCH ESO DET QL ST",
	"I:HIERARCH ESO DET RM NFRAM",
	"I:HIERARCH ESO DET RSPEED",
	"I:HIERARCH ESO DET WIN NX",
	"I:HIERARCH ESO DET WIN NY",
	"I:HIERARCH ESO DET WIN STARTX",
	"I:HIERARCH ESO DET WIN STARTY",
	"I:HIERARCH ESO DET WIN TYPE",
	"I:HIERARCH ESO DET WIN1 NX",
	"I:HIERARCH ESO DET WIN1 NY",
	"I:HIERARCH ESO DET WIN1 STRX",
	"I:HIERARCH ESO DET WIN1 STRY",
	"I:HIERARCH ESO DET WIN2 NX",
	"I:HIERARCH ESO DET WIN2 NY",
	"I:HIERARCH ESO DET WIN2 STRX",
	"I:HIERARCH ESO DET WIN2 STRY",
	"I:HIERARCH ESO DET WIN3 NX",
	"I:HIERARCH ESO DET WIN3 NY",
	"I:HIERARCH ESO DET WIN3 STRX",
	"I:HIERARCH ESO DET WIN3 STRY",
	"I:HIERARCH ESO DET WIN4 NX",
	"I:HIERARCH ESO DET WIN4 NY",
	"I:HIERARCH ESO DET WIN4 STRX",
	"I:HIERARCH ESO DET WIN4 STRY",
	"C:HIERARCH ESO DPR CATG",
	"C:HIERARCH ESO DPR TECH",
	"C:HIERARCH ESO INS DID",
	"C:HIERARCH ESO INS FILT1 ID",
	"C:HIERARCH ESO INS FILT1 NAME",
	"I:HIERARCH ESO INS FILT1 NO",
	"D:HIERARCH ESO INS FOCU1 POS",
	"D:HIERARCH ESO INS FOCU2 POS",
	"D:HIERARCH ESO INS FOCU3 POS",
	"D:HIERARCH ESO INS FOCU4 POS",
	"C:HIERARCH ESO INS ID",
	"C:HIERARCH ESO INS LAMP1 NAME",
	"L:HIERARCH ESO INS LAMP1 ST",
	"C:HIERARCH ESO INS LAMP1 ID",
	"C:HIERARCH ESO INS LAMP2 NAME",
	"L:HIERARCH ESO INS LAMP2 ST",
	"C:HIERARCH ESO INS LAMP2 ID",
	"C:HIERARCH ESO INS LAMP3 NAME",
	"L:HIERARCH ESO INS LAMP3 ST",
	"C:HIERARCH ESO INS LAMP3 ID",
	"C:HIERARCH ESO INS MIRR1 ID",
	"C:HIERARCH ESO INS MIRR1 NAME",
	"I:HIERARCH ESO INS MIRR1 NSCAN",
	"D:HIERARCH ESO INS MIRR1 RANGE",
	"D:HIERARCH ESO INS MIRR1 SPEED",
	"L:HIERARCH ESO INS MIRR1 SWSIM",
	"D:HIERARCH ESO INS MIRR1 UTC1",
	"D:HIERARCH ESO INS MIRR1 UTC2",
	"C:HIERARCH ESO INS MIRR2 ID",
	"C:HIERARCH ESO INS MIRR2 NAME",
	"I:HIERARCH ESO INS MIRR2 NSCAN",
	"D:HIERARCH ESO INS MIRR2 RANGE",
	"D:HIERARCH ESO INS MIRR2 SPEED",
	"L:HIERARCH ESO INS MIRR2 SWSIM",
	"D:HIERARCH ESO INS MIRR2 UTC1",
	"D:HIERARCH ESO INS MIRR2 UTC2",
	"C:HIERARCH ESO INS MODE",
	"D:HIERARCH ESO INS OPTI1 POS",
	"C:HIERARCH ESO INS OPTI2 ID",
	"C:HIERARCH ESO INS OPTI2 NAME",
	"I:HIERARCH ESO INS OPTI2 NO",
	"C:HIERARCH ESO INS OPTI2 ID",
	"C:HIERARCH ESO INS OPTI3 NAME",
	"I:HIERARCH ESO INS OPTI3 NO",
	"C:HIERARCH ESO INS OPTI3 ID",
	"C:HIERARCH ESO INS OPTI4 NAME",
	"I:HIERARCH ESO INS OPTI4 NO",
	"C:HIERARCH ESO INS OPTI4 ID",
	"C:HIERARCH ESO INS OPTI5 NAME",
	"I:HIERARCH ESO INS OPTI5 NO",
	"C:HIERARCH ESO INS OPTI5 ID",
	"C:HIERARCH ESO INS OPTI6 NAME",
	"I:HIERARCH ESO INS OPTI6 NO",
	"C:HIERARCH ESO INS OPTI6 ID",
	"C:HIERARCH ESO INS PATH",
	"D:HIERARCH ESO INS ROT1 POS",
	"D:HIERARCH ESO INS ROT2 POS",
	"D:HIERARCH ESO INS ROT2 SWSIM",
	"D:HIERARCH ESO INS ROT3 POS",
	"L:HIERARCH ESO INS ROT3 SWSIM",
	"C:HIERARCH ESO INS SENS1 ID",
	"C:HIERARCH ESO INS SENS1 NAME",
	"C:HIERARCH ESO INS SENS1 STAT",
	"C:HIERARCH ESO INS SENS2 ID",
	"C:HIERARCH ESO INS SENS2 NAME",
	"C:HIERARCH ESO INS SENS2 STAT",
	"C:HIERARCH ESO INS SENS3 ID",
	"C:HIERARCH ESO INS SENS3 NAME",
	"C:HIERARCH ESO INS SENS3 STAT",
	"C:HIERARCH ESO INS SENS4 ID",
	"C:HIERARCH ESO INS SENS4 NAME",
	"C:HIERARCH ESO INS SENS4 STAT",
	"C:HIERARCH ESO INS SENS5 ID",
	"C:HIERARCH ESO INS SENS5 NAME",
	"C:HIERARCH ESO INS SENS5 STAT",
	"C:HIERARCH ESO INS SENS6 ID",
	"C:HIERARCH ESO INS SENS6 NAME",
	"C:HIERARCH ESO INS SENS6 STAT",
	"C:HIERARCH ESO INS SENS7 ID",
	"C:HIERARCH ESO INS SENS7 NAME",
	"C:HIERARCH ESO INS SENS7 STAT",
	"C:HIERARCH ESO INS SENS8 ID",
	"C:HIERARCH ESO INS SENS8 NAME",
	"C:HIERARCH ESO INS SENS8 STAT",
	"C:HIERARCH ESO INS SENS9 ID",
	"C:HIERARCH ESO INS SENS9 NAME",
	"C:HIERARCH ESO INS SENS9 STAT",
	"C:HIERARCH ESO INS SENS10 ID",
	"C:HIERARCH ESO INS SENS10 NAME",
	"C:HIERARCH ESO INS SENS10 STAT",
	"C:HIERARCH ESO INS SENS11 ID",
	"C:HIERARCH ESO INS SENS11 NAME",
	"C:HIERARCH ESO INS SENS11 STAT",
	"C:HIERARCH ESO INS SENS12 ID",
	"C:HIERARCH ESO INS SENS12 NAME",
	"C:HIERARCH ESO INS SENS12 STAT",
	"C:HIERARCH ESO INS SENS13 ID",
	"C:HIERARCH ESO INS SENS13 NAME",
	"C:HIERARCH ESO INS SENS13 STAT",
	"C:HIERARCH ESO INS SENS14 ID",
	"C:HIERARCH ESO INS SENS14 NAME",
	"C:HIERARCH ESO INS SENS14 STAT",
	"C:HIERARCH ESO INS SENS15 ID",
	"C:HIERARCH ESO INS SENS15 NAME",
	"C:HIERARCH ESO INS SENS15 STAT",
	"C:HIERARCH ESO INS SHUT1 ID",
	"C:HIERARCH ESO INS SHUT1 NAME",
	"L:HIERARCH ESO INS SHUT1 ST",
	"C:HIERARCH ESO INS SWSIM",
	"D:HIERARCH ESO INS TILT1 POS",
	"D:HIERARCH ESO INS TILT2 POS",
	"D:HIERARCH ESO INS TILT3 POS",
	"D:HIERARCH ESO INS TIP1 POS",
	"D:HIERARCH ESO INS TIP2 POS",
	"D:HIERARCH ESO INS TIP3 POS",
        /* Do not edit after this line */ 
        "END" /* The list ALWAYS ends with this line */
  };

/* must break this sucker into two halves - gcc too feeble */
  char*  export_keylist2[MAX_STRING_LENGTH] = {
	"D:HIERARCH ESO ISS AIRM END",
	"D:HIERARCH ESO ISS AIRM STAR",
	"D:HIERARCH ESO ISS ALT",
	"D:HIERARCH ESO ISS AMBI FWHM END",
	"D:HIERARCH ESO ISS AMBI FWHM START",
	"D:HIERARCH ESO ISS ALT",
	"D:HIERARCH ESO ISS AMBI FWHM END",
	"D:HIERARCH ESO ISS AMBI RHUM",
	"D:HIERARCH ESO ISS AMBI TEMP",
	"D:HIERARCH ESO ISS AMBI WINDDIR",
	"D:HIERARCH ESO ISS AMBI WINDSP",
	"D:HIERARCH ESO ISS AZ",
	"D:HIERARCH ESO ISS CHOP FREQ",
	"C:HIERARCH ESO ISS CHOP NULLING",
	"D:HIERARCH ESO ISS CHOP POSANG",
	"D:HIERARCH ESO ISS CHOP PVRATIO",
	"C:HIERARCH ESO ISS CHOP ST",
	"D:HIERARCH ESO ISS CHOP THROW",
	"C:HIERARCH ESO ISS CONF DL1",
	"C:HIERARCH ESO ISS CONF DL2",
	"C:HIERARCH ESO ISS CONF DL3",
	"I:HIERARCH ESO ISS CONF INPUT1",
	"C:HIERARCH ESO ISS CONF INPUT12",
	"I:HIERARCH ESO ISS CONF INPUT2",
	"C:HIERARCH ESO ISS CONF INPUT22",
	"I:HIERARCH ESO ISS CONF INPUT",
	"C:HIERARCH ESO ISS CONF INPUT32",
	"I:HIERARCH ESO ISS CONF NTEL",
	"C:HIERARCH ESO ISS CONF STATION1",
	"C:HIERARCH ESO ISS CONF STATION2",
	"C:HIERARCH ESO ISS CONF STATION3",
	"C:HIERARCH ESO ISS CONF SWYARD",
	"C:HIERARCH ESO ISS CONF SWYARD1",
	"C:HIERARCH ESO ISS CONF SWYARD2",
	"C:HIERARCH ESO ISS CONF SWYARD22",
	"C:HIERARCH ESO ISS CONF SWYARD32",
	"C:HIERARCH ESO ISS CONF T1NAME",
	"D:HIERARCH ESO ISS CONF T1X",
	"D:HIERARCH ESO ISS CONF T1Y",
	"D:HIERARCH ESO ISS CONF T1Z",
	"C:HIERARCH ESO ISS CONF T2NAME",
	"D:HIERARCH ESO ISS CONF T2X",
	"D:HIERARCH ESO ISS CONF T2Y",
	"D:HIERARCH ESO ISS CONF T2Z",
	"C:HIERARCH ESO ISS CONF T3NAME",
	"D:HIERARCH ESO ISS CONF T3X",
	"D:HIERARCH ESO ISS CONF T3Y",
	"D:HIERARCH ESO ISS CONF T3Z",
	"C:HIERARCH ESO ISS DATE",
	"C:HIERARCH ESO ISS DID",
	"D:HIERARCH ESO ISS GEOELEV",
	"D:HIERARCH ESO ISS GEOLAT",
	"D:HIERARCH ESO ISS GEOLON",
	"C:HIERARCH ESO ISS ID",
	"C:HIERARCH ESO ISS OPER",
	"D:HIERARCH ESO ISS PARANG END",
	"D:HIERARCH ESO ISS PARANG START",
	"D:HIERARCH ESO ISS PUPIL 1U",
	"D:HIERARCH ESO ISS PUPIL 1V",
	"D:HIERARCH ESO ISS PUPIL 1W",
	"D:HIERARCH ESO ISS PUPIL 2U",
	"D:HIERARCH ESO ISS PUPIL 2V",
	"D:HIERARCH ESO ISS PUPIL 2W",
	"D:HIERARCH ESO ISS PUPIL 3U",
	"D:HIERARCH ESO ISS PUPIL 3V",
	"D:HIERARCH ESO ISS PUPIL 3W",
	"D:HIERARCH ESO ISS TEMP LAB1",
	"D:HIERARCH ESO ISS TEMP LAB2",
	"D:HIERARCH ESO ISS TEMP LAB3",
	"D:HIERARCH ESO ISS TEMP LAB4",
	"D:HIERARCH ESO ISS TEMP TUN1",
	"D:HIERARCH ESO ISS TEMP TUN2",
	"D:HIERARCH ESO ISS TEMP TUN3",
	"D:HIERARCH ESO ISS TEMP TUN4",
	"C:HIERARCH ESO ISS TRAK STATUS",
	"C:HIERARCH ESO OBS DID",
	"C:HIERARCH ESO OBS GRP",
	"C:HIERARCH ESO OBS ID",
	"C:HIERARCH ESO OBS NAME",
	"C:HIERARCH ESO OBS OBSERVER",
	"I:HIERARCH ESO OBS PI-COI ID",
	"C:HIERARCH ESO OBS PI-COI NAME",
	"C:HIERARCH ESO OBS PROG ID",
	"C:HIERARCH ESO OBS START",
	"I:HIERARCH ESO OBS TPLNO",
	"C:HIERARCH ESO OCS DET1 IMGNAME",
	"C:HIERARCH ESO OCS PATH",
	"C:HIERARCH ESO TPL DID",
	"C:HIERARCH ESO TPL EXPNO",
	"C:HIERARCH ESO TPL ID",
	"C:HIERARCH ESO TPL NAME",
	"I:HIERARCH ESO TPL NEXP",
	"C:HIERARCH ESO TPL PRESEQ",
	"C:HIERARCH ESO TPL START",
	"C:HIERARCH ESO TPL VERSION",
	"C:ORIGFILE",
	"C:ARCFILE",

        "#: The data file contain the raw data and processed data with the same format:",
        "#: <obsmode> <scan_nb> <sigmode> <time> <dl_off> <mu1> <mu2> <value(1..NFRAMES)>", 
        "#: ",
        "#: 1. <obsmode> is the value of ESO DET QL OBSMODE (1:OffSource, 2:BeamA, 3:BeamB, 4:OnSource)",
        "#: 2. <scan_nb> is the scan number, running from 1 to the number of scans for the current OBSMODE batch",
        "#: 3. <sigmode> is the type of signal (1:I1, 2:I2, 3:PA, 4:PB, 5:I1_Corrected, 6:I2_Corrected", 
        "#: 4. <time>    is the time stamp of starting the scan, read from SCAN_DATA table", 
        "#: 5. <dl_off>  is the delay line offset for the scan, read from the SCAN_DATA table", 
        "#: 6. <mu1>     is the squared coherence factor derived from I1_Corrected", 
        "#: 7. <mu2>     is the squared coherence factor derived from I2_Corrected", 
        "#: 8. <value(1..NFRAMES)> is a list of NFRAMES='ESO DET RM NFRAM' values with each measurement", 
        "#: ",
        "#: The data file export_data.txt contains the raw data with the following default values",
        "#: a. All 4 values of OBSMODE are represented", 
        "#: b. Only the signals I1,I2,PA,PB (SIGMODE=1..4) are used", 
        "#: c. <mu1> and <mu2> are set to the value -99.00", 
        "#: ",
        "#: The data file export_proc.txt contains the photometry corrected data with the following default",
        "#: a. Only the corrected OnSource signals are written to file (OBSMODE=4)", 
        "#: b. Only the corrected I1_Corrected and I2_Corrected are written to file (SIGMODE=5..6)", 
        "#: c. <mu1> and <mu2> are set to the values resulting from the fringe processing", 
 
        /* Do not edit after this line */ 
        "END" /* The list ALWAYS ends with this line */
   };
  /* End of configuration of the keyword list */ 


/* Routine to dump double vectors in ASCII format into a file */

void vndrs_generate_full_filename(const char *filename,
                                  char *Full_Filename
                                 )
{
    strcpy(Full_Filename, DataPath);
    strcat(Full_Filename, "/");
    strcat(Full_Filename, filename);
    return;
}

void vndrs_fprintf_vector_double(
                                 const vector_double *vector,
                                 const char *filename
                                )
{
    long I;
    FILE *stream;
    char Full_Filename[255];

    if (strcmp("STDERR", filename) == 0)
    {
        /* Output will be written to screen */
        stream = stderr;
    }
    else
    {
        /* Generate full filename */
        vndrs_generate_full_filename(filename, Full_Filename);

        if ((stream = fopen(Full_Filename,"w")) == NULL)
        {
            fprintf(stderr, "Failed trying to open file %s\n", Full_Filename);
            vndrs_error_alert();
        }
    }

    for (I=0; I<vector->dimension; I++)
    {
        fprintf(stream, "%22.14e\n", vector->data[I]);
    }

    fprintf(stream, "\n");

    if (strcmp("STDERR", filename) != 0)
    {
        fclose(stream);
    }

    return;
}

/* Routine to dump complex vectors in ASCII format into a file */

void vndrs_fprintf_vector_complex(
const vector_complex *vector,
const char *filename
)
{
    long I;
    FILE *stream;
    char Full_Filename[255];

    if (strcmp("STDERR", filename) == 0)
    {
        stream = stderr;
    }
    else
    {
        /* Generate full filename */
        vndrs_generate_full_filename(filename, Full_Filename);

        if ((stream = fopen(Full_Filename,"w")) == NULL)
        {
            fprintf(stderr, "Failed trying to open file %s.\n", Full_Filename);
            vndrs_error_alert();
        }
    }
    for (I=0; I<vector->dimension; I++)
        fprintf(stream, "%22.14e,%22.14e\n", vector->data[I].re, vector->data[I].im);

    fprintf(stream, "\n");

    if (strcmp("STDERR", filename) != 0)
    {
        fclose(stream);
    }

    return;
}

/* Routine to dump double matrices in ASCII format into a file */

void vndrs_fprintf_matrix_double(
const matrix_double *matrix,
const char *filename
)
{
    long I, J;
    FILE *stream;
    char Full_Filename[255];

    if (strcmp("STDERR", filename) == 0)
    {
        stream = stderr;
    }
    else
    {
        /* Generate full filename */
        vndrs_generate_full_filename(filename, Full_Filename);

        if ((stream = fopen(Full_Filename,"w")) == NULL)
        {
            fprintf(stderr, "Failed trying to open file %s.\n", Full_Filename);
            vndrs_error_alert();
        }
    }

    for (I=0; I<matrix->dimension_1; I++)
    {
        for (J=0; J<matrix->dimension_2; J++)
            fprintf(stream, "%22.14e ", get_element_matrix_double(matrix, I, J));
        fprintf(stream, "\n");
    }

    if (strcmp("STDERR", filename) != 0)
    {
        fclose(stream);
    }

    return;
}

/* Routine to dump complex matrices in ASCII format into a file */

void vndrs_fprintf_matrix_complex(
const matrix_complex *matrix,
const char *filename
)
{
    long I, J;
    complex c_dummy;
    FILE *stream;
    char Full_Filename[255];

    if (strcmp("STDERR", filename) == 0)
    {
        stream = stderr;
    }
    else
    {
        /* Generate full filename */
        vndrs_generate_full_filename(filename, Full_Filename);

        if ((stream = fopen(Full_Filename,"w")) == NULL)
        {
            fprintf(stderr, "Failed trying to open file %s.\n", Full_Filename);
            vndrs_error_alert();
        }
    }

    for (I=0; I<matrix->dimension_1; I++)    
    {
        for (J=0; J<matrix->dimension_2; J++)
        {
            c_dummy = get_element_matrix_complex(matrix, I, J);
            fprintf(stream, "%22.14e,%22.14e ", c_dummy.re, c_dummy.im);
        }
        fprintf(stream, "\n");
    }

    if (strcmp("STDERR", filename) != 0)
    {
        fclose(stream);
    }

    return;
}



/******************************************************************************
*				European Southern Observatory
*		 Vlti ViNci Data Reduction Software (VNDRS)
*
* Module name:	vndrs_load_fluor_test_data
* Input/Output:	See function arguments to avoid duplication
* Description:	Routine to read FLUOR test data into the internal batch format
*
* History:		
* 01-May-01		(csabet) Revised
*				Most code recycled from P. Kervella's test program
******************************************************************************/
batch *vndrs_load_fluor_test_data(
	long	which)
{

	/*  Local Declarations
	--------------------*/
	batch	*Batch = NULL;	/* csabet 09-Aug-2001. Compiler may not initialise */
	long	nframes, nscans_A, nscans_B, nscans_off, nscans_on;
	long	I, J;
	FILE	*stream;


	/*  Algorithm
	-----------*/

    /*	Number of frames
	------------------*/
    nframes = 256;

    /*	Number of scans (Beam A and Beam B data)
	------------------------------------------*/
	nscans_A = 25;
	nscans_B = 25;
    
	/*	Off Source data
	-----------------*/
	nscans_off = 125;
    
	/*	On Source data
	----------------*/
	nscans_on = 10;

	switch (which)
	{
		case 0:
			/* On source data */
			/* ************** */
			Batch = vndrs_calloc_batch(nscans_on, nframes);

			if ( (stream = fopen("on_source.txt","r")) != NULL)
			{
				for (I=0;I<nscans_on;I++)
						for (J=0;J<nframes;J++)
				{
					for (J=0;J<nframes;J++)
						fscanf(stream,"%lf ", &(Batch->fullscans[I]->I1->data[J]));
					for (J=0;J<nframes;J++)
						fscanf(stream,"%lf ", &(Batch->fullscans[I]->I2->data[J]));
					for (J=0;J<nframes;J++)
						fscanf(stream,"%lf ", &(Batch->fullscans[I]->PA->data[J]));
                    for (J=0;J<nframes;J++)
                        fscanf(stream,"%lf ", &(Batch->fullscans[I]->PB->data[J]));
                    for (J=0;J<nframes;J++) Batch->fullscans[I]->QL->data[J] = 0.0;
                }
            }
            else
            {
                fprintf(stderr, "Failed trying to open file on_source.txt.\n");
                vndrs_error_alert();
            }
            break;

        case 1:
            /* Off source data */
            /* *************** */
            Batch = vndrs_calloc_batch(nscans_off, nframes);
        
            if ( (stream = fopen("off_source.txt","r")) != NULL)
            {
                for (I=0;I<nscans_off;I++)
                        for (J=0;J<nframes;J++)
                {
                    for (J=0;J<nframes;J++)
                        fscanf(stream,"%lf ", &(Batch->fullscans[I]->I1->data[J]));
                    for (J=0;J<nframes;J++)
                        fscanf(stream,"%lf ", &(Batch->fullscans[I]->I2->data[J]));
                    for (J=0;J<nframes;J++)
                        fscanf(stream,"%lf ", &(Batch->fullscans[I]->PA->data[J]));
                    for (J=0;J<nframes;J++)
                        fscanf(stream,"%lf ", &(Batch->fullscans[I]->PB->data[J]));
                }
            }
            else
            {
                fprintf(stderr, "Failed trying to open file off_source.txt.\n");
                vndrs_error_alert();
            }
            break;

        case 2:
            /* Beam A only data */
            /* **************** */
            Batch = vndrs_calloc_batch(nscans_A, nframes);

            if ( (stream = fopen("beam_A.txt","r")) != NULL)
            {
                for (I=0;I<nscans_A;I++)
                {
                    for (J=0;J<nframes;J++)
                        fscanf(stream,"%lf ", &(Batch->fullscans[I]->I1->data[J]));
                    for (J=0;J<nframes;J++)
                        fscanf(stream,"%lf ", &(Batch->fullscans[I]->I2->data[J]));
                    for (J=0;J<nframes;J++)
                        fscanf(stream,"%lf ", &(Batch->fullscans[I]->PA->data[J]));
                    for (J=0;J<nframes;J++)
                        fscanf(stream,"%lf ", &(Batch->fullscans[I]->PB->data[J]));
                }
            }
            else
            {
                fprintf(stderr, "Failed trying to open file beam_A.txt.\n");
                vndrs_error_alert();
            }
            break;

        case 3:
            /* Beam B only data */
            /* **************** */
            Batch = vndrs_calloc_batch(nscans_B, nframes);

            if ( (stream = fopen("beam_B.txt","r")) != NULL)
            {
                for (I=0;I<nscans_B;I++)
                        for (J=0;J<nframes;J++)
                {
                    for (J=0;J<nframes;J++)
                        fscanf(stream,"%lf ", &(Batch->fullscans[I]->I1->data[J]));
                    for (J=0;J<nframes;J++)
                        fscanf(stream,"%lf ", &(Batch->fullscans[I]->I2->data[J]));
                    for (J=0;J<nframes;J++)
                        fscanf(stream,"%lf ", &(Batch->fullscans[I]->PA->data[J]));
                    for (J=0;J<nframes;J++)
                        fscanf(stream,"%lf ", &(Batch->fullscans[I]->PB->data[J]));
                }
            }
            else
            {
                fprintf(stderr, "Failed trying to open file beam_B.txt.\n");
                vndrs_error_alert();
            }
            break;
    }

    return Batch;
}



void vndrs_calloc_vlti_file(FLUOR_file* fff, int dim) {

  int  i;

  /* printf("Allocating FLUOR file...\n"); */ 

  fff->dimension = dim;

  fff->isValid   = (int*)malloc(dim*sizeof(int));
  fff->source    = (int*)malloc(dim*sizeof(int));
  fff->targtype  = (int*)malloc(dim*sizeof(int));
  fff->step      = (int*)malloc(dim*sizeof(int));
  fff->on_beam1  = (int*)malloc(dim*sizeof(int));
  fff->on_beam2  = (int*)malloc(dim*sizeof(int));
  fff->datai1    = (int*)malloc(dim*sizeof(int));
  fff->datai2    = (int*)malloc(dim*sizeof(int));
  fff->photo1    = (int*)malloc(dim*sizeof(int));
  fff->photo2    = (int*)malloc(dim*sizeof(int));
  fff->filtered  = (int*)malloc(dim*sizeof(int));

  for(i=0; i<dim; i++) fff->isValid[i] = 0;

  /* printf("Done.\n"); */

}

void vndrs_free_vlti_file(FLUOR_file* fff) {

  fff->dimension = 0;

  if (fff->isValid)  free(fff->isValid);
  if (fff->source)   free(fff->source);
  if (fff->targtype) free(fff->targtype);
  if (fff->step)     free(fff->step);
  if (fff->on_beam1) free(fff->on_beam1);
  if (fff->on_beam2) free(fff->on_beam2);
  if (fff->datai1)   free(fff->datai1);
  if (fff->datai2)   free(fff->datai2);
  if (fff->photo1)   free(fff->photo1);
  if (fff->photo2)   free(fff->photo2);
  if (fff->filtered) free(fff->filtered);
}

void vndrs_calloc_vlti_file_def(FLUOR_file_def* ffd, int dim, int maxstep) {

  int i;

  /* printf("Allocating FLUOR file def...\n"); */ 

  ffd->nb_sources = dim;
  ffd->maxstep    = maxstep;

  ffd->nb_off    = (int*)malloc(dim*sizeof(int));
  ffd->nb_on     = (int*)malloc(dim*sizeof(int));
  ffd->nb_beam_A = (int*)malloc(dim*sizeof(int));
  ffd->nb_beam_B = (int*)malloc(dim*sizeof(int));

  for(i=0; i<dim; i++) {
        ffd->nb_off[i]    = 0;
        ffd->nb_on[i]     = 0;
        ffd->nb_beam_A[i] = 0;
        ffd->nb_beam_B[i] = 0;
  }

  /* printf("Done.\n"); */

}

void vndrs_free_vlti_file_def(FLUOR_file_def* ffd) {

  if (ffd->nb_off )   free(ffd->nb_off);
  if (ffd->nb_on )    free(ffd->nb_on); 
  if (ffd->nb_beam_A) free(ffd->nb_beam_A);
  if (ffd->nb_beam_B) free(ffd->nb_beam_B);
}

void check_fluor_data (fitsfile *fptr, imaging_data_table_def* tabledef, int *ierr, FLUOR_file_def* ffd, FLUOR_file* fff)

/*----------------------------------------------------------------------- */
/*   Loop reading table, fitting fringes, writing solutions if requested  */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      tabledef  imaging_data table definition structure                 */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*----------------------------------------------------------------------- */
{
  long i, nreg, step, nx, maxstep;
/*	double sumtau=0.0, sumd=0.0, sumd2=0.0, cnttau=0.0, cntd2=0.0; */	/* csabet 09-Aug-2001. Unused variable */

/*	int done=0, FOinit=0, row_nb; */
	int done=0, row_nb; 	/* csabet 09-Aug-2001. Unused variable */
  imaging_data_row *row; 
/*	float *OPD_off=NULL, *times=NULL, fract_corr=-1.0; */ /* csabet 09-Aug-2001. Unused variable */
/*	fitsfile *outfptr = NULL;
  float *IntChan=NULL;	*/	/* csabet 09-Aug-2001. Unused variable */
  /* which columns are the interferometric data */
  int i1col=0, i2col=1, p1col=2, p2col=3;

  /* Number of sources */
  int  source_index, max_source_id = 0;

  /* number of rows in input and last read */
  long input_nrows = 0;
  long input_last = 0;

  /* Current data variables */
  long on_beam1, on_beam2, datai1, datai2, photo1, photo2, targtype, sourceid;

  if (*ierr>0) return; /* existing error condition? */
  if (tabledef==NULL) {*ierr=10000; return;} /* check pointer */

  /* inputfptr = fptr;  save cfitsio pointer in global variable */ 

  /* allocate storage array */
  maxstep = tabledef->maxstep;
  /* 
  fprintf (stdout,"maxstep = %ld \n",maxstep);
  Arbitrary Initialisation
  Must be replaced by reading the source table of the FITS file 
  */
  vndrs_calloc_vlti_file_def(ffd, 10, maxstep);

  nx  = tabledef->detector->naxes[0][0]; /* number of pixels in Freq */
  if (nx>1) { /* error test */
/*	fprintf(stderr,"ERROR: too many (%d) frequency channels\n", nx); */
    fprintf(stderr,"ERROR: too many (%ld) frequency channels\n", nx); /* csabet 09-Aug-2001 */
    return;
  }

  nreg = tabledef->nregion;        /* number of regions */

  row = make_imaging_data_row (tabledef);

  /* get size of table */
  fits_get_num_rows (fptr, &input_nrows, ierr);
  /* printf("Expecting %d rows \n",input_nrows); */ 
  vndrs_calloc_vlti_file(fff, input_nrows);

  /* read first row only 
     input_last = 1;
  */ 
  *ierr = 0;

  /* printf("--> Looping\n"); */ 

  /* looping over scans */
  for (row_nb=0; row_nb<input_nrows; row_nb++) {
 
    read_imaging_data_table(fptr, tabledef, row, input_last, ierr);
    if (*ierr>0) { /* error test */
      fprintf(stderr,"ERROR %d reading IMAGING_DATA_TABLE\n", *ierr);
      return;
    }

    if (*ierr==-1) { /* finished? */
      done = 1;
      *ierr = 0;
    }

    if (done) break; /* no more data? */

       on_beam1  = row->opt_train[0]  <= 0  ? 0 : 1;
       on_beam2  = row->opt_train[1]  <= 0  ? 0 : 1;

        targtype = row->targtype[i1col];
        sourceid = row->source[i1col];
        source_index = sourceid-1; /* The first source is at index 0  */ 
	datai1 = row->data[i1col][0];
	datai2 = row->data[i2col][0];
	photo1 = row->data[p1col][0];
	photo2 = row->data[p2col][0];
        step   = row->step;

	/* 
        printf("%d %d %d %d %d %d %d %d %d\n",on_beam1,on_beam2,targtype,sourceid,step,datai1,datai2,photo1,photo2);
        */ 

        /* Loading data in memory */ 
        fff->isValid[row_nb]  = 1; /* Validated entry */ 

        fff->on_beam1[row_nb] = on_beam1;
        fff->on_beam2[row_nb] = on_beam2;

        fff->step[row_nb]     = step;
        fff->targtype[row_nb] = targtype;
        fff->source[row_nb]   = sourceid;

        /* Beware the inversion of indexes, due to a mismatch with the Labview code */ 
        fff->datai1[row_nb] = datai2;
        fff->datai2[row_nb] = datai1;
        fff->photo1[row_nb] = photo2;
        fff->photo2[row_nb] = photo1;

	/* Counting the number of scans for each source and category */ 
        if (on_beam1 == 0 && on_beam2 == 0) ffd->nb_off[source_index] += 1;
        if (on_beam1 == 1 && on_beam2 == 1) ffd->nb_on[source_index] += 1;
        if (on_beam1 == 0 && on_beam2 == 1) ffd->nb_beam_B[source_index] += 1;
        if (on_beam1 == 1 && on_beam2 == 0) ffd->nb_beam_A[source_index] += 1;
        if (source_index > max_source_id) max_source_id = (int) source_index;

   /* end loop over scans */
  }


  /* printf("Done reading table \n"); */ 
  
  /* delete row structure */
  kill_imaging_data_row(row);

  /* printf("Done freeing memory \n"); */ 

  /* Display table structure */ 

  printf("Number of sources : %d\n",max_source_id+1);
  ffd->nb_sources = max_source_id+1;

  for (i = 0; i<=max_source_id; i++) { 
/*	printf("--> Source %d (Name)\n",(i+1)); */
    printf("--> Source %ld (Name)\n",(i+1));	/* csabet 09-Aug-2001 */
    printf("    Off-source scans: %.0f (%d)\n",((float)ffd->nb_off[i]/(float)maxstep),ffd->nb_off[i]);
    ffd->nb_off[i] = ffd->nb_off[i]/maxstep;

    printf("    On-source scans : %.0f (%d)\n",((float)ffd->nb_on[i]/(float)maxstep), ffd->nb_on[i]);
    ffd->nb_on[i] = ffd->nb_on[i]/maxstep;

    printf("    Beam A scans    : %.0f (%d)\n",((float)ffd->nb_beam_A[i]/(float)maxstep), ffd->nb_beam_A[i]);
    ffd->nb_beam_A[i] = ffd->nb_beam_A[i]/maxstep;

    printf("    Beam B scans    : %.0f (%d)\n",((float)ffd->nb_beam_B[i]/(float)maxstep), ffd->nb_beam_B[i]);
    ffd->nb_beam_B[i] = ffd->nb_beam_B[i]/maxstep;

    }
} /* end check_data_table */



/******************************************************************************
*				European Southern Observatory
*		 Vlti ViNci Data Reduction Software (VNDRS)
*
* Module name:	
* Input/Output:	See function arguments to avoid duplication
* Description:	Routine to read FLUOR test data in FITS format.
*
*
*
* History:		
* 01-May-01		(csabet) Revised
* 5/11/2000		(P.Ballester)	Most code recycled from B.Cotton's test 
*				programs: FLUORfit.c FLUORfsub.c, ESO/DMD
*
******************************************************************************/
batch* vndrs_select_fluor_data(
	long	which,
	long	source_id)

{

	/*  Local Declarations
	--------------------*/
	batch	*Batch;
/*	int		ierr=0, row_nb, scan_nb=0, data_nb=0, selectedEntry; */
	int		row_nb, scan_nb=0, data_nb=0, selectedEntry; /* csabet 07-Aug-2001 */
	long	nframes, nscans, on_beam1, on_beam2, datai1, datai2, photo1, 
			photo2, targtype, sourceid, step;


	/*  Algorithm
	-----------*/

	/*	Check source_id
	-----------------*/
	if (source_id > ffd.nb_sources) 
	{ 
/*		printf("vndrs_select_fluor_data: Too large source_id = %d (max. %d)\n",source_id, ffd.nb_sources); */
		printf("vndrs_select_fluor_data: Too large source_id = %ld (max. %d)\n",source_id, ffd.nb_sources);
						/* csabet 09-Aug-2001 */
		exit(333);
	}

	if (source_id <= 0) 
	{ 
/*		printf("vndrs_select_fluor_data: source_id = %d is a strictly positive integer\n",source_id, ffd.nb_sources); */
		printf("vndrs_select_fluor_data: source_id = %ld is a strictly positive integer\n",
			source_id);
						/* csabet 07-Aug-2001. Too many arguments for format */
		exit(334);
	}



	/*	Which batch do we need
	------------------------*/
	nframes    = ffd.maxstep;
	if (which == 0) nscans = ffd.nb_off[source_id-1];
	else if (which == 1) nscans = ffd.nb_on[source_id-1];
	else if (which == 2) nscans = ffd.nb_beam_A[source_id-1];
	else if (which == 3) nscans = ffd.nb_beam_B[source_id-1];
	else 
	{
/*		printf("vndrs_select_fluor_data: Incorrect argument which=%d\n",which); */
		printf("vndrs_select_fluor_data: Incorrect argument which=%ld\n",which);
						/* csabet 07-Aug-2001 */
		exit(500);
	}

	/*	Allocate Batch
	----------------*/
	if (nscans <= 0) 
	{
/*		printf("vndrs_select_fluor_data: Incorrect scan number %d\n",nscans); */
		printf("vndrs_select_fluor_data: Incorrect scan number %ld\n",nscans); 
							/* csabet 07-Aug-2001 */
		exit(111);
		
	}
	else 
		Batch = vndrs_calloc_batch(nscans, nframes);


	/*	Selecting the data
	--------------------*/ 
	for (row_nb=0; row_nb<fff.dimension; row_nb++) 
	{
		on_beam1  = fff.on_beam1[row_nb];
		on_beam2  = fff.on_beam2[row_nb];
		targtype  = fff.targtype[row_nb];
		sourceid  = fff.source[row_nb];
		datai1    = fff.datai1[row_nb];
		datai2    = fff.datai2[row_nb];
		photo1    = fff.photo1[row_nb];
		photo2    = fff.photo2[row_nb];
		step      = fff.step[row_nb];
	
		/* 
        printf("%d %d %d %d %d %d %d %d %d\n",on_beam1,on_beam2,targtype,sourceid,step,datai1,
			datai2,photo1,photo2);
        */ 

		/*	Selecting entries
		-------------------*/ 
		selectedEntry = ( sourceid == source_id && ( 
		(on_beam1 == 0 && on_beam2 == 0 && which == 0) ||
		(on_beam1 == 1 && on_beam2 == 1 && which == 1) ||
		(on_beam1 == 0 && on_beam2 == 1 && which == 2) ||
		(on_beam1 == 1 && on_beam2 == 0 && which == 3) ));

		/*	Counting the number of scans for each source and category
		-----------------------------------------------------------*/ 
        if (selectedEntry) 
		{
			Batch->fullscans[scan_nb]->I1->data[data_nb] = datai1;
            Batch->fullscans[scan_nb]->I2->data[data_nb] = datai2;
            Batch->fullscans[scan_nb]->PA->data[data_nb] = photo1;
			Batch->fullscans[scan_nb]->PB->data[data_nb] = photo2;
			data_nb += 1;
			if (step == nframes) 
			{ 
				/*	For consistency, check that data_nb == nframes */ 
				scan_nb += 1, data_nb = 0;
			}
		}
	}

	printf("Select FLUOR data: Read %d complete scans\n",scan_nb);

	return(Batch);
}



/******************************************************************************
*				European Southern Observatory
*		 Vlti ViNci Data Reduction Software (VNDRS)
*
* Module name:	vndrs_open_fluor_data
* Input/Output:	See function arguments to avoid duplication
* Description:	
*
*
* History:		
* 01-May-01		(csabet) Revised
******************************************************************************/
int vndrs_open_fluor_data(
	char*	inpfile)
{

	/*  Local Declarations
	--------------------*/
   imaging_data_table_def	*tabledef;
   fitsfile					*infptr=NULL;
   int						ierr=0;

	/*  Algorithm
	-----------*/
	printf("Loading FLUOR file: %s\n", inpfile);

	/*	Open FITS input file
	----------------------*/
	fits_open_file(&infptr, inpfile, READONLY, &ierr);
	if (ierr>0) 
	{ /* error check */
		/* fprintf(stderr,"ERROR %d opening input FITS file %s \n", &ierr, inpfile); */
		fprintf(stderr,"ERROR %d opening input FITS file %s \n", ierr, inpfile);
				/* csabet 07-Aug-2001 */
		table_print_error_stack();  /* Show any cfitsio error messages */
		/* return ierr; */
	}/* end error test */


	/*	Get basic table structures and read IMAGING_DETECTOR table
	------------------------------------------------------------*/
	tabledef = open_imaging_data_table(infptr, &ierr);
	if ((tabledef==NULL) || (ierr>0)) 
	{ /* error test */
		fprintf (stderr,"error %d after table create\n",ierr);
		table_print_error_stack();  /* Show any cfitsio error messages */
		/* return ierr; */ 
	}		/* end error test */


	/*	Check table  fitting data
	---------------------------*/
	check_fluor_data(infptr, tabledef, &ierr, &ffd, &fff);
	if (ierr>0) { /* error test */
		fprintf (stderr,"error %d after table read\n",ierr);
		table_print_error_stack();  /* Show any cfitsio error messages */
		/* return ierr; */ 
	} /* end error test */


	/*	Close FITS file
	-----------------*/
	fits_close_file (infptr, &ierr);


	/*	Show any cfitsio error messages
	---------------------------------*/
	table_print_error_stack(); 


	return(ffd.nb_sources);
}



void vndrs_close_fluor_data()
{
  vndrs_free_vlti_file_def(&ffd);
  vndrs_free_vlti_file(&fff);

}

void printerror( int status)
{
    /*****************************************************/
    /* Print out cfitsio error messages and exit program */
    /*****************************************************/


    if (status)
    {
       fits_report_error(stderr, status); /* print error report */

       exit( status );    /* terminate the program, returning error status */
    }
    return;
}

/* Routine to read the VINCI IMAGING_DATA table */ 
void vndrs_load_vinci_imaging_data(batch* Batch, fitsfile **fptr, int nscans, int nframes) {

    int status,  nfound, anynull, ii, I, J;
    long frow, felem, nelem, elem, longnull; 
    float  *data1, *data2, *data3, *data4, *data5;
    float floatnull;
    char  strnull[10], *ttype[6], comment[80]; 

    status = 0;

    for (ii = 0; ii < 6; ii++)      /* allocate space for the column labels */
        ttype[ii] = (char *) malloc(FLEN_VALUE);  /* max label length = 69 */

      status = 0;
      fits_read_key(*fptr, TLONG, "NAXIS2", &nelem, comment, &status);

      data1 = (float*)  malloc(nelem*sizeof(float));
      data2 = (float*)  malloc(nelem*sizeof(float));
      data3 = (float*)  malloc(nelem*sizeof(float));
      data4 = (float*)  malloc(nelem*sizeof(float));
      data5 = (float*)  malloc(nelem*sizeof(float));
      /* time  = (double*) malloc(nelem*sizeof(double)); */ 

      /* read the column names from the TTYPEn keywords */
      fits_read_keys_str(*fptr, "TTYPE", 1, 6, ttype, &nfound, &status);

      /* printf(" Row %12s %12s %12s %12s %12s %12s \n", ttype[1], ttype[2], ttype[3],ttype[4], ttype[5]); */ 

      frow      = 1;
      felem     = 1;
      strcpy(strnull, " ");
      longnull  = 0;
      floatnull = 0.;

      /*  read the columns */ 
      /*  
      fits_read_col(*fptr, TSTRING, 1, frow, felem, nelem, strnull,  name,
                    &anynull, &status);
      fits_read_col(*fptr, TLONG, 2, frow, felem, nelem, &longnull,  dia,
                    &anynull, &status);
      */ 

      /*
      fits_read_col(*fptr, TDOUBLE, 1, frow, felem, nelem, &floatnull, time,
                    &anynull, &status);
      */ 

      fits_read_col(*fptr, TFLOAT, 2, frow, felem, nelem, &floatnull, data1,
                    &anynull, &status);

      fits_read_col(*fptr, TFLOAT, 3, frow, felem, nelem, &floatnull, data2,
                    &anynull, &status);

      fits_read_col(*fptr, TFLOAT, 4, frow, felem, nelem, &floatnull, data3,
                    &anynull, &status);

      fits_read_col(*fptr, TFLOAT, 5, frow, felem, nelem, &floatnull, data4,
                    &anynull, &status);

      fits_read_col(*fptr, TFLOAT, 6, frow, felem, nelem, &floatnull, data5,
                    &anynull, &status);

      /*       
      for (ii = 0; ii < 5; ii++)
        printf("%5d %12.4f %12.4f %12.4f %12.4f %12.4f\n",ii+1,data1[ii],data2[ii],data3[ii],data4[ii],data5[ii]);
      */

       /* Storing data in Batch structure */
       /* ******************************* */

       for (I=0;I<nscans;I++) {
             for (J=0;J<nframes;J++)
                {
                elem = I*nframes + J; 
                Batch->fullscans[I]->I1->data[J] = data1[elem];
                Batch->fullscans[I]->I2->data[J] = data2[elem];
                Batch->fullscans[I]->PA->data[J] = data3[elem];
                Batch->fullscans[I]->PB->data[J] = data4[elem];
                Batch->fullscans[I]->QL->data[J] = data5[elem];
                }
       }

    for (ii = 0; ii < 6; ii++)      /* free the memory for the column labels */
        free( ttype[ii] );

    free(data1);
    free(data2);
    free(data3);
    free(data4);
    free(data5);
    /* free(time); */ 
}


/* Routine to read the VINCI SCAN_DATA table */ 
void vndrs_load_vinci_scan_data(batch* Batch, fitsfile **fptr, int nscans) {

    int    status, nfound, anynull, ii, I;
    long   frow, felem, nelem, longnull; 
    double  *data1, *data2, *data3, *data4, *data5;
    double floatnull;
    char   strnull[10], *ttype[6], comment[80]; 

    status = 0;

    for (ii = 0; ii < 5; ii++)      /* allocate space for the column labels */
        ttype[ii] = (char *) malloc(FLEN_VALUE);  /* max label length = 69 */

      status = 0;
      fits_read_key(*fptr, TLONG, "NAXIS2", &nelem, comment, &status);

      data1 = (double*)  malloc(nelem*sizeof(double));
      data2 = (double*)  malloc(nelem*sizeof(double));
      data3 = (double*)  malloc(nelem*sizeof(double));
      data4 = (double*)  malloc(nelem*sizeof(double));
      data5 = (double*)  malloc(nelem*sizeof(double));
      /* time  = (double*) malloc(nelem*sizeof(double)); */ 

      /* read the column names from the TTYPEn keywords */
      fits_read_keys_str(*fptr, "TTYPE", 1, 5, ttype, &nfound, &status);

      /* printf(" Row %12s %12s %12s %12s %12s  \n", ttype[0], ttype[1], ttype[2],ttype[3], ttype[4]); */ 

      frow      = 1;
      felem     = 1;
      strcpy(strnull, " ");
      longnull  = 0;
      floatnull = 0.;

      /*  read the columns */ 
      /*  
      fits_read_col(*fptr, TSTRING, 1, frow, felem, nelem, strnull,  name,
                    &anynull, &status);
      fits_read_col(*fptr, TLONG, 2, frow, felem, nelem, &longnull,  dia,
                    &anynull, &status);
      fits_read_col(*fptr, TDOUBLE, 1, frow, felem, nelem, &floatnull, time,
                    &anynull, &status);
      */ 

      fits_read_col(*fptr, TDOUBLE, 1, frow, felem, nelem, &floatnull, data1,
                    &anynull, &status);

      fits_read_col(*fptr, TDOUBLE, 2, frow, felem, nelem, &floatnull, data2,
                    &anynull, &status);

      fits_read_col(*fptr, TDOUBLE, 3, frow, felem, nelem, &floatnull, data3,
                    &anynull, &status);

      fits_read_col(*fptr, TDOUBLE, 4, frow, felem, nelem, &floatnull, data4,
                    &anynull, &status);

      fits_read_col(*fptr, TDOUBLE, 5, frow, felem, nelem, &floatnull, data5,
                    &anynull, &status);

      /* 
      for (ii = 0; ii < 5; ii++)
        printf("%5d %12.4f %12.4f %12.4f %12.4f %12.4f\n",ii+1,data1[ii],data2[ii],data3[ii],data4[ii],data5[ii]);

      */ 

       /* Storing data in Batch structure */
       /* ******************************* */
     
       for (I=0;I<nscans;I++) {
                Batch->scan_data[I].time        = data1[I];
                Batch->scan_data[I].dl_offset   = data3[I];
                Batch->scan_data[I].mu1_squared = -99.00;
                Batch->scan_data[I].mu2_squared = -99.00;
       }

    for (ii = 0; ii < 5; ii++)      /* free the memory for the column labels */
        free( ttype[ii] );

    free(data1);
    free(data2);
    free(data3);
    free(data4);
    free(data5);

}


float* vndrs_load_vinci_batch(fitsfile **fptr) {

    int status,  nfound, anynull;
    long naxes[2], fpixel, nbuffer, npixels, ii;

#define buffsize 1000000
    float datamin, datamax, nullval, *buffer;
    char  extname[80], comment[80];

    status = 0;
    /* read the column names from the TTYPEn keywords */
    fits_read_key(*fptr, TSTRING, "EXTNAME", extname, comment, &status);
    /* printf("Extension name: %s // %s\n",extname, comment); */ 

    /* read the NAXIS1 and NAXIS2 keyword to get image size */
    if ( fits_read_keys_lng(*fptr, "NAXIS", 1, 2, naxes, &nfound, &status) )
         printerror( status );

    npixels  = naxes[0] * naxes[1];         /* number of pixels in the image */
    buffer   = (float*) malloc(npixels*sizeof(float*));

    /* printf("Reading image size: %dx%d\n",naxes[0],naxes[1]); */ 

    fpixel   = 1;
    nullval  = 0;                /* don't check for null values in the image */
    /* datamin  = 1.0E30; */
    datamin  = (float) (1.0E30);	/*	Is float large enough? csabet 26.Jun.01 */
    /* datamax  = -1.0E30; */
    datamax  = (float) (-1.0E30);	/*	Is float large enough? csabet 26.Jun.01 */

      /* Note that even though the FITS images contains unsigned integer */
      /* pixel values (or more accurately, signed integer pixels with    */
      /* a bias of 32768),  this routine is reading the values into a    */
      /* float array.   Cfitsio automatically performs the datatype      */
      /* conversion in cases like this.                                  */

    nbuffer = npixels;

      if ( fits_read_img(*fptr, TFLOAT, fpixel, nbuffer, &nullval,
                  buffer, &anynull, &status) )
           printerror( status );

      for (ii = 0; ii < nbuffer; ii++)  {
        if ( buffer[ii] < datamin )
            datamin = buffer[ii];

        if ( buffer[ii] > datamax )
            datamax = buffer[ii];
      }

      fpixel  += nbuffer;    /* next pixel to be read in image */

      /* printf("\nMin and max image pixels =  %.0f, %.0f\n", datamin, datamax); */ 

    return buffer;
}



/* Reads a FITS keyword from a FITS header unit. 

   The value is printed in a character string in the form: <keyword_name> = <keyword_value>  // Comment 

   The mode defined the behavior in case of missing keywords: 
       mode=-1 aborts with error message, 
       mode=0  prints a warning and returns default value, 
       mode=1 silently assigns the default value. 

   default value is returned if the keyword does not exist and if mode=1
*/ 
double vndrs_read_double_fits_keyword(fitsfile *infptr, 
                                      char*    key_name, 
                                      char*    key_string,
                                      int      key_mode,
                                      double   key_default)
{
  double key_value;
  int    status=0;
  char   error_message[MAX_STRING_LENGTH]; 
  char   comment[MAX_STRING_LENGTH];

  strcpy(comment,"");  
  /* read the FRAMERATE keyword */ 
  if (fits_read_key(infptr, TDOUBLE, key_name, &key_value, comment, &status)) { 

        key_value=key_default;
        sprintf(error_message,"Missing FITS keyword: %s",key_name);

        if (key_mode == -1)  { 
	  printf("Error: %s (Abort).\n",error_message);
                 exit(155);
	}
        else if (key_mode == 0) {
	  /* printf("Warning: %s (Assigned default value '%s')\n",error_message,key_value);  */ 
                 sprintf(key_string, "%-35s = 'UNKNOWN' / ",key_name);       
	}
        else if (key_mode == 1) { 
	  /*       sprintf(key_string, "%-35s = %lf / %s",key_name,key_value,comment);  */ 
                 sprintf(key_string, "%-35s = 'UNKNOWN' / ",key_name);       
	}
        else { 
          printf("Invalid option key_mode=%d in module vndrs_IO::vndrs_read_xxx_fits_keyword()\n",key_mode);
        }

  }
  else 
     /* sprintf(key_string, "%-35s = %20.9lf / %s",key_name,key_value,comment); */
	 sprintf(key_string, "%-35s = %20.9f / %s",key_name,key_value,comment); 
						 /* csabet 07-Aug-2001 */

  return(key_value);
}

/* Reads a FITS keyword from a FITS header unit. 

   The value is printed in a character string in the form: <keyword_name> = <keyword_value>  // Comment 

   The mode defined the behavior in case of missing keywords: 
       mode=-1 aborts with error message, 
       mode=0  prints a warning and returns default value, 
       mode=1 silently assigns the default value. 

   default value is returned if the keyword does not exist and if mode=1
*/ 

int vndrs_read_int_fits_keyword(fitsfile *infptr, 
                                char*    key_name, 
                                char*    key_string,
                                int      key_mode,
                                int   key_default)
{
  int    key_value, status=0;
  char   error_message[MAX_STRING_LENGTH]; 
  char   comment[MAX_STRING_LENGTH];

  strcpy(comment,"");  
  /* read the FRAMERATE keyword */ 
  if (fits_read_key(infptr, TINT, key_name, &key_value, comment, &status)) { 

        key_value=key_default;
        sprintf(error_message,"Missing FITS keyword: %s",key_name);

        if (key_mode == -1)  { 
	  printf("Error: %s (Abort).\n",error_message);
                 exit(155);
	}
        else if (key_mode == 0) { 
	  /* printf("Warning: %s (Assigned default value '%s')\n",error_message,key_value);  */ 
                 sprintf(key_string, "%-35s = 'UNKNOWN' /",key_name);       
	}
        else if (key_mode == 1) { 
                 sprintf(key_string, "%-35s = 'UNKNOWN' /",key_name);       
	}
        else { 
          printf("Invalid option key_mode=%d in module vndrs_IO::vndrs_read_xxx_fits_keyword()\n",key_mode);
        }

  }
  else 
     sprintf(key_string, "%-35s = %d / %s",key_name,key_value,comment); 

  return(key_value);
}


/* Reads a FITS keyword from a FITS header unit. 

   The value is printed in a character string in the form: <keyword_name> = <keyword_value>  // Comment 

   The mode defined the behavior in case of missing keywords: 
       mode=-1 aborts with error message, 
       mode=0  prints a warning and returns default value, 
       mode=1 silently assigns the default value. 

   default value is returned if the keyword does not exist and if mode=1
*/
char* vndrs_read_string_fits_keyword(fitsfile *infptr, 
                                      char*    key_name, 
                                      char*    key_string,
                                      int      key_mode,
                                      char*    key_default)
{
  int    status=0;
  char*  key_value, error_message[MAX_STRING_LENGTH]; 
  char   comment[MAX_STRING_LENGTH];

  key_value = (char*) malloc(MAX_STRING_LENGTH);

  strcpy(comment,"");  
  /* read the FRAMERATE keyword */ 
  if (fits_read_key(infptr, TSTRING, key_name, key_value, comment, &status)) { 

        strcpy(key_value,key_default);
        sprintf(error_message,"Missing FITS keyword: %s",key_name);

        if (key_mode == -1)  { 
	  printf("Error: %s (Abort).\n",error_message);
                 exit(155);
	}
        else if (key_mode == 0)  
                 printf("Warning: %s (Assigned default value '%s')\n",error_message,key_value); 
        else if (key_mode == 1);
        else { 
          printf("Invalid option key_mode=%d in module vndrs_IO::vndrs_read_xxx_fits_keyword()\n",key_mode);
        }
  }

  /* sprintf(key_string, "%-30s = %s / %s",key_name,key_value,comment);  */ 
  sprintf(key_string, "%-35s = '%s' / %s",key_name,key_value,comment); 
  return(key_value);
}


/* Reads a FITS keyword from a FITS header unit. 

   The value is printed in a logical in the form: <keyword_name> = <keyword_value>  // Comment 

   The mode defined the behavior in case of missing keywords: 
       mode=-1 aborts with error message, 
       mode=0  prints a warning and returns default value, 
       mode=1 silently assigns the default value. 

   default value is returned if the keyword does not exist and if mode=1
*/
char* vndrs_read_logical_fits_keyword(fitsfile *infptr, 
                                      char*    key_name, 
                                      char*    key_string,
                                      int      key_mode,
                                      char     key_default)
{
  int    status=0;
  char*  key_value, error_message[MAX_STRING_LENGTH]; 
  char   comment[MAX_STRING_LENGTH];

  key_value = (char*) malloc(MAX_STRING_LENGTH);

  strcpy(comment,"");  
  /* read the keyword */ 
  if (fits_read_key(infptr, TLOGICAL, key_name, key_value, comment, &status)) { 
	key_value[0] = key_default; key_value[1] = 0;
        sprintf(error_message,"Missing FITS keyword: %s",key_name);

        if (key_mode == -1)  { 
	  printf("Error: %s (Abort).\n",error_message);
                 exit(155);
	}
        else if (key_mode == 0)  
                 printf("Warning: %s (Assigned default value '%s')\n",error_message,key_value); 
        else if (key_mode == 1);
        else { 
          printf("Invalid option key_mode=%d in module vndrs_IO::vndrs_read_xxx_fits_keyword()\n",key_mode);
        }
  }
   key_value[1] = 0;
  /* sprintf(key_string, "%-30s = %s / %s",key_name,key_value,comment);  */ 
  sprintf(key_string, "%-35s = '%s' / %s",key_name,key_value,comment); 
  return(key_value);
}

void   vndrs_read_vinci_header(char* inpfile, 
                               int* obsmode, 
                               int* nscan, 
                               int* nfram, 
                               int* insmode, 
                               int* acqmode, 
                               double* fringespeed, 
                               double* framerate, 
                               double* mjdobs
                               ) 
{
   fitsfile *infptr=NULL;
   int ierr=0, nb_hdu,hdunum, hdutype,status=0;
   char comment[80];

   /* printf("Loading VINCI file: %s\n", inpfile); */ 

  /* Open FITS input file */
  fits_open_file(&infptr, inpfile, READONLY, &ierr);
  if (ierr>0) { /* error check */
    /* fprintf(stderr,"ERROR %d opening input FITS file %s \n", &ierr, inpfile); */
	fprintf(stderr,"ERROR %d opening input FITS file %s \n", ierr, inpfile); /* csabet 07-Aug-2001 */
    table_print_error_stack();  /* Show any cfitsio error messages */
    /* return ierr; */ 
  }/* end error test */

  status = 0; /* The status variable must be initialised */
  if (fits_get_num_hdus (infptr, &nb_hdu, &status)) printerror( status );

  /* printf("Found %d HDUs \n",nb_hdu); */ 
  hdunum = 1;
  status = 0;

  /* move to the first HDU */
  if ( fits_movabs_hdu(infptr, hdunum, &hdutype, &status) )  printerror( status );
  if ( hdutype != IMAGE_HDU ) { 
     printf("File %s not starting with an image type extension. Abort...\n",inpfile);
     exit(0);
  } 

  strcpy(comment,"no comment...");

  /* read the INSTRUME keyword */ 
  /* 
  fits_read_key(infptr, TSTRING, "INSTRUME", line, comment, &status);
  printf("INSTRUME = %s / %s\n",line,comment);
  */

  /* read the MJD-OBS keyword */ 
  if ( fits_read_key(infptr, TDOUBLE, "MJD-OBS", mjdobs, comment, &status) ) *mjdobs = -1;
  /* printf("MJD-OBS = %lf / %s\n",*mjdobs,comment); */ 

  /* read the FRINGESPEED keyword */ 
  if (fits_read_key(infptr, TDOUBLE, "HIERARCH ESO DET QL FRINGESPD", fringespeed, comment, &status)) *fringespeed=0.00;
  /* printf("DET QL FRINGESPD = %lf / %s\n",*fringespeed,comment); */ 

  /* read the FRAMERATE keyword */ 
  if (fits_read_key(infptr, TDOUBLE, "HIERARCH ESO DET QL FRAMERATE", framerate, comment, &status)) *framerate=0.00;
  /* printf("DET QL FRAMERATE = %lf / %s\n",*framerate,comment); */ 

  /* read the OBSMODE keyword */ 
  if (fits_read_key(infptr, TINT, "HIERARCH ESO DET QL OBSMODE", obsmode, comment, &status)) *obsmode = -1;
  /* printf("DET QL OBSMODE = %d / %s\n",*obsmode,comment); */ 

  /* read the INSMODE keyword */ 
  if (fits_read_key(infptr, TINT, "HIERARCH ESO DET QL INSMODE", insmode, comment, &status)) *insmode=-1;
  /* printf("DET QL INSMODE = %d / %s\n",*insmode,comment); */ 

  /* read the ACQMODE keyword */ 
  if (fits_read_key(infptr, TINT, "HIERARCH ESO DET DACQ MODE",  acqmode, comment, &status)) *acqmode=-1;
  /* printf("DET DACQ MODE = %d / %s\n",*acqmode,comment); */ 

  /* read the NSCAN keyword */ 
  if (fits_read_key(infptr, TINT, "HIERARCH ESO DET NC NSCAN", nscan, comment, &status)) *nscan=-1;
  /* printf("DET NC NSCAN = %d / %s\n",*nscan,comment); */ 

  /* read the NFRAMES keyword */ 
  if (fits_read_key(infptr, TINT, "HIERARCH ESO DET RM NFRAM", nfram, comment, &status)) *nfram = -1;
  /* printf("DET RM NFRAM = %d / %s\n",*nfram,comment); */

  /* Close FITS file */
  fits_close_file (infptr, &ierr);

   /* Show any cfitsio error messages */
  table_print_error_stack(); 

}



/*	csabet 10-Oct-2001. Reorganised */
batch* vndrs_open_vinci_data(
	char*	inpfile,
	long	numberOfScansRequested)
{
	/*	Local declarations
	--------------------*/
	fitsfile	*infptr=NULL;
	int			ierr=0, nb_hdu,hdunum, hdutype,status=0;
	batch		*Batch;
	int			dimension, nframes = 0, nscans = 0, nscans_requested = 0;
        /* csabet 09-Aug-2001. Uninitialized variables */
	char		extname[80], comment[80],text_string[MAX_STRING_LENGTH];
	long		naxis2;


	/*	printf("Loading VINCI binary table: %s\n", inpfile); */

	Batch = NULL; 

	/*	Open FITS input file 
	----------------------*/
	fits_open_file(&infptr, inpfile, READONLY, &ierr);

	if (ierr>0) 
	{	/*	Error check */
		fprintf(stderr,"ERROR %d opening input FITS file %s \n", ierr, inpfile);
		/*	csabet 07-Aug-2001. Pointer conflict corrected */
		table_print_error_stack();  /* Show any cfitsio error messages */
/*		return ierr; */ 
	}	/*	end error test */

	status = 0; /* The status variable must be initialised */
	if (fits_get_num_hdus (infptr, &nb_hdu, &status))
		printerror( status );

/*		printf("Found %d HDUs \n",nb_hdu); */

	/*	Reading the 5 first image extensions */ 
	for (hdunum=1; hdunum<=nb_hdu; hdunum++) {

		/*	move to the HDU */
		if ( fits_movabs_hdu(infptr, hdunum, &hdutype, &status) ) 
			printerror( status );

/*		if (fits_movrel_hdu(infptr,1,&hdutype,&status)) 
			printerror(status);	
*/

		if (hdutype == ASCII_TBL)
/*			printf("\nReading ASCII table in HDU %d:\n",  hdunum) */;
		else if (hdutype == IMAGE_HDU) 
		{ 
		    /*			printf("\nReading image extension in HDU %d:\n", hdunum); */
			nframes = vndrs_read_int_fits_keyword(infptr,"HIERARCH ESO DET RM NFRAM",text_string,(-1),(0));
			nscans  = vndrs_read_int_fits_keyword(infptr,"HIERARCH ESO DET NC NSCAN",text_string,(-1),(0));

			if (numberOfScansRequested != - 1)	
				nscans_requested = numberOfScansRequested;
			else                            
				nscans_requested = nscans;


			Batch   = vndrs_calloc_batch(nscans_requested, nframes); 

		}
		else if (hdutype == BINARY_TBL) 
		{ 
/*			printf("\nReading binary table in HDU %d:\n", hdunum) */ ;
			status = 0;
			/*	Read the column names from the TTYPEn keywords */
			fits_read_key(infptr, TSTRING, "EXTNAME", extname, comment, &status);
/*			printf("Extension name: %s // %s\n",extname, comment); */ 

			/*	Read the NAXIS1 and NAXIS2 keyword to get image size */
			fits_read_key(infptr, TLONG, "NAXIS2", &naxis2, comment, &status);

			/*			printf("NAXIS2 = %d \n",naxis2); */ 

			printerror( status );

			if (strcmp(extname,"SCAN_DATA") == 0)  
			{ 
				if (nscans != naxis2) 
				{ 
					printf("Invalid number of scans. Expecting %d. Found %ld. (Abort)\n",nscans,naxis2); 
					exit(222); 
				}
				else 
				{ 
					vndrs_load_vinci_scan_data(Batch, &infptr, nscans_requested); 
				} 
			}  /* End of SCAN_DATA */ 

			if (strcmp(extname,"IMAGING_DATA") == 0)  
			{ 
				if (nframes  != naxis2/nscans) 
				{
					printf("Invalid number of frames. Expecting %d. Found %ld. (Abort)\n",
						nframes,naxis2/nscans); 
					exit(222);              
				}

				else 
				{ 

/*					printf("NFRAMES = %d\n",nframes); */ 
					dimension  = naxis2;
					/*	Dimension is the number of scans, nframes */ 
					vndrs_calloc_vlti_file(&fff, dimension);      /* Total number of rows */ 
					vndrs_calloc_vlti_file_def(&ffd, 1, nframes); /* Only 1 source, nframes */ 
					/* Reading the table extension */ 
					vndrs_load_vinci_imaging_data(Batch, &infptr, nscans_requested, nframes); 


				}
			} /* End of IMAGING_DATA */ 
		}
		else
		{
			printf("Error: this HDU is not an ASCII or binary table\n");
			printerror( status );
		}
	} 


	/*	Close FITS file */
	fits_close_file (infptr, &ierr);



	/*	Show any cfitsio error messages */
	table_print_error_stack(); 

	return(Batch);
}



void vndrs_close_vinci_data()
{
  vndrs_free_vlti_file_def(&ffd);
  vndrs_free_vlti_file(&fff);
}




/* The routine vndrs_export_header() writes FITS keywords to an ASCII file */ 
void vndrs_export_header(char* filename,           /* Name of the produced ASCII header file */ 
                         char* inpfile)            /* Name of the input on-source FITS file */ 
{ 
  double key_double;
  int    key_int;
  char   *key_string, text_string[MAX_STRING_LENGTH];
  char   key_name[MAX_STRING_LENGTH], key_type;


   fitsfile *infptr=NULL;
   int ierr=0, nb_hdu,hdunum, hdutype,status=0;

    int  I=0;
    FILE *stream;

    if (strcmp("STDERR", filename) == 0)
    {
        /* Output will be written to screen */
        stream = stderr;
    }
    else
    {
        if ((stream = fopen(filename,"w")) == NULL)
        {
            fprintf(stderr, "Failed trying to open file %s\n", filename);
            vndrs_error_alert();
        }
    }

   fprintf(stream,"# Header of VINCI FITS file: %s\n", inpfile);

  /* Open FITS input file */
  fits_open_file(&infptr, inpfile, READONLY, &ierr);
  if (ierr>0) { /* error check */
    fprintf(stderr,"ERROR %d opening input FITS file %s \n", ierr, inpfile);
		/* csabet 07-Aug-2001. Type corrected */
		
    table_print_error_stack();  /* Show any cfitsio error messages */
    /* return ierr; */ 
  }/* end error test */

  status = 0; /* The status variable must be initialised */
  if (fits_get_num_hdus (infptr, &nb_hdu, &status)) printerror( status );

  fprintf(stream,"# Found %d HDUs \n",nb_hdu);
  hdunum = 1;
  status = 0;

  /* move to the first HDU */
  if ( fits_movabs_hdu(infptr, hdunum, &hdutype, &status) )  printerror( status );
  if ( hdutype != IMAGE_HDU ) { 
     fprintf(stderr,"File %s not starting with an image type extension. Abort...\n",inpfile);
     exit(0);
  } 

  for (I=0; (strcmp(export_keylist[I],"END") != 0); I++) {
      key_type = toupper(export_keylist[I][0]);
      strncpy(key_name,export_keylist[I] + 2,strlen(export_keylist[I])-1); 
      if (export_dbx==1) fprintf(stream,"# Reading keyword %s of type %c\n",key_name,key_type);
      switch(key_type) {

      case('D'): 
         /* read a double keyword */ 
         key_double = vndrs_read_double_fits_keyword(infptr,key_name,text_string,export_mode,export_dbl_default); 
         break;

      case('I'): 
         /* read a double keyword */ 
        key_int = vndrs_read_int_fits_keyword(infptr,key_name,text_string,export_mode,export_int_default); 
         break;

      case('C'): 
         /* read a string keyword */ 
         key_string = vndrs_read_string_fits_keyword(infptr,key_name,text_string,export_mode,export_str_default); 
         free(key_string);
	   break;

      case('L'): 
         /* read a logical keyword */ 
         key_string = vndrs_read_logical_fits_keyword(infptr,key_name,text_string,export_mode,export_log_default); 
         free(key_string);
	   break;

      case('#'): 
	/* Prints a comment line */ 
        sprintf(text_string,"# %s",key_name);
        break;
       }
      /* Prints the keyword name, value, and comment */  
      fprintf(stream,"%s\n",text_string); 
  }
      /* do it again for second half */
  for (I=0; (strcmp(export_keylist2[I],"END") != 0); I++) {
      key_type = toupper(export_keylist2[I][0]);
      strncpy(key_name,export_keylist2[I] + 2,strlen(export_keylist2[I])-1); 
      if (export_dbx==1) fprintf(stream,"# Reading keyword %s of type %c\n",key_name,key_type);
      switch(key_type) {

      case('D'): 
         /* read a double keyword */ 
         key_double = vndrs_read_double_fits_keyword(infptr,key_name,text_string,export_mode,export_dbl_default); 
         break;

      case('I'): 
         /* read a double keyword */ 
        key_int = vndrs_read_int_fits_keyword(infptr,key_name,text_string,export_mode,export_int_default); 
         break;

      case('C'): 
         /* read a string keyword */ 
         key_string = vndrs_read_string_fits_keyword(infptr,key_name,text_string,export_mode,export_str_default); 
         free(key_string);
	   break;

      case('L'): 
         /* read a logical keyword */ 
         key_string = vndrs_read_logical_fits_keyword(infptr,key_name,text_string,export_mode,export_log_default); 
         free(key_string);
	   break;

      case('#'): 
	/* Prints a comment line */ 
        sprintf(text_string,"# %s",key_name);
        break;
  }

      /* Prints the keyword name, value, and comment */  
      fprintf(stream,"%s\n",text_string); 
  }

  /* Close FITS file */
  fits_close_file (infptr, &ierr);

   /* Show any cfitsio error messages */
  table_print_error_stack(); 

    if (strcmp("STDERR", filename) != 0)
    {
        fclose(stream);
    }

} 

/* The routine vndrs_export_data() writes 4 batches to an ASCII file */ 
void vndrs_export_data  (char*  filename,      /* Name of the produced ASCII header file */ 
                         batch* Off_Source,    /* Input off-source batch */ 
                         batch* On_Source,     /* Input on-source batch */ 
                         batch* Beam_A,        /* Input beamA batch */ 
                         batch* Beam_B)        /* Input beamB batch */ 
{ 

    int    obsmode,I,J;
    int    nscans,nframes;
    FILE   *stream;
    batch* Batch[4];
    char   vndrs_raw_prefix_format[MAX_STRING_LENGTH]={"%d\t%d\t%d\t%18.9lf\t%12.6lf\t%6.1lf\t%6.1lf\t"}; 
    char   vndrs_raw_number_format[MAX_STRING_LENGTH]={"%.0lf\t"}; 

    if (strcmp("STDERR", filename) == 0)
    {
        /* Output will be written to screen */
        stream = stderr;
    }
    else
    {
        if ((stream = fopen(filename,"w")) == NULL)
        {
            fprintf(stderr, "Failed trying to open file %s\n", filename);
            vndrs_error_alert();
        }
    }

    Batch[0] = Off_Source; /* OBSMODE = 1 */
    Batch[1] = Beam_A;     /* OBSMODE = 2 */
    Batch[2] = Beam_B;     /* OBSMODE = 3 */
    Batch[3] = On_Source;  /* OBSMODE = 4 */

    for (obsmode=1; obsmode<=4; obsmode++) { 

       nscans  = Batch[obsmode-1]->number_of_scans;
       nframes = Batch[obsmode-1]->fullscans[0]->I1->dimension; 

       /* 
       fprintf(stream,"# Batch OBSMODE=%d with %d scans of dimension %d\n",obsmode,nscans,nframes);
       fprintf(stream,"# Signals I1, I2, PA, PB\n");
       */ 

       for (I=0;I<nscans;I++) {

	 /* I1 row */ 
	 fprintf(stream,vndrs_raw_prefix_format,
                        obsmode,
                        (I+1),
                        1,
                        Batch[obsmode-1]->scan_data[I].time,
                        Batch[obsmode-1]->scan_data[I].dl_offset,
                        Batch[obsmode-1]->scan_data[I].mu1_squared,
                        Batch[obsmode-1]->scan_data[I].mu2_squared
                        );
         for (J=0;J<nframes;J++) fprintf(stream,vndrs_raw_number_format,Batch[obsmode-1]->fullscans[I]->I1->data[J]);
	 fprintf(stream,"\n");

	 /* I2 row */ 
	 fprintf(stream,vndrs_raw_prefix_format,
                        obsmode,
                        (I+1),
                        2,
                        Batch[obsmode-1]->scan_data[I].time,
                        Batch[obsmode-1]->scan_data[I].dl_offset,
                        Batch[obsmode-1]->scan_data[I].mu1_squared,
                        Batch[obsmode-1]->scan_data[I].mu2_squared
                        );
         for (J=0;J<nframes;J++) fprintf(stream,vndrs_raw_number_format,Batch[obsmode-1]->fullscans[I]->I2->data[J]);
	 fprintf(stream,"\n");

	 /* PA row */ 
	 fprintf(stream,vndrs_raw_prefix_format,
                        obsmode,
                        (I+1),
                        3,
                        Batch[obsmode-1]->scan_data[I].time,
                        Batch[obsmode-1]->scan_data[I].dl_offset,
                        Batch[obsmode-1]->scan_data[I].mu1_squared,
                        Batch[obsmode-1]->scan_data[I].mu2_squared
                        );
         for (J=0;J<nframes;J++) fprintf(stream,vndrs_raw_number_format,Batch[obsmode-1]->fullscans[I]->PA->data[J]);
	 fprintf(stream,"\n");

	 /* PB row */ 
	 fprintf(stream,vndrs_raw_prefix_format,
                        obsmode,
                        (I+1),
                        4,
                        Batch[obsmode-1]->scan_data[I].time,
                        Batch[obsmode-1]->scan_data[I].dl_offset,
                        Batch[obsmode-1]->scan_data[I].mu1_squared,
                        Batch[obsmode-1]->scan_data[I].mu2_squared
                        );
         for (J=0;J<nframes;J++) fprintf(stream,vndrs_raw_number_format,Batch[obsmode-1]->fullscans[I]->PB->data[J]);
	 fprintf(stream,"\n");

         /* Empty line between each scan 	 fprintf(stream,"\n"); */ 
       }

    }

    if (strcmp("STDERR", filename) != 0)
    {
        fclose(stream);
    }


} 

/* The routine vndrs_export_data() writes 4 batches to an ASCII file */ 
void vndrs_export_proc  (char*  filename,            /* Name of the produced ASCII header file */ 
                         batch* Batch                /* Input processed on-source batch */ 
                         )
{ 

    int    obsmode,I,J;
    int    nscans,nframes;
    FILE   *stream;
    char   vndrs_proc_prefix_format[MAX_STRING_LENGTH]={"%d\t%d\t%d\t%18.9lf\t%12.6lf\t%.6lf\t%.6lf\t"}; 
    char   vndrs_proc_number_format[MAX_STRING_LENGTH]={"%.6lf\t"}; 

    if (strcmp("STDERR", filename) == 0)
    {
        /* Output will be written to screen */
        stream = stderr;
    }
    else
    {
        if ((stream = fopen(filename,"w")) == NULL)
        {
            fprintf(stderr, "Failed trying to open file %s\n", filename);
            vndrs_error_alert();
        }
    }

       nscans  = Batch->number_of_scans;
       nframes = Batch->fullscans[0]->I1->dimension; 

       /* 
       fprintf(stream,"# Batch OBSMODE=%d with %d scans of dimension %d\n",obsmode,nscans,nframes);
       fprintf(stream,"# Signals I1, I2, PA, PB\n");
       */ 

       obsmode = 4;

       for (I=0;I<nscans;I++) {

	 /* I1 row */ 
	 fprintf(stream,vndrs_proc_prefix_format, 
                        obsmode,
                        (I+1),
                        5,
                        Batch->scan_data[I].time,
                        Batch->scan_data[I].dl_offset,
                        Batch->scan_data[I].mu1_squared,
                        Batch->scan_data[I].mu2_squared
                        );
         for (J=0;J<nframes;J++) fprintf(stream,vndrs_proc_number_format,Batch->fullscans[I]->I1->data[J]);
	 fprintf(stream,"\n");

	 /* I2 row */ 
	 fprintf(stream,vndrs_proc_prefix_format, 
                        obsmode,
                        (I+1),
                        6,
                        Batch->scan_data[I].time,
                        Batch->scan_data[I].dl_offset,
                        Batch->scan_data[I].mu1_squared,
                        Batch->scan_data[I].mu2_squared
                        );
         for (J=0;J<nframes;J++) fprintf(stream,vndrs_proc_number_format,Batch->fullscans[I]->I2->data[J]);
	 fprintf(stream,"\n");

       }


    if (strcmp("STDERR", filename) != 0)
    {
        fclose(stream);
    }


} 

