/*******************************************************************/
/* VLTI VINCI DATA REDUCTION SOFTWARE (VNDRS)                      */
/*                                                                 */
/* FILE: vndrs_IO.h                                                */
/*                                                                 */
/* Contains definitions of constant parameters, types and          */
/* prototypes for VINCI DRS I/O ANSI C functions.                  */
/*                                                                 */
/* Written by Pascal Ballester, Markus Schoeller & Rainer Wilhelm  */
/* ESO, 2000-2001                                                  */
/*******************************************************************/ 

#ifndef _VNDRS_IO_H
#define _VNDRS_IO_H

/* System includes */ 

#include <math.h>
#include <malloc.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

/* Library includes */ 

#include "imaging_data.h"

/* FLUOR file definition */
/* --------------------- */
typedef struct {
  int nb_sources;   /* number of sources */
  int maxstep;      /* Number of steps */
  int *nb_off;
  int *nb_on;
  int *nb_beam_A;
  int *nb_beam_B;
  } FLUOR_file_def;

/* FLUOR file content */
/* ------------------ */
typedef struct {
  int dimension;   /* number of rows */
  int *isValid;
  int *source;
  int *targtype;
  int *step;
  int *on_beam1;
  int *on_beam2;
  int *datai1;
  int *datai2;
  int *photo1;
  int *photo2;
  int *filtered;
  } FLUOR_file;

/* Pointers for FLUOR data structures */
  FLUOR_file_def ffd;
  FLUOR_file     fff;
  
/* Pointers for VINCI data structures */ 
int            vinci_hdu;

/* Prototypes */

void vndrs_calloc_vlti_file(
         FLUOR_file* fff, 
         int dim
         );

void vndrs_free_vlti_file(
         FLUOR_file* fff
         );

void vndrs_calloc_vlti_file_def(
         FLUOR_file_def* ffd, 
         int dim, 
         int maxstep
         );

void vndrs_free_vlti_file_def(
         FLUOR_file_def* ffd
         );

void check_fluor_data (
         fitsfile *fptr, 
         imaging_data_table_def* tabledef, 
         int *ierr, 
         FLUOR_file_def* ffd, 
         FLUOR_file* fff
         );


batch	*vndrs_select_fluor_data(
	long	which,
	long	source_id);


int vndrs_open_fluor_data(
	char	*inpfile); 


batch	*vndrs_open_vinci_data(
	char	*inpfile,
	long	numberOfScansRequested); 


void vndrs_close_fluor_data();

void vndrs_close_vinci_data();

void vndrs_write_waterfall(batch* Batch, char* file);

void vndrs_read_vinci_header(
         char* inpfile, 
         int* obsmode, 
         int* nscan, 
         int* nfram, 
         int* insmode, 
         int* acqmode, 
         double* fringespeed, 
         double* framerate, 
         double* mjdobs
         );

/* Routine to export header data */ 
void vndrs_export_header(
         char* outfile_header, 
         char* infile_on
         ); 

/* Routine to export raw data */ 
void vndrs_export_data  (
         char* outfile_data,   
         batch* Off_source, 
         batch* On_source, 
         batch* Beam_A, 
         batch* Beam_B
         ); 

/* Routine to export processed data */ 
void vndrs_export_proc  (
         char* outfile_data,   
         batch* On_source_processed
         ); 

#endif 


