/******************************************************************************
*******************************************************************************
*				European Southern Observatory
*		 Vlti ViNci Data Reduction Software (VNDRS)
*
* File name:	vndrs_MemoryHandling.c
* Description:	Contains the routines for dynamic memory allocation 
*				and freeing
*
*
* History:		
* 01-May-01		(csabet) Revised
* 2000			Written by Markus Schoeller & Rainer Wilhelm, ESO
*
*******************************************************************************
******************************************************************************/

/******************************************************************************
*	Compiler directives
******************************************************************************/

/******************************************************************************
*	Include files
******************************************************************************/
#include "vndrs.h"
#include "vndrs_Globalvar.h"
/*============================ C O D E    A R E A ===========================*/

/******************************************************************************
*				European Southern Observatory
*		 Vlti ViNci Data Reduction Software (VNDRS)
*
* Module name:	vndrs_initialization.c
* Input/Output:	See function arguments to avoid duplication
* Description:	
*
*
* History:		
* 01-May-01		(csabet) Revised
* 2000			Written by Markus Schoeller & Rainer Wilhelm, ESO
******************************************************************************/
void vndrs_initialization(
			  char	*protocol_mode,
			  char    *file_name)
{
	/*  Local Declarations
	--------------------*/

	/*  Algorithm
	-----------*/

	/*	Set start time (global (!!) variable)
	---------------------------------------*/
	StartTime = time(NULL);


	/*	Initialising (global) 'DataPath' variable (set it to the local directory)
	---------------------------------------------------------------------------*/
	strcpy(DataPath,".");


	/*	Set the protocol mode (global variable)
	-----------------------------------------*/
	strcpy(vndrs_protocol.mode, protocol_mode);    


	/*	Start the runtime protocol
	----------------------------*/
	vndrs_start_runtime_protocol(protocol_mode, file_name);


	/*	Start the memory usage protocol
	---------------------------------*/
	/* vndrs_start_memory_usage_protocol();*/


	return;
}



void vndrs_exit(void)
{
    /* Local variables */ 
    char StartTimeString[MAX_STRING_LENGTH]; 
    char EndTimeString[MAX_STRING_LENGTH];

    /* =========================================================== */
    /* SHOW COMPUTATION TIME                                       */
    /* =========================================================== */
    /* Set end time (global variable) to current time. The start time
       is still in the memory (global variable 'StartTime')  */
    EndTime = time(NULL);

    /* Write the start and the end time to strings */
    strcpy(StartTimeString,ctime(&StartTime));
    strcpy(EndTimeString,  ctime(&EndTime));
    
    /* Print runtime information on the screen */
    /*    fprintf(stderr, "\nVNDRS Runtime Info:\n");
    fprintf(stderr, "-- Started      %s",StartTimeString);
    fprintf(stderr, "-- Finished     %s",EndTimeString);
    fprintf(stderr, "-- Total        %f sec\n",difftime(EndTime, StartTime)); */

    /* ... and in the protocol (if it's on the disk only!) */
    if(strcmp(vndrs_protocol.mode, "Disk")==0)
    {
        sprintf(vndrs_protocol.text, "#\n# VNDRS Runtime Info:\n");             vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Started      %s", StartTimeString); vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Finished     %s", EndTimeString);   vndrs_runtime_protocol();
        sprintf(vndrs_protocol.text, "# Total        %f sec\n",
                difftime(EndTime, StartTime));                               vndrs_runtime_protocol();
    }

    /* =========================================================== */
    /* CLOSE THE MEMORY USAGE PROTOCOL                             */
    /* =========================================================== */
    vndrs_end_memory_usage_protocol();
    
    /* =========================================================== */
    /* CLOSE THE RUNTIME PROTOCOL                                  */
    /* =========================================================== */
    vndrs_end_runtime_protocol();

    /* EXIT TO SHELL */
    exit(0); 
    
    return; 
}

/* ------------------------------------------------------------------- */
/* PROTOCOL ROUTINES                                                   */
/* ------------------------------------------------------------------- */

void vndrs_start_runtime_protocol(char *protocol_mode, char *file_name)
{ 
    /* Local variables */
    time_t CurrentTime;
    char CurrentTimeString[MAX_STRING_LENGTH]; 

    /* GET CURRENT TIME (IN SECONDS) AND WRITE IT THE STRING */
    CurrentTime = time(NULL);
    sprintf(CurrentTimeString, ctime(&CurrentTime));

    if((strcmp(vndrs_protocol.mode,"Nowhere")!=0)
     &&(strcmp(vndrs_protocol.mode,"Screen")!=0)
     &&(strcmp(vndrs_protocol.mode,"Disk")!=0))
      {
	sprintf(protocol_mode, "Disk"); /* default value */
      }
    
    /* (Optionally) open runtime protocol file (ASCII mode) */
    /* ---------------------------------------------------- */
    if(strcmp(vndrs_protocol.mode, "Disk")==0)
    {
        if((vndrs_protocol.file_pointer = fopen(file_name, "w")) == NULL)
        {
            printf("*** ERROR Message ***\n");
            printf("Runtime protocol file can not be opened for writing.\n");
            vndrs_error_alert(); 
        }
        
        /* write protocol header */
        fprintf(vndrs_protocol.file_pointer,"#\n");
        fprintf(vndrs_protocol.file_pointer,"#                   VINCI DATA REDUCTION                   \n");
        fprintf(vndrs_protocol.file_pointer,"#              Final Version 3.3, (2004, 2018)             \n");
        fprintf(vndrs_protocol.file_pointer,"#           Processing date: %s", CurrentTimeString);
	fprintf(vndrs_protocol.file_pointer,"#                                                          \n");
	fprintf(vndrs_protocol.file_pointer,"#                                                          \n");
	fprintf(vndrs_protocol.file_pointer,"# - Software and algorithms by:                            \n");
	fprintf(vndrs_protocol.file_pointer,"#       P. Kervella (ESO), D. Segransan (Obs. de Geneve)   \n");
	fprintf(vndrs_protocol.file_pointer,"#   with contributions by:                                 \n");
	fprintf(vndrs_protocol.file_pointer,"#       M. Schoeller, R. Wilhelm (ESO)                     \n");
	fprintf(vndrs_protocol.file_pointer,"#                                                          \n");
	fprintf(vndrs_protocol.file_pointer,"# - Detailed algorithm described in                        \n");
	fprintf(vndrs_protocol.file_pointer,"# Kervella P., Segransan D., Coude du Foresto V.,          \n");
	fprintf(vndrs_protocol.file_pointer,"# Data Processing Methods for Long-Baseline Interferometry \n");
	fprintf(vndrs_protocol.file_pointer,"# A&A, 425, 1161 (2004) doi: 10.1051/0004-6361:20034562    \n");
	fprintf(vndrs_protocol.file_pointer,"#                                                          \n");
	fprintf(vndrs_protocol.file_pointer,"# - Fourier method by V. Coude du Foresto et al.,          \n");
	fprintf(vndrs_protocol.file_pointer,"# Observatoire de Paris / Jean-Marie Mariotti Center       \n");
	fprintf(vndrs_protocol.file_pointer,"# ref: A&ASS, 121, 379 (1997) doi: 10.1051/aas:1997290     \n");
	fprintf(vndrs_protocol.file_pointer,"#                                                          \n");
	fprintf(vndrs_protocol.file_pointer,"# - Wavelets method developed by D. Segransan,             \n");
	fprintf(vndrs_protocol.file_pointer,"# Observatoire de Geneve / Observatoire de Grenoble        \n");
	fprintf(vndrs_protocol.file_pointer,"# ref: Segransan D., Ph.D. Thesis, Grenoble (2001)         \n");
        fprintf(vndrs_protocol.file_pointer,"#                                                          \n");
        fflush(vndrs_protocol.file_pointer);
    }
    
    /* Screen Info (in any case on screen) */
    /* ----------------------------------- */
    fprintf(stderr, "\n\n");
    fprintf(stderr, "-----------------------------------------------------\n");
    fprintf(stderr, "            VINCI DATA REDUCTION SOFTWARE            \n");
    fprintf(stderr, "            Final Version 3.3, 2004 (2018).          \n");
    fprintf(stderr, "  		                                          \n");
    /* fprintf(stderr, "                                                     \n");
    fprintf(stderr, "                                                     \n");
    fprintf(stderr, "       Written by P. Kervella and D. Segransan       \n");
    fprintf(stderr, "  		                                          \n");*/
    if(strcmp(vndrs_protocol.mode,"Nowhere")==0)
    {
        fprintf(stderr, "   Protocol Mode has been set to 'Nowhere'.          \n");
    }
    else if(strcmp(vndrs_protocol.mode,"Disk")==0)
    {
        fprintf(stderr, "   Protocol Mode has been set to 'Disk'.             \n");
    }
    else
    {
        fprintf(stderr, "   Protocol Mode has been set to 'Disk'.             \n");
    }
    
}


/******************************************************************************
*				European Southern Observatory
*		 Vlti ViNci Data Reduction Software (VNDRS)
*
* Module name:	vndrs_runtime_protocol
* Input/Output:	See function arguments to avoid duplication
* Description:	
*
*
* History:		
* 01-May-01		(csabet) Revised
******************************************************************************/
void vndrs_runtime_protocol(void)
{   
	if (strcmp(vndrs_protocol.mode, "Nowhere")==0)
	{
		/*	The user wants no protocol --> just leave the routine
		-------------------------------------------------------*/    
		return;
	}
	else if (strcmp(vndrs_protocol.mode, "Disk")==0)
	{
		/*	Protocol in File (File Pointer was set appropriately before)
		--------------------------------------------------------------*/
	  fprintf(vndrs_protocol.file_pointer, vndrs_protocol.text);
	  fflush(vndrs_protocol.file_pointer);
	  return;
	}
	/*	else if (strcmp(vndrs_protocol.mode, "Screen")==0)
	  {
	    fprintf(stderr, vndrs_protocol.text);
	    return;
	    }*/
	/* pkervell 15-mar-02: the screen display of the protocol is removed, replaced by "printf" commands */
	
	/*	If the program reaches this point, the protocol mode 
		has not been set to a valid value --> print an error message and exit
		-------------------------------------------------------------------*/
	fprintf(stderr, "*** ERROR Message ***\n");
	fprintf(stderr, "The protocol mode has an invalid value = %s\n", vndrs_protocol.mode);
	fprintf(stderr, "Valid values are 'Nowhere', 'Screen' or 'Disk'.");
	vndrs_error_alert();
	
	return;
}



void vndrs_end_runtime_protocol(void)
{   
    /* If the protocol has been written to disk, --close the protocol file */
    if (strcmp(vndrs_protocol.mode,"Disk")==0)
    {
        fclose(vndrs_protocol.file_pointer);
    }    
    return;
}

/* ------------------------------------------------------------------- */
/* MEMORY LEAKAGE CONTROL                                              */
/* ------------------------------------------------------------------- */

void vndrs_start_memory_usage_protocol(void)
{
    /* Local variables */
    time_t CurrentTime;
    char CurrentTimeString[MAX_STRING_LENGTH]; 
    

    /* GET CURRENT TIME (IN SECONDS) AND WRITE IT THE STRING */
    CurrentTime = time(NULL);
    sprintf(CurrentTimeString, ctime(&CurrentTime));

    /* Open memory usage protocol file */
    /* ------------------------------- */
    if((vndrs_memory.file_pointer = fopen("vndrs_MemoryUsage.txt", "w")) == NULL)
    {
        fprintf(stderr, "*** ERROR Message ***\n");
        fprintf(stderr, "Memory usage protocol file can not be opened for writing.\n");
        vndrs_error_alert(); 
    }
        
    /* write header line */
    /* ----------------- */
    fprintf(vndrs_memory.file_pointer,"%% --------------------------------------------------\n");
    fprintf(vndrs_memory.file_pointer,"%% VINCI DATA REDUCTION SOFTWARE                     \n");
    fprintf(vndrs_memory.file_pointer,"%% Memory Usage Protocol                             \n");
    fprintf(vndrs_memory.file_pointer,"%% Created %s", CurrentTimeString);
    fprintf(vndrs_memory.file_pointer,"%% Threshold = %d bytes\n", MEMORY_PROTOCOL_THRESHOLD);
    fprintf(vndrs_memory.file_pointer,"%% --------------------------------------------------\n\n");
    fprintf(vndrs_memory.file_pointer,"%% TIME [SECONDS]  --  TOTAL MEMORY [BYTES]\n\n");
    
    /* Initialize counters */
    vndrs_memory.calloc_vector_double_calls  = 0;
    vndrs_memory.calloc_vector_complex_calls = 0;
    vndrs_memory.calloc_matrix_double_calls  = 0;
    vndrs_memory.calloc_matrix_complex_calls = 0;
    vndrs_memory.calloc_full_scan_calls      = 0;
    vndrs_memory.calloc_full_pow_calls       = 0;
    vndrs_memory.calloc_batch_calls          = 0;
    vndrs_memory.calloc_powbatch_calls       = 0;

    return;
}

void vndrs_protocol_memory_usage(long MemoryIncrement)
/* a positive value of "MemoryIncrement" means that memory is needed;
   a negative value means that memory has been given free. */
{
    char TextToStore[MAX_STRING_LENGTH];
    double CurrentTime;
    /* current time in seconds */
    
    /* the memory protocol file is open (FILE-pointer "vndrs_memory.file_pointer")          */
    
    /* UPDATE TOTAL MEMORY (GLOBAL VARIABLE)       */
    /* ------------------------------------------- */
    vndrs_memory.currently_allocated += MemoryIncrement;
    
    /* CHECK IF IT IS STILL > 0                    */
    /* ------------------------------------------- */
    if(vndrs_memory.currently_allocated < 0)
    {
        fprintf(stderr, "*** ERROR Message ***\n");
        fprintf(stderr, "Memory leakage ... the total memory has become negative :-(\n");
        fprintf(stderr, "The total allocated memory is now equal to %ld BYTES.\n", 
                vndrs_memory.currently_allocated); /* csabet 07-Aug-2001. Format precision corrected */
        vndrs_error_alert(); 
    }
    
/*    fprintf(stderr, "                                                      Allocated memory: %ld bytes.\n", vndrs_memory.currently_allocated);    added pkervell 04-Aug-02 */
    
    /* UPDATE MEMORY DIFFERENCE (GLOBAL VARIABLE)  */
    /* ------------------------------------------- */
    vndrs_memory.delta += MemoryIncrement;
    
    /* GET CURRENT TIME (IN SECONDS)               */
    /* ------------------------------------------- */
    CurrentTime = difftime(time(NULL), StartTime); /* 'StartTime' is a GLOBAL variable! */
    
    /* WRITE CURRENT MEMORY SIZE TO DISK           */
    /* ------------------------------------------- */
    if(fabs(vndrs_memory.delta) >= MEMORY_PROTOCOL_THRESHOLD)
    {
        /* sprintf(TextToStore, "%7ld  %10ld\n", (long)CurrentTime, vndrs_memory.currently_allocated);
        fprintf(vndrs_memory.file_pointer, TextToStore);*/
        
        /* RESET MEMORY DIFFERENCE (GLOBAL VARIABLE) */
        vndrs_memory.delta = 0;
        
        /*sprintf(TextToStore, "Difference calls to vec_dbl: %10d\n",
                vndrs_memory.calloc_vector_double_calls);
        fprintf(vndrs_memory.file_pointer, TextToStore);
        sprintf(TextToStore, "vec_cplx: %10d\n",
                vndrs_memory.calloc_vector_complex_calls);
        fprintf(vndrs_memory.file_pointer, TextToStore);
        sprintf(TextToStore, "mat_dbl: %10d\n",
                vndrs_memory.calloc_matrix_double_calls);
        fprintf(vndrs_memory.file_pointer, TextToStore);
        sprintf(TextToStore, "mat_cplx: %10d\n",
                vndrs_memory.calloc_matrix_complex_calls);
        fprintf(vndrs_memory.file_pointer, TextToStore);
        sprintf(TextToStore, "full_scan: %10d\n",
                vndrs_memory.calloc_full_scan_calls);
        fprintf(vndrs_memory.file_pointer, TextToStore);
        sprintf(TextToStore, "full_pow: %10d\n",
                vndrs_memory.calloc_full_pow_calls);
        fprintf(vndrs_memory.file_pointer, TextToStore);
        sprintf(TextToStore, "batch: %10d\n",
                vndrs_memory.calloc_batch_calls);
        fprintf(vndrs_memory.file_pointer, TextToStore);
        sprintf(TextToStore, "powbatch: %10d\n",
                vndrs_memory.calloc_powbatch_calls);
        fprintf(vndrs_memory.file_pointer, TextToStore);*/
        
        /* fflush(vndrs_memory.file_pointer);*/
    }
    return;
}


void vndrs_end_memory_usage_protocol(void)
{
    char TextToStore[MAX_STRING_LENGTH];
    double CurrentTime;
    /* current time in seconds */
   
    /* GET CURRENT TIME (IN SECONDS)               */
    /* ------------------------------------------- */
    CurrentTime = difftime(time(NULL),StartTime);
    
    /* Write total memory to disk */
    /* -------------------------- */
    /*  pkervell - 04-Aug-02 commented out to take less space on disk with vndrs_MemoryUsage.txt
    sprintf(TextToStore, "%7ld  %10ld\n", (long)CurrentTime, vndrs_memory.currently_allocated);
    fprintf(vndrs_memory.file_pointer, TextToStore);
    */
    
    
    /* sprintf(TextToStore, "Difference calls to vndrs_calloc_vector_double: %10d\n",
            vndrs_memory.calloc_vector_double_calls);
    fprintf(vndrs_memory.file_pointer, TextToStore);
    sprintf(TextToStore, "Difference calls to vndrs_calloc_vector_complex: %10d\n",
            vndrs_memory.calloc_vector_complex_calls);
    fprintf(vndrs_memory.file_pointer, TextToStore);
    sprintf(TextToStore, "Difference calls to vndrs_calloc_matrix_double: %10d\n",
            vndrs_memory.calloc_matrix_double_calls);
    fprintf(vndrs_memory.file_pointer, TextToStore);
    sprintf(TextToStore, "Difference calls to vndrs_calloc_matrix_complex: %10d\n",
            vndrs_memory.calloc_matrix_complex_calls);
    fprintf(vndrs_memory.file_pointer, TextToStore);
    sprintf(TextToStore, "Difference calls to vndrs_calloc_full_scan: %10d\n",
            vndrs_memory.calloc_full_scan_calls);
    fprintf(vndrs_memory.file_pointer, TextToStore);
    sprintf(TextToStore, "Difference calls to vndrs_calloc_full_pow: %10d\n",
            vndrs_memory.calloc_full_pow_calls);
    fprintf(vndrs_memory.file_pointer, TextToStore);
    sprintf(TextToStore, "Difference calls to vndrs_calloc_batch: %10d\n",
            vndrs_memory.calloc_batch_calls);
    fprintf(vndrs_memory.file_pointer, TextToStore);
    sprintf(TextToStore, "Difference calls to vndrs_calloc_powbatch: %10d\n",
            vndrs_memory.calloc_powbatch_calls);
    fprintf(vndrs_memory.file_pointer, TextToStore);*/
    

    /* close memory usage protocol */
    /* --------------------------- */
    /* fclose(vndrs_memory.file_pointer);*/
    
    /* check if total memory equals zero (should be the case!) */
    /* ------------------------------------------------------- */
    if(vndrs_memory.currently_allocated != 0)
    {
        fprintf(stderr, "*** ERROR: There are %ld BYTES of memory not released.\n", vndrs_memory.currently_allocated);
        /* sprintf(TextToStore, "There are %ld BYTES left. %10d\n", vndrs_memory.currently_allocated);
        fprintf(vndrs_memory.file_pointer, TextToStore);*/
        vndrs_error_alert();
    }

    return;
}

/* ------------------------------------------------------------------- */
/* MEMORY ALLOCATION ROUTINES                                          */
/* ------------------------------------------------------------------- */

/* Double vector */
/* ------------- */
vector_double *vndrs_calloc_vector_double(
                                          /* --------------- */
                                          /* input variables */
                                          /* --------------- */
                                          long dimension)
/* Allocates memory for a double vector of size 'dimension' */
{
    /* local variables */
    vector_double *vector;
    long MemoryIncrement;

    /*fprintf(stderr, "vndrs_calloc_vector_double: %d\n", dimension);*/
    
    /* Compute required amount of memory */
    MemoryIncrement = dimension*sizeof(double);

    if((vector = (vector_double*)calloc(1, sizeof(vector_double)))==NULL)
    {
        fprintf(stderr, "Not enough memory for allocation of a double vector.\n");
        vndrs_error_alert();
    }
    if((vector->data = (double*)calloc(dimension, sizeof(double)))==NULL)
    {
        fprintf(stderr, "Not enough memory for allocation of a double vector.\n");
        fprintf(stderr, "You tried to allocate memory for %ld elements of type\n", dimension);
			/* csabet 07-Aug-2001. Format precision corrected */
        fprintf(stderr, "'double'. This corresponds to %ld BYTES.\n", MemoryIncrement);
			/* csabet 07-Aug-2001. Format precision corrected */
        vndrs_error_alert();
    }
    /* Set dimension of vector */
    vector->dimension = dimension;

    /* write to memory usage protocol */
    vndrs_protocol_memory_usage(MemoryIncrement);

    vndrs_memory.calloc_vector_double_calls++;

    return vector;
}

void vndrs_free_vector_double( 
                              /* --------------- */
                              /* input variables */
                              /* --------------- */
                              vector_double *vector
                             )
{
    /* Local variables */
    long MemoryIncrement;
    
    /*fprintf(stderr, "vndrs_free_vector_double: %d\n", vector->dimension);*/
    
    /* compute the amount of memory in bytes  */
    MemoryIncrement = -(vector->dimension)*sizeof(double);
    
    /* Free the memory */
    free(vector->data);
    free(vector);

    /* write to memory usage protocol */
    vndrs_protocol_memory_usage(MemoryIncrement);

    vndrs_memory.calloc_vector_double_calls--;

    return;
}


/* Complex vector */
/* -------------- */
vector_complex *vndrs_calloc_vector_complex(
                                            /* --------------- */
                                            /* input variables */
                                            /* --------------- */
                                            long dimension
                                           )
/* Allocates memory for a complex vector of size 'dimension' */
{
    /* local variable */
    vector_complex *vector;
    long MemoryIncrement;

    /*fprintf(stderr, "vndrs_calloc_vector_complex: %d\n", dimension);*/
    
    /* Compute required amount of memory */
    MemoryIncrement = dimension*sizeof(complex);

    if((vector = (vector_complex*)calloc(1, sizeof(vector_complex)))==NULL)
    {
        fprintf(stderr, "Not enough memory for allocation of a complex vector.\n");
        vndrs_error_alert();
    }
    if((vector->data = (complex*)calloc(dimension, sizeof(complex)))==NULL)
    {
        fprintf(stderr, "Not enough memory for allocation of a complex vector.\n");
        fprintf(stderr, "You tried to allocate memory for %ld elements of type\n", dimension);
			/* csabet 07-Aug-2001. Format precision corrected */
        fprintf(stderr, "'double'. This corresponds to %ld BYTES.\n", MemoryIncrement);
			/* csabet 07-Aug-2001. Format precision corrected */
        vndrs_error_alert();
    }
    /* Set dimension of vector */
    vector->dimension = dimension;

    /* write to memory usage protocol */
    vndrs_protocol_memory_usage(MemoryIncrement);

    vndrs_memory.calloc_vector_complex_calls++;

    return vector;
}

void vndrs_free_vector_complex(
                               /* --------------- */
                               /* input variables */
                               /* --------------- */
                               vector_complex *vector
                              )
{
    /* Local variables */
    long MemoryIncrement;

    /*fprintf(stderr, "vndrs_free_vector_complex: %d\n", vector->dimension);*/
    
    /* compute the amount of memory in bytes  */
    MemoryIncrement = -(vector->dimension)*sizeof(complex);
    
    /* Free the memory */    
    free(vector->data);
    free(vector);

    /* write to memory usage protocol */
    vndrs_protocol_memory_usage(MemoryIncrement);   
    
    vndrs_memory.calloc_vector_complex_calls--;

    return;
}


/******************************************************************************
*				European Southern Observatory
*		 Vlti ViNci Data Reduction Software (VNDRS)
*
* Module name:	vndrs_calloc_matrix_double
* Input/Output:	See function arguments to avoid duplication
* Description:	Allocates memory for a double matrix of size 
*				'dimension_1' x 'dimension_2
*
*
* History:		
* 01-Jul-01		(csabet) Revised
******************************************************************************/
matrix_double *vndrs_calloc_matrix_double(
	long	dimension_1,	/*	In:	Input dimension */ 
	long	dimension_2)	/*	In:	Input dimension */ 
{

	/*  Local Declarations
	--------------------*/
	matrix_double	*matrix;
	long			MemoryIncrement;


	/*  Algorithm
	-----------*/
	/*fprintf(stderr, "vndrs_calloc_matrix_double: %d,%d\n", dimension_1, dimension_2);*/
    
    
	/*	Compute required amount of memory
	-----------------------------------*/
	MemoryIncrement = dimension_1*dimension_2*sizeof(double);

	if((matrix = (matrix_double*)calloc(1, sizeof(matrix_double)))==NULL)
	{
		fprintf(stderr, "Not enough memory for allocation of a double matrix.\n");
		vndrs_error_alert();
	}
	if((matrix->data = (double*)calloc(dimension_1*dimension_2, sizeof(double)))==NULL)
	{
		fprintf(stderr, "Not enough memory for allocation of a double matrix.\n");
		fprintf(stderr, "You tried to allocate memory for a %ld x %ld matrix.\n",
			dimension_1, dimension_2); /* csabet 07-Aug-2001. Format precision corrected */
		fprintf(stderr, "This corresponds to %ld BYTES.\n", MemoryIncrement);
			/* csabet 07-Aug-2001. Format precision corrected */
		vndrs_error_alert();
	}
    
	/*	Set dimensions of matrix
	--------------------------*/
	matrix->dimension_1 = dimension_1;
	matrix->dimension_2 = dimension_2;


	/*	Write to memory usage protocol
	--------------------------------*/
	vndrs_protocol_memory_usage(MemoryIncrement);
	vndrs_memory.calloc_matrix_double_calls++;

	/*	Return required matrix
	------------------------*/
	return (matrix);
}



void vndrs_free_matrix_double(
                              /* --------------- */
                              /* input variables */
                              /* --------------- */
                              matrix_double *matrix
                             )
{
    /* Local variables */
    long MemoryIncrement;

    /*fprintf(stderr, "vndrs_free_matrix_double: %d,%d\n", matrix->dimension_1, matrix->dimension_2);*/
    
    /* compute the amount of memory in bytes  */
    MemoryIncrement = -(matrix->dimension_1)*(matrix->dimension_2)*sizeof(double);    

    /* Free the memory */
    free(matrix->data);
    free(matrix);

    /* write to memory usage protocol */
    vndrs_protocol_memory_usage(MemoryIncrement);   

    vndrs_memory.calloc_matrix_double_calls--;

    return;
}


/* Complex matrix */
/* -------------- */

matrix_complex *vndrs_calloc_matrix_complex(
                                            /* --------------- */
                                            /* input variables */
                                            /* --------------- */
                                            long dimension_1, long dimension_2
                                           )
/* Allocates memory for a complex matrix of size 'dimension_1' x 'dimension_2' */
{
    /* local variable */
    matrix_complex *matrix;
    long MemoryIncrement;

    /*fprintf(stderr, "vndrs_calloc_matrix_complex: %d,%d\n", dimension_1, dimension_2);*/
    
    /* Compute required amount of memory */
    MemoryIncrement = dimension_1*dimension_2*sizeof(complex);

    if((matrix = (matrix_complex*)calloc(1, sizeof(matrix_complex)))==NULL)
    {
        fprintf(stderr, "Not enough memory for allocation of a complex matrix.\n");
        vndrs_error_alert();
    }
    if((matrix->data = (complex*)calloc(dimension_1*dimension_2, sizeof(complex)))==NULL)
    {
        fprintf(stderr, "Not enough memory for allocation of a complex matrix.\n");
        fprintf(stderr, "You tried to allocate memory for a %ld x %ld matrix.\n", dimension_1, dimension_2);
			/* csabet 07-Aug-2001. Format precision corrected */
        fprintf(stderr, "This corresponds to %ld BYTES.\n", MemoryIncrement);
			/* csabet 07-Aug-2001. Format precision corrected */
        vndrs_error_alert();
    }
    /* Set dimensions of matrix */
    matrix->dimension_1 = dimension_1;
    matrix->dimension_2 = dimension_2;

    /* write to memory usage protocol */
    vndrs_protocol_memory_usage(MemoryIncrement);

    vndrs_memory.calloc_matrix_complex_calls++;

    return matrix;
}

void vndrs_free_matrix_complex(
                               /* --------------- */
                               /* input variables */
                               /* --------------- */
                               matrix_complex *matrix
                              )
{
    /* Local variables */
    long MemoryIncrement;

    /*fprintf(stderr, "vndrs_free_matrix_complex: %d,%d\n", matrix->dimension_1, matrix->dimension_2);*/
    /* compute the amount of memory in bytes  */
    MemoryIncrement = -(matrix->dimension_1)*(matrix->dimension_2)*sizeof(complex);
    
    /* Free the memory */
    free(matrix->data);
    free(matrix);

    /* write to memory usage protocol */
    vndrs_protocol_memory_usage(MemoryIncrement);

    vndrs_memory.calloc_matrix_complex_calls--;

    return;
}



/* Full Scan      */
/* -------------- */

/* The 'full scan' consists of 4 vectors, each of type 'individual scan'
   (= vector_double). The data type is defined as follows (see 'vndrs.h'):
            typedef struct
            {
                individual_scan *I1;
                individual_scan *I2;
                individual_scan *PA;
                individual_scan *PB;
            } full_scan;
*/

full_scan *vndrs_calloc_full_scan(
                                  /* --------------- */
                                  /* input variables */
                                  /* --------------- */
                                  const long dimension_individual_scan
                                 )
/* Allocates memory for a full scan (which consists of four individual scans
   I1, I2, PA and PB, --each having the size 'dimension'. */
{
    /* local variable */
    full_scan *FullScan;
    
    /*fprintf(stderr, "vndrs_calloc_full_scan: %d\n", dimension_individual_scan);*/
    
    /* Allocate memory for the full scan pointer */
    if((FullScan = (full_scan*)calloc(1, sizeof(full_scan)))==NULL)
    {
        fprintf(stderr, "Not enough memory for allocation of a full scan\n");
        fprintf(stderr, "(= 4 individual scans I1, I2, PA, PB)\n");
        vndrs_error_alert();
    }
    
    /* Allocate memory for the 4 individual scans (each is a double vector) */
    FullScan->I1 = vndrs_calloc_vector_double(dimension_individual_scan);
    FullScan->I2 = vndrs_calloc_vector_double(dimension_individual_scan);
    FullScan->PA = vndrs_calloc_vector_double(dimension_individual_scan);
    FullScan->PB = vndrs_calloc_vector_double(dimension_individual_scan);
    FullScan->QL = vndrs_calloc_vector_double(dimension_individual_scan);
   
    vndrs_memory.calloc_full_scan_calls++;

    return FullScan;
}

void vndrs_free_full_scan(
                          /* --------------- */
                          /* input variables */
                          /* --------------- */
                          full_scan *FullScan
                         )
/* Frees memory for a full scan which consists of four individual scans I1, I2, PA and PB */
{
    /*fprintf(stderr, "vndrs_free_full_scan: %d\n", FullScan->I1->dimension);*/
    vndrs_free_vector_double(FullScan->I1);
    vndrs_free_vector_double(FullScan->I2);
    vndrs_free_vector_double(FullScan->PA);
    vndrs_free_vector_double(FullScan->PB);
    vndrs_free_vector_double(FullScan->QL);

    free(FullScan);

    vndrs_memory.calloc_full_scan_calls--;

    return;
}

/* Full powerspectrum (consists of 2 individual powerspectra I1_pow, I2_pow) */
/* ------------------------------------------------------------------------- */
full_pow *vndrs_calloc_full_pow(
                                 /* --------------- */
                                 /* input variables */
                                 /* --------------- */
                                 const long dimension_individual_spectrum
                                )
/* Allocates memory for a full power spectrum (which consists of two individual spectra
   I1_pow, I2_pow --each having the size 'dimension_individual_spectrum'. */
{
    /* local variable */
    full_pow *FullPowerSpectrum;
    
    /*fprintf(stderr, "vndrs_calloc_full_pow: %d\n", dimension_individual_spectrum);*/
    
    /* Allocate memory for the full pow pointer */
    if((FullPowerSpectrum = (full_pow*)calloc(1, sizeof(full_pow)))==NULL)
    {
        fprintf(stderr, "Not enough memory for allocation of a full power spectrum\n");
        fprintf(stderr, "(= 2 individual spectra I1_pow, I2_pow)\n");
        vndrs_error_alert();
    }
    
    /* Allocate memory for the 2 individual spectra (each is a double vector) */
    FullPowerSpectrum->I1_pow = vndrs_calloc_vector_double(dimension_individual_spectrum);
    FullPowerSpectrum->I2_pow = vndrs_calloc_vector_double(dimension_individual_spectrum);
   
    vndrs_memory.calloc_full_pow_calls++;

    return FullPowerSpectrum;
}


void vndrs_free_full_pow(
                         /* --------------- */
                         /* input variables */
                         /* --------------- */
                         full_pow *FullPowerSpectrum
                        )
/* Frees memory for a full powerspectrum which consists of two individual spectra I1_pow, I2_pow */
{
    /*fprintf(stderr, "vndrs_free_full_pow: %d\n", FullPowerSpectrum->I1_pow->dimension);*/
    vndrs_free_vector_double(FullPowerSpectrum->I1_pow);
    vndrs_free_vector_double(FullPowerSpectrum->I2_pow);

    free(FullPowerSpectrum);

    vndrs_memory.calloc_full_pow_calls--;

    return;
}
	    

/* Batch (a 1-D field of full scans) */
/* --------------------------------- */

/* A 'batch' is a 1-D array of 'full scans' (see above). Each full scan consists of
   the four individual scans I1, I2, PA and PB. The 'batch' type is defined as follows:
   (see 'vndrs.h'):

               typedef struct
               {
                   long number_of_scans;
                   full_scan **fullscans;
               } batch;
*/

batch *vndrs_calloc_batch(
                          /* --------------- */
                          /* input variables */
                          /* --------------- */
                          const long number_of_full_scans, const long dimension_individual_scan
                         )
/* Allocates memory for a batch of full scans. The number of full scans included in this batch
   is passed as input parameter 'number_of_full_scans'. Each of the full scans consists of four
   individual scans (I1, I2, PA, PB). The size of each of this four individual scans is passed
   as input parameter 'dimension_individual_scan'. */
{
    /* local variable */
    batch *Batch;
    long i;

    /*fprintf(stderr, "vndrs_calloc_batch: %d,%d\n", number_of_full_scans, dimension_individual_scan);*/
    /* Allocate memory for the batch pointer */
    if((Batch = (batch*)calloc(1, sizeof(batch)))==NULL)
    {
        fprintf(stderr, "Not enough memory for allocation of a batch of full scans\n");
        vndrs_error_alert();
    }

    /* Allocate memory for the array of pointers to the scan data */
    if((Batch->scan_data = (scan_data*) calloc(number_of_full_scans, sizeof(scan_data)))==NULL)
    {
        fprintf(stderr, "Not enough memory for a batch of scan data.\n");
        fprintf(stderr, "You tried to allocate memory for %ld scans.\n", number_of_full_scans);
			/* csabet 07-Aug-2001. Format precision corrected */
        fprintf(stderr, "This corresponds to %ld bytes.\n", number_of_full_scans*sizeof(scan_data));
			/* csabet 07-Aug-2001. Format precision corrected */
        vndrs_error_alert();   
    }


    /* Allocate memory for the array of pointers to the full scans */
    if((Batch->fullscans = (full_scan**) calloc(number_of_full_scans, sizeof(full_scan*)))==NULL)
    {
        fprintf(stderr, "Not enough memory for a batch of full scans.\n");
        fprintf(stderr, "You tried to allocate memory for %ld full scans.\n", number_of_full_scans);
			/* csabet 07-Aug-2001. Format precision corrected */
        fprintf(stderr, "This corresponds to %ld bytes.\n", number_of_full_scans*sizeof(full_scan));
			/* csabet 07-Aug-2001. Format precision corrected */
        vndrs_error_alert();   
    }

    /* Allocate memory for the full scans */
    for(i=0; i<number_of_full_scans; i++)
    {
        Batch->fullscans[i] = vndrs_calloc_full_scan(dimension_individual_scan);
    }

    Batch->number_of_scans = number_of_full_scans;

    vndrs_memory.calloc_batch_calls++;

    return Batch;
}

void vndrs_free_batch(
                      /* --------------- */
                      /* input variables */
                      /* --------------- */
                      batch *Batch
                     )
/* Frees memory for a batch of full scans */
{
    /* long i; */
    unsigned int i;	/* long not required here csabet 26.Jun.01 */
    
    /*fprintf(stderr, "vndrs_free_batch: %d,%d\n", Batch->number_of_scans, Batch->fullscans[0]->I1->dimension);*/
    
    /* Free memory for full scans of the batch */ 
    for(i=0; i < Batch->number_of_scans; i++)
    {
        vndrs_free_full_scan(Batch->fullscans[i]);
    } 
    
    /* Free memory for the array of pointers pointing to the scan data */
    free(Batch->scan_data);

    /* Free memory for the array of pointers pointing to the full scans */
    free(Batch->fullscans);

    /* Free memory for batch pointer */
    free(Batch);

    vndrs_memory.calloc_batch_calls--;

    return;
}


/* Batch of full powerspectra (a 1-D field of full powerspectra) */
/* ------------------------------------------------------------- */

/* A 'powbatch' is a 1-D array of 'full powerspectra' (see above). Each full powerspectrum 
   consists of 2 individual spectra I1_pow, I2_pow. The batch of full powerspectra is
   defined as: (see "vndrs.h"):

       typedef struct
       {
           size_t number_of_pows;
           full_pow **fullpows;
       } powbatch;
   
   with each full powerspectrum being defined as an ensemble of 2 individual powerspectra: 

       typedef struct
       {
           power_spectrum *I1_pow;
           power_spectrum *I2_pow;
       } full_pow;

*/


powbatch *vndrs_calloc_powbatch(
                                /* --------------- */
                                /* input variables */
                                /* --------------- */
                                const long number_of_full_spectra,
                                const long dimension_individual_spectrum
                               )
/* Allocates memory for a batch of full powerspectra. The number of full powerspectra included
   in this batch is passed as input parameter 'number_of_full_spectra'. Each of the full spectra
   consists of 2 individual spectra (I1_pow, I2_pow). The size of each of this 2 individual spectra
   is passed as input parameter 'dimension_individual_spectrum'. */
{
    /* local variable */
    powbatch *Batch;
    long i;

    /*fprintf(stderr, "vndrs_calloc_powbatch: %d,%d\n", number_of_full_spectra, dimension_individual_spectrum);*/
    
    /* Allocate memory for the batch pointer */
    if((Batch = (powbatch*)calloc(1, sizeof(powbatch)))==NULL)
    {
        fprintf(stderr, "Not enough memory for allocation of a batch of full powerspectra.\n");
        vndrs_error_alert();
    }

    /* Allocate memory for the array of pointers to the full spectra */
    if((Batch->fullpows = (full_pow**) calloc(number_of_full_spectra, sizeof(full_pow*)))==NULL)
    {
        fprintf(stderr, "Not enough memory for a batch of full spectra.\n");
        fprintf(stderr, "You tried to allocate memory for %ld full spectra.\n", number_of_full_spectra);
			/* csabet 07-Aug-2001. Format precision corrected */
        fprintf(stderr, "This corresponds to %ld bytes.\n", number_of_full_spectra*sizeof(full_pow));
			/* csabet 07-Aug-2001. Format precision corrected */
        vndrs_error_alert();   
    }

    /* Allocate memory for the full spectra */
    for(i=0; i<number_of_full_spectra; i++)
    {
        Batch->fullpows[i] = vndrs_calloc_full_pow(dimension_individual_spectrum);
    }

    Batch->number_of_pows = number_of_full_spectra;

    vndrs_memory.calloc_powbatch_calls++;

    return Batch;
}

void vndrs_free_powbatch(
                         /* --------------- */
                         /* input variables */
                         /* --------------- */
                         powbatch *Batch
                        )
/* Frees memory for a batch of full powerspectra */
{
    /* long i; */
    unsigned int i; /* long is not required here csabet 26.Jun.01 */
    
    /*fprintf(stderr, "vndrs_free_powbatch: %d,%d\n", Batch->number_of_pows, Batch->fullpows[0]->I1_pow->dimension);*/
    
    /* Free memory for full spectra of the batch */ 
    for(i=0; i < Batch->number_of_pows; i++)
    {
        vndrs_free_full_pow(Batch->fullpows[i]);
    } 
    
    /* Free memory for the array of pointers pointing to the full spectra */
    free(Batch->fullpows);

    /* Free memory for batch pointer */
    free(Batch);

    vndrs_memory.calloc_powbatch_calls--;

    return;
}
