/* $Id: vndrs_OPD.c,v 1.2 2001/08/14 10:10:42 csabet Exp $ */
/**********************************************************
***********************************************************
*				European Southern Observatory
*		 Vlti ViNci Data Reduction Software (VNDRS)
*
* Module name:	
* Description:	
*
*
* History:
* Mar-01		(bcotton) Written		
* 01-May-01		(csabet) Revised
***********************************************************
**********************************************************/

/**********************************************************
*	Compiler directives
**********************************************************/
#define DEBUG_SESSIONx		/* csabet 27.Jun.01 */

/**********************************************************
*	Include files
**********************************************************/
#include "vndrs_OPD.h"
#include "VLTIfileUtil.h"	/* csabet 27.Jun.01 */

/**********************************************************
*	Constant definitions
**********************************************************/

/**********************************************************
*	Function prototypes
**********************************************************/
/* internal prototypes */
void vndrs_OPD_get_info (char *infile,
			 double *time_step, double *delay_step,
			 double *freq, double *minfreq,
			 double *maxfreq, int *scan_mode, 
                         double *speed, double *rate, 
			 char *name, char *Date_Obs, char *Observer,
			 char *band, char *station1,  char *station2, 
			 char *telescope, char *OBname, 
                         double *RA, double *Dec, 
                         char *equinox, char *Catg, char *ReadMode,
			 double *t1x, double *t1y, double *t1z,
			 double *t2x, double *t2y, double *t2z,
			 double *t3x, double *t3y, double *t3z,
			 int *ierr);

void vndrs_get_cp_mom (fourier_spectrum *Spectrum,  int hi, int lo, 
		       double ScanSpeed, double FrameRate, int nFrames, 
		       double *mom);

void vndrs_get_scan_photometry (batch *on_raw,  batch *off_raw, 
				 double *photom);

double vndrs_get_OPD (fourier_spectrum *Spectrum, double delta_f,
		      int hi, int lo, float centerp, float *snr);

double vndrs_get_moment(power_spectrum *Amp, int hi, int lo, 
			float centerp, float *snr);


/* public functions */
opd_data* vndrs_calloc_opd_data (long number)
/*----------------------------------------------------------------------- */
/*   Allocate an opd_data structure                                       */
/*   Inputs:                                                              */
/*      number    number of elements in structure                         */
/*   Returns:                                                             */
/*      an opd_data structure pointer, NULL on error                      */
/*----------------------------------------------------------------------- */
{
  opd_data* out;
  long    i, size, MemoryIncrement;
  if (number<=0) return NULL;  /* sanity check */

  /* basic structure */
  size = sizeof (opd_data);
  if((out = (opd_data*)calloc(1, size))==NULL)
    {        
      fprintf(stderr, "Not enough memory for allocation of a opd_data.\n");
      vndrs_error_alert();
    }
  MemoryIncrement = size;

  /* initialize values */
  out->number = number;
  out->time = NULL;
  out->OPDresid = NULL;
  out->OPDdot = NULL;
  out->photom = NULL;
  out->corpower = NULL;
  out->mean_OPD = 0.0;
  out->mean_OPDdot = 0.0;
  out->mean_ph[0] = 0.0;
  out->mean_ph[1] = 0.0;
  out->mean_cp[0] = 0.0;
  out->mean_cp[1] = 0.0;
  out->mean_cp[2] = 0.0;
  out->sigma_OPD = 0.0;
  out->sigma_OPDdot = 0.0;
  out->sigma_ph[0] = 0.0;
  out->sigma_ph[1] = 0.0;
  out->sigma_cp[0] = 0.0;
  out->sigma_cp[1] = 0.0;
  out->sigma_cp[2] = 0.0;
  out->tau_0 = 0.0;
  out->RA = 0.0;
  out->Dec = 0.0;
  for (i=0; i<50; i++) out->name[i] = 0;
  strcpy (out->name,"not set ");
  for (i=0; i<50; i++) out->Date_Obs[i] = 0;
  for (i=0; i<50; i++) out->Observer[i] = 0;
  for (i=0; i<10; i++) out->band[i] = 0;
  for (i=0; i<10; i++) out->station1[i] = 0;
  for (i=0; i<10; i++) out->station2[i] = 0;
  for (i=0; i<20; i++) out->telescope[i] = 0;
  for (i=0; i<50; i++) out->OBname[i] = 0;
  for (i=0; i<10; i++) out->equinox[i] = 0;
  for (i=0; i<20; i++) out->Catg[i] = 0;
  for (i=0; i<20; i++) out->ReadMode[i] = 0;
  /* allocate array elements */
  size = number * sizeof (double);

  /* time */
  if((out->time = (double*)calloc(1, size))==NULL)
    {        
      fprintf(stderr, "Not enough memory for allocation of opd_data.time\n");
      vndrs_error_alert();
    }
  MemoryIncrement += size;

  /* OPDresid */
  if((out->OPDresid = (double*)calloc(1, size))==NULL)
    {        
      fprintf(stderr, "Not enough memory for allocation of opd_data.OPDresid\n");
      vndrs_error_alert();
    }
  MemoryIncrement += size;

  /* OPDdot */
  if((out->OPDdot = (double*)calloc(1, size))==NULL)
    {        
      fprintf(stderr, "Not enough memory for allocation of opd_data.OPDdot\n");
      vndrs_error_alert();
    }
  MemoryIncrement += size;

  /* OPDdotdot */
  if((out->OPDdotdot = (double*)calloc(1, size))==NULL)
    {        
      fprintf(stderr, "Not enough memory for allocation of opd_data.OPDdotdot\n");
      vndrs_error_alert();
    }
  MemoryIncrement += size;

  /* Weight */
  if((out->weight = (double*)calloc(1, size))==NULL)
    {        
      fprintf(stderr, "Not enough memory for allocation of opd_data.weight\n");
      vndrs_error_alert();
    }
  MemoryIncrement += size;

  /* photom */
  if((out->photom = (double*)calloc(1, 2*size))==NULL)
    {        
      fprintf(stderr, "Not enough memory for allocation of opd_data.photom\n");
      vndrs_error_alert();
    }
  MemoryIncrement += 2*size;

  /* photom */
  if((out->corpower = (double*)calloc(1, 3*size))==NULL)
    {        
      fprintf(stderr, "Not enough memory for allocation of opd_data.corpower\n");
      vndrs_error_alert();
    }
  MemoryIncrement += 3*size;

  /* write to memory usage protocol */
  vndrs_protocol_memory_usage(MemoryIncrement);

  return out;
} /* end  vndrs_calloc_opd_data */


void vndrs_free_opd_data(opd_data* data)
/*----------------------------------------------------------------------- */
/*   Free an opd_data structure                                           */
/*   Inputs:                                                              */
/*      data    the structure to be deallocated                           */
/*----------------------------------------------------------------------- */
{
/*	long    size, MemoryIncrement; */
	long    size, MemoryIncrement = 0; /* csabet 09-Aug-2001 */

  if (!data) return; /* check pointer validity */

  /* free array elements */
  size = data->number * sizeof(double);
  if (data->time) {
    free (data->time);
    MemoryIncrement -= size;
  }

  if (data->OPDresid) {
    free (data->OPDresid);
    MemoryIncrement -= size;
  }

  if (data->OPDdot) {
    free (data->OPDdot);
    MemoryIncrement -= size;
  }

  if (data->OPDdotdot) {
    free (data->OPDdotdot);
    MemoryIncrement -= size;
  }
  if (data->weight) {
    free (data->weight);
    MemoryIncrement -= size;
  }
  if (data->photom) {
    free (data->photom);
    MemoryIncrement -= 2*size;
  }
  if (data->corpower) {
    free (data->corpower);
    MemoryIncrement -= 3*size;
  }
  /* free basic structure */
  free (data);
  MemoryIncrement -= sizeof (opd_data);

  /* write to memory usage protocol */
  vndrs_protocol_memory_usage(MemoryIncrement);

} /* end vndrs_free_opd_data */




void vndrs_OPD_fit (char *infile, batch *on_raw,  batch *on_batch, 
		    batch *off_raw, opd_data** opd,  int *ierr)
/*----------------------------------------------------------------------- */
/*   Estimate OPD residual and first time derivative.                     */
/*   OPD residual is estimated from the slope of the phase across the     */
/*   bandpass, the time derivative is estimated from the centroid of the  */
/*   power spectrum.  In both cases, the results are adjusted that the    */
/*   batch average is zero.                                               */
/*   OPDs are corrected for the VINCI tracking delays                     */
/*   values from input file header:                                       */
/*  HIERARCH ESO DET QL FRAMERATE= 802.89   / frame rate (Hz)             */
/*  HIERARCH ESO DET QL FRINGESPD= 0.000351 / fringe speed (m/s)          */
/*  HIERARCH ESO DET QL MAXWAVEK = 2.5E-06  / max wave length K           */
/*  HIERARCH ESO DET QL MINWAVEK = 1.95E-06 / min wave length K           */
/*  HIERARCH ESO DET DACQ MODE   = 1 / 0:No wave 1:Rising + Falling...    */
/*      maxfreq     highest frequency of bandpass in Hz                   */
/*   Inputs:                                                              */
/*      infile    input file with header keywords                         */
/*      on_raw    on source raw batch to be evaluated                     */
/*      on-batch  on source processed batch to be evaluated               */
/*      off_raw   off source raw batch to be evaluated                    */
/*   Output:                                                              */
/*      opd       opd_data structure giving the results                   */
/*                This structure is allocated and the memory must be      */
/*                deallocated using vndrs_free_opd_data.                  */
/*                OPDs in meters and rates in m/s                         */
/*      ierr      Error code: 0 => ok, checked on input                   */
/* Usage note:                                                            */
/*  The vndrs implementation of FFTW is rather inefficient as it does     */
/*  not support real to half-plane complex FFTs which is all that is      */
/*  needed here but does a full complex to complex transform.             */
/*  It also doesn't appear to use the multi threaded features of FFTW.    */
/*----------------------------------------------------------------------- */
{
	unsigned int	i;	/* csabet 27.Jun.01 */
  int hi, lo, nfreq, ndelay, scan_mode;
  double time_step, delay_step, freq, minfreq, maxfreq, delta_f;
  double sumopd, sumopd2, sumopddot, sumopddot2, dOPDdt;
  double summu12, summu22, sum2mu12, sum2mu22;
  double rate, speed, dummy; /* dummy added 14-Apr-02 pkervell */
  double summom0, sum2mom0, summom1, sum2mom1, summom2, sum2mom2;
  double sumph1, sum2ph1, sumph2, sum2ph2;
  double RA, Dec;
  float centerp, mean_mom, snr;
  long   countopd, countopddot, countscan, nelem;
  fourier_spectrum *Spectrum;
  power_spectrum *Amp;
  char name[100];

  if (*ierr!=0) return; /* existing error */

  /* allocate opd_data structure needed */
  *opd = vndrs_calloc_opd_data(on_raw->number_of_scans);

  /* get weights - ignore scans with no solutions */
  for (i=0; i<on_raw->number_of_scans; i++) 
  {
    if (fabs (on_batch->scan_data[i].mu1_squared+
	      on_batch->scan_data[i].mu2_squared) > 0.001)
      (*opd)->weight[i] = 1.0; /* good */
    else
      (*opd)->weight[i] = 0.0; /* bad */
  }

  /* get scan average photometric values */
  vndrs_get_scan_photometry (on_raw, off_raw, &((*opd)->photom[0]));

  /* copy/average times */
  countopd = 0; sumopd = 0;
  for (i=0; i<on_raw->number_of_scans; i++) {
    if ((*opd)->weight[i]>0.01) {
      (*opd)->time[i] = on_raw->scan_data[i].time;
      countopd++;
      sumopd += (*opd)->time[i];
    }
  }
  if (countopd>0) {
    (*opd)->avg_time = sumopd/countopd;
    (*opd)->time_interv = 
      on_raw->scan_data[on_raw->number_of_scans-1].time - 
      on_raw->scan_data[0].time;
  } else {
    (*opd)->avg_time = 0.0;
    (*opd)->time_interv = 0.0;
  }

  /* get info needed from file header */
  for (i=0; i<40; i++) name[i] = 0;
  vndrs_OPD_get_info (infile, &time_step, &delay_step, &freq,
		      &minfreq, &maxfreq, &scan_mode, &speed, &rate, 
		      name, (*opd)->Date_Obs, (*opd)->Observer, 
		      (*opd)->band, (*opd)->station1, (*opd)->station2, 
		      (*opd)->telescope, (*opd)->OBname, 
		      &RA, &Dec, 
		      (*opd)->equinox, (*opd)->Catg, (*opd)->ReadMode,
		      &dummy,&dummy,&dummy,&dummy,&dummy,&dummy,&dummy,
		      &dummy,&dummy,
		      ierr);
  if (*ierr!=0) return; /* error? */

/*  (*opd)->fringerate = rate / 5.0; */ /* fringe rate for report */
	 /* Fringe rate for report
	 -----------------------*/
	(*opd)->fringerate = (float) (rate / 5.0);	/* csabet 27.Jun.01 */

  strcpy ((*opd)->name, name);
  (*opd)->RA = RA;
  (*opd)->Dec = Dec;

  /* compute frequency step in spectrum */
  nelem = on_raw->fullscans[0]->QL->dimension;
  nfreq = nelem/2;
  ndelay = nfreq * 2;
  delta_f = CLIGHT / (delay_step * (ndelay-1));

  /* debug */
  /*fprintf (stdout,"delta_f %lg delay_step %lg ndelay %d\n", 
	   delta_f, delay_step*1.0e6, ndelay);
  fprintf (stdout,"minfreq %lg maxfreq %lg freq %lg\n", 
           minfreq, maxfreq,freq);*/

  /* low and high ends of the band */
  /* hi = 1.5 + maxfreq / delta_f; */
	hi = (int) (1.5 + maxfreq / delta_f);	/* csabet 27.Jun.01 */

  if (hi>nfreq) hi = nfreq;
  /* lo = (minfreq / delta_f) - 0.5; */
	lo = (int) ((minfreq / delta_f) - 0.5);	/* csabet 27.Jun.01 */

  if (lo<1) lo = 1;
  /* centerp = freq / delta_f;  */ /* center pixel */
	/* center pixel */
	centerp = (float) (freq / delta_f);	/* csabet 27.Jun.01 */

  /* create structures */
  Spectrum = vndrs_calloc_vector_complex(nelem);
  Amp = vndrs_calloc_vector_double(nelem);

  /* initialize statistics */
  countopd = countopddot = 0;
  sumopd = sumopd2 = sumopddot = sumopddot2 = 0.0;

  /* loop over scans determining opd and derivative */
  for (i=0; i<on_raw->number_of_scans; i++) {
    /* Use QL scan, (I1-I2) filtered. */
    /* estimate delay from complex spectrum */
    vndrs_fft_real_2_complex(on_raw->fullscans[i]->QL , Spectrum);

    /* get moments */
    vndrs_get_cp_mom (Spectrum, hi, lo, speed, rate, ndelay,
		      &((*opd)->corpower[3*i]));

    /* compute residual */
    (*opd)->OPDresid[i] = vndrs_get_OPD (Spectrum, delta_f,
					 hi, lo, centerp, &snr);

  /* debug */
    /*fprintf (stdout,"delay %d %f %f\n",
	   i,(*opd)->OPDresid[i]*1.0e6,
	   on_raw->scan_data[i].dl_offset);*/
    /* add VINCI delay (to m) */
    (*opd)->OPDresid[i] +=  1.0e-6*on_raw->scan_data[i].dl_offset; 
    countopd++; /* statistics */
    sumopd  += (*opd)->OPDresid[i];
    sumopd2 += (*opd)->OPDresid[i] * (*opd)->OPDresid[i];

    /* estimate OPD rate from centroid of amplitude spectrum */
    vndrs_retrieve_modulus (Spectrum, Amp);
    /* compute first moment */
    (*opd)->OPDdot[i] = vndrs_get_moment(Amp, hi, lo, centerp, &snr);
    (*opd)->OPDdotdot[i] = 0.0; /* for now */
    /* set weight to snr */
    if ((*opd)->weight[i]>0.01) (*opd)->weight[i] = snr;
  } /* end of loop over scans */

  /* clean up */
  vndrs_free_vector_complex (Spectrum);
  vndrs_free_vector_double (Amp);

  /* adjust moments to mean zero */
  countopddot = 0;
  sumopddot = 0.0;
  for (i=0; i<on_raw->number_of_scans; i++) {
    if (((*opd)->OPDdot[i]!=MAGICBLANK) && ((*opd)->weight[i]>0.01)) {
      countopddot++;
      sumopddot += (*opd)->OPDdot[i];
    }
  }

  /* mean moment - assume is zero rate */
  if (countopddot>0) 
    /* mean_mom = sumopddot / countopddot; */
	mean_mom = (float) (sumopddot / countopddot);	/* csabet 27.Jun.01 */
  else
    mean_mom = 0.0;

  /* convert moments to OPD rates */
  countopddot = 0;
  sumopddot = 0.0;
  dOPDdt = delay_step / time_step; /* scanning rate */

  /* debug */
  /*fprintf (stdout,
	   "mean_mom %f lo %d hi %d scan_mode %d dOPDdt %f centerp %f\n",
	   mean_mom,lo,hi,scan_mode,dOPDdt,centerp);*/
  
  for (i=0; i<on_raw->number_of_scans; i++) {
  /* debug */
    /*fprintf (stdout,"rate %d %f %f \n",
	   i, (*opd)->OPDdot[i],
	   ((*opd)->OPDdot[i]-mean_mom)*dOPDdt*1.0e6 / centerp);*/
    if (((*opd)->OPDdot[i]!=MAGICBLANK) && ((*opd)->weight[i]>0.01)) {
      (*opd)->OPDdot[i] -= mean_mom;
      (*opd)->OPDdot[i] *= dOPDdt / centerp;
      countopddot++; /* statistics */
      sumopddot  += (*opd)->OPDdot[i];
      sumopddot2 += (*opd)->OPDdot[i] * (*opd)->OPDdot[i];
    } 
  }

  /* batch statistics, first OPD */
  if (countopd>1) {
    sumopd /= countopd;
    sumopd2 /= countopd;
    (*opd)->mean_OPD = sumopd;
    (*opd)->sigma_OPD = sqrt (sumopd2 - sumopd*sumopd);
  } else {
    (*opd)->mean_OPD = MAGICBLANK;
    (*opd)->sigma_OPD = MAGICBLANK;
  }
  /* then opd derivative*/
  if (countopd>1) {
    sumopddot /= countopddot;
    sumopddot2 /= countopddot;
    (*opd)->mean_OPDdot = sumopddot;
    (*opd)->sigma_OPDdot = sqrt (sumopddot2 - sumopddot*sumopddot);
  } else {
    (*opd)->mean_OPDdot  = MAGICBLANK;
    (*opd)->sigma_OPDdot = MAGICBLANK;
  }

  /* diagnostics */
  /* fprintf (stderr,
	   "# mean OPD %f sigma %f micron\n",
	   (*opd)->mean_OPD*1.0e6, (*opd)->sigma_OPD*1.0e6);
  fprintf (stderr,
	   "# mean OPD rate %f sigma %f micron/sec\n",
	   (*opd)->mean_OPDdot*1.0e6, (*opd)->sigma_OPDdot*1.0e6);
  */

  /* average coherence factors */
  countopd = 0;
  summu12 = summu22 = sum2mu12 = sum2mu22 = 0.0;
  for (i=0; i<on_batch->number_of_scans; i++) {
    if ((*opd)->weight[i]>0.01) {
      countopd++;
      summu12 += on_batch->scan_data[i].mu1_squared;
      summu22 += on_batch->scan_data[i].mu2_squared;
      sum2mu12 += on_batch->scan_data[i].mu1_squared * 
	on_batch->scan_data[i].mu1_squared;
      sum2mu22 += on_batch->scan_data[i].mu2_squared * 
	on_batch->scan_data[i].mu2_squared;
    }
  }
  (*opd)->nscans = countopd;
  if (countopd>1) {
     summu12 /= countopd;
     summu22 /= countopd;
     sum2mu12 /= countopd;
     sum2mu22 /= countopd;
    /* (*opd)->mu12 = summu12; */
	(*opd)->mu12 = (float) summu12;	/* csabet 27.Jun.01 */
    /* (*opd)->mu22 = summu22; */
	(*opd)->mu22 = (float) summu22;	/* csabet 27.Jun.01 */
    /* (*opd)->emu12 = sqrt ((sum2mu12-summu12*summu12) / (countopd-1)); */
	(*opd)->emu12 = (float) (sqrt ((sum2mu12-summu12*summu12) / (countopd-1)));	/* csabet 27.Jun.01 */
    /* (*opd)->emu22 = sqrt ((sum2mu22-summu22*summu22) / (countopd-1)); */
	(*opd)->emu22 = (float) (sqrt ((sum2mu22-summu22*summu22) / (countopd-1)));	/* csabet 27.Jun.01 */
  } else {
    (*opd)->mu12 = 0.0;
    (*opd)->mu22 = 0.0;
    (*opd)->emu12 = 0.0;
    (*opd)->emu22 = 0.0;
  }

  /* average photometry, correlated power moments */
  countscan = 0;
  summom0 = sum2mom0 = summom1 = sum2mom1 = summom2 = sum2mom2 = 0.0;
  sumph1 = sum2ph1 = sumph2 = sum2ph2 = 0.0;
  for (i=0; i<on_batch->number_of_scans; i++) {
    if ((*opd)->weight[i]>0.01) {
      countscan++;
      summom0  += (*opd)->corpower[3*i];
      sum2mom0 += (*opd)->corpower[3*i] * (*opd)->corpower[3*i];
      summom1  += (*opd)->corpower[3*i+1];
      sum2mom1 += (*opd)->corpower[3*i+1] * (*opd)->corpower[3*i+1];
      summom2  += (*opd)->corpower[3*i+2];
      sum2mom2 += (*opd)->corpower[3*i+2] * (*opd)->corpower[3*i+2];
      sumph1  += (*opd)->photom[2*i];
      sum2ph1 += (*opd)->photom[2*i] * (*opd)->photom[2*i];
      sumph2  += (*opd)->photom[2*i+1];
      sum2ph2 += (*opd)->photom[2*i+1] * (*opd)->photom[2*i+1];
    }
  }
  if (countscan>1) {
    summom0  /= countscan;
    sum2mom0 /= countscan;
    summom1  /= countscan;
    sum2mom1 /= countscan;
    summom2  /= countscan;
    sum2mom2 /= countscan;
    sumph1  /= countscan;
    sum2ph1 /= countscan;
    sumph2  /= countscan;
    sum2ph2 /= countscan;
    sum2mom0 = sqrt ((sum2mom0-summom0*summom0));
    sum2mom1 = sqrt ((sum2mom1-summom1*summom1));
    sum2mom2 = sqrt ((sum2mom2-summom2*summom2));
    sum2ph1 = sqrt ((sum2ph1-sumph1*sumph1));
    sum2ph2 = sqrt ((sum2ph2-sumph2*sumph2));
    (*opd)->mean_ph[0] = sumph1;
    (*opd)->mean_ph[1] = sumph2;
    (*opd)->mean_cp[0] = summom0;
    (*opd)->mean_cp[1] = summom1;
    (*opd)->mean_cp[2] = summom2;
    (*opd)->sigma_ph[0] = sum2ph1;
    (*opd)->sigma_ph[1] = sum2ph2;
    (*opd)->sigma_cp[0] = sum2mom0;
    (*opd)->sigma_cp[1] = sum2mom1;
    (*opd)->sigma_cp[2] = sum2mom2;
  } else {
    (*opd)->mean_ph[0] = 0.0;
    (*opd)->mean_ph[1] = 0.0;
    (*opd)->mean_cp[0] = 0.0;
    (*opd)->mean_cp[1] = 0.0;
    (*opd)->mean_cp[2] = 0.0;
    (*opd)->sigma_ph[0] = 0.0;
    (*opd)->sigma_ph[1] = 0.0;
    (*opd)->sigma_cp[0] = 0.0;
    (*opd)->sigma_cp[1] = 0.0;
    (*opd)->sigma_cp[2] = 0.0;
  }

} /* end vndrs_OPD_fit */

void vndrs_OPD_get_info (char *infile,
			 double *time_step, double *delay_step,
			 double *freq, double *minfreq,
			 double *maxfreq, int *scan_mode, 
			 double *speed, double *rate, 
			 char *name, char *Date_Obs, char* Observer,
			 char *band, char *station1,  char *station2, 
			 char *telescope, char *OBname, 
                         double *RA, double *Dec, 
                         char *equinox, char *Catg, char *ReadMode,
			 double *t1x, double *t1y, double *t1z,
			 double *t2x, double *t2y, double *t2z,
			 double *t3x, double *t3y, double *t3z,
			 int *ierr)
/*----------------------------------------------------------------------- */
/*   Get information from header needed to interprete the scans           */
/*   Inputs:                                                              */
/*      infile    input file with header keywords                         */
/*   Output:                                                              */
/*      time_step   time interval in seconds between frames in s          */
/*      delay_step  delay step between frames in meters                   */
/*      freq        "center" frequency of bandpass in Hz                  */
/*      minfreq     lowest frequency of bandpass in Hz                    */
/*      maxfreq     highest frequency of bandpass in Hz                   */
/*      scan_mode   Scaning mode, 0=none, 1=up+down, 2=up, 3=down only    */
/*      name        target name                                           */
/*      Date_Obs    Observing date                                        */
/*      Observer    Observer's name                                       */
/*      band        Observing band                                        */
/*      station1    first station name                                    */
/*      station2    second station name                                   */
/*      telescope   telescope name                                        */
/*      OBname      Bob blob name                                         */
/*      RA          Target RA (deg)                                       */
/*      Dec         Target Dec (deg)                                      */
/*      equinox     Coordinate equinox                                    */
/*      Catg        Observation category                                  */
/*      ReadMode    Readout Mode                                          */
/*      t1x,t1y,... Station coordinates                                   */
/*      ierr        Error code: 0 => ok, checked on input                 */
/*----------------------------------------------------------------------- */
{

	/*	Local declaration
	-------------------*/
	double		maxwave, minwave;
	char		comment[60], mode[40];
	fitsfile	*infptr;
	char		*default_str="Calibration";
	int			i;


	/*	Algorithm
	-----------*/

	/*	Allocate memory (csabet 02.Jul.01)
	-----------------*/
	default_str = (char *) calloc(MAX_STRING_LENGTH, sizeof(char));
	if(!default_str) 
	{
		printf("\nError. Memory allocation. vndrs_OPD_get_info(default_str).\n");
		vndrs_error_alert();
	}


	if (*ierr!=0) return; /* existing error? */


	/*	Open input
	------------*/
	fits_open_file (&infptr, infile, READONLY, ierr);
	if (*ierr>0) 
	{
		fprintf (stderr,"vndrs_OPD_get_info: error %d opening input %s\n", *ierr, infile);
		return;  /* bomb out */
	}

  /* frame rate in Hz */
  fits_read_key(infptr, TDOUBLE, "HIERARCH ESO DET QL FRAMERATE", 
		rate, comment, ierr); 
  if (*ierr>0) return;

  /* fringe speed (m/s)  */
  fits_read_key(infptr, TDOUBLE, "HIERARCH ESO DET QL FRINGESPD", 
		speed, comment, ierr); 
  if (*ierr>0) return;

  /*  max wave length microns  */
  fits_read_key(infptr, TDOUBLE, "HIERARCH ESO DET QL MAXWAVEK", 
		&maxwave, comment, ierr); 
  if (*ierr>0) return;

  /* min wave length microns  */
  fits_read_key(infptr, TDOUBLE, "HIERARCH ESO DET QL MINWAVEK", 
		&minwave, comment, ierr); 
  if (*ierr>0) return;

  /* scanning mode  */
  fits_read_key(infptr, TINT, "HIERARCH ESO DET DACQ MODE",
		scan_mode, comment, ierr); 
  if (*ierr>0) return;

  /* source name - use INS MODE as the default for autotest/col */
  for (i=0; i<40; i++) mode[i] = 0;
  default_fits_read_key_str(infptr, "HIERARCH ESO INS MODE", 
     mode, comment, (void*)default_str, ierr);
  if (*ierr>0) return;
  default_fits_read_key_str(infptr, "HIERARCH ESO OBS TARG NAME", 
     name, comment, (void*)mode, ierr);
  if (*ierr>0) return;

  /* other info */
  default_fits_read_key_str(infptr, "DATE-OBS", Date_Obs, comment,  
			    (void*)default_str, ierr);
  if (*ierr>0) return;
  default_fits_read_key_str(infptr, "HIERARCH ESO OBS OBSERVER", 
			    Observer, comment,  
			    (void*)default_str, ierr);
  if (*ierr>0) return;
  default_fits_read_key_str(infptr, "HIERARCH ESO INS FILT1 NAME",
			    band, comment,  
			    (void*)default_str, ierr);
  if (*ierr>0) return;
  default_fits_read_key_str(infptr, "HIERARCH ESO ISS CONF STATION1", 
			    station1, comment,  
			    (void*)default_str, ierr);
  if (*ierr>0) return;
  default_fits_read_key_str(infptr, "HIERARCH ESO ISS CONF STATION2", 
			    station2, comment,  
			    (void*)default_str, ierr);
  if (*ierr>0) return;

  /* added pkervell 14-apr-02 : station coordinates */

  fits_read_key(infptr, TDOUBLE, "HIERARCH ESO ISS CONF T1X", 
			    t1x, comment, ierr);
  if (*ierr>0) return;

  fits_read_key(infptr, TDOUBLE, "HIERARCH ESO ISS CONF T1Y", 
			    t1y, comment, ierr);
  if (*ierr>0) return;

  fits_read_key(infptr, TDOUBLE, "HIERARCH ESO ISS CONF T1Z", 
			    t1z, comment, ierr);
  if (*ierr>0) return;

  fits_read_key(infptr, TDOUBLE, "HIERARCH ESO ISS CONF T2X", 
			    t2x, comment, ierr);
  if (*ierr>0) return;

  fits_read_key(infptr, TDOUBLE, "HIERARCH ESO ISS CONF T2Y", 
			    t2y, comment, ierr);
  if (*ierr>0) return;

  fits_read_key(infptr, TDOUBLE, "HIERARCH ESO ISS CONF T2Z", 
			    t2z, comment, ierr);
  if (*ierr>0) return;
/*
  fits_read_key(infptr, TDOUBLE, "HIERARCH ESO ISS CONF T3X", 
			    t3x, comment, ierr);
  if (*ierr>0) return;
  fits_read_key(infptr, TDOUBLE, "HIERARCH ESO ISS CONF T3Y", 
			    t3y, comment, ierr);
  if (*ierr>0) return;
  fits_read_key(infptr, TDOUBLE, "HIERARCH ESO ISS CONF T3Z", 
			    t3z, comment, ierr);
			    if (*ierr>0) return; */

  default_fits_read_key_str(infptr, "TELESCOP", 
			    telescope, comment,  
			    (void*)default_str, ierr);
  if (*ierr>0) return;

  default_fits_read_key_str(infptr, "HIERARCH ESO OBS NAME", 
			    OBname, comment,  
			    (void*)default_str, ierr);
  if (*ierr>0) return;

  /* RA */
  *RA = 0.0;
  default_fits_read_key_dbl(infptr, "RA", RA, comment,  
			    (void*)RA, ierr);
  if (*ierr>0) return;

  /* Dec */
  *Dec = 0.0;
  default_fits_read_key_dbl(infptr, "DEC", Dec, comment,  
			    (void*)Dec, ierr);
  if (*ierr>0) return;

  /* Equinox */
  strcpy (default_str, "J2000");
  default_fits_read_key_str(infptr, "EQUINOX", 
			    equinox, comment,  
			    (void*)default_str,ierr);
  if (*ierr>0) return;

  /* category */
  strcpy (default_str, "     ");
  default_fits_read_key_str(infptr, "HIERARCH ESO DPR CATG", 
			    Catg, comment,  
			    (void*)default_str, ierr);
  if (*ierr>0) return;

  /* read out mode */
  strcpy (ReadMode, "     ");
  default_fits_read_key_str(infptr, "HIERARCH ESO DET NCORRS NAME", 
			    ReadMode, comment,  
			    (void*)default_str, ierr);
  if (*ierr>0) return;


  /* convert to return values */
  *time_step  = 1.0 / (*rate);
  *delay_step = *time_step * (*speed);
  *freq       = CLIGHT / (0.5 * (maxwave+minwave));
  *minfreq    = CLIGHT / maxwave;
  *maxfreq    = CLIGHT / minwave;

	/*	Free allocated memory
	-----------------------*/
	free (default_str);

} /* end vndrs_OPD_get_info */

void vndrs_get_cp_mom (fourier_spectrum *Spectrum,  int hi, int lo, 
		       double ScanSpeed, double FrameRate, int nFrames, 
		       double *mom)
/*----------------------------------------------------------------------- */
/*   Calculate moments of a power spectrum.                               */
/*   moment 0 = integrated power                                          */
/*   moment 1 = position of the spectral peak (meters)                    */
/*   moment 2 = width of the spectral peak (meters)                       */
/*   Inputs:                                                              */
/*      Spectrum     Complex spectrum                                     */
/*      delta_f      Spectral frequency step in Hz                        */
/*      hi           last channel in selected region                      */
/*      lo           first channel in selected region                     */
/*      centerp      Center pixel of the bandpass                         */
/*   Outputs:                                                             */
/*      mom          Moments 0, 1, 2 in meters                            */
/*----------------------------------------------------------------------- */
{
  int   i;
  float amp, wave;
  double sum0, sum1, sum2, mom0, mom1, mom2;

   sum0 = sum1 = sum2 = 0.0;
  /* loop over channels */
   for (i=lo; i<=hi; i++) {
     /* spectral amplitude */
     /* amp = Spectrum->data[i].re*Spectrum->data[i].re + Spectrum->data[i].im*Spectrum->data[i].im; */
	amp = (float) (Spectrum->data[i].re*Spectrum->data[i].re + 
		Spectrum->data[i].im*Spectrum->data[i].im);	/* csabet 27.Jun.01 */

     /* wave = ScanSpeed / (i * FrameRate / nFrames); */
	wave = (float) (ScanSpeed / (i * FrameRate / nFrames)); /* csabet 27.Jun.01 */

     sum0 += amp;
     sum1 += wave * amp;
     sum2 += wave * wave * amp;
   } /* end loop over channels */
   /* compute moments */
   mom0 = sum0;
   if (fabs(mom0) < 1.0e-10) mom0 = 1.0; /* sanity check */
   mom1 = sum1 / mom0;
   mom2 = (sum2 / mom0) - mom1*mom1;
   if (mom2 < 1.0e-30) mom2 = 0.0;       /* sanity check */
   mom2 = sqrt(mom2);

   /* debug */
   /*fprintf (stderr, "moments = %lf %lf %lf\n",mom0,mom1*1.0e6,mom2*1.0e6);*/
   /* return values */
   mom[0] = mom0;
   mom[1] = mom1;
   mom[2] = mom2;
} /* end  vndrs_get_cp_mom */

void vndrs_get_scan_photometry (batch *on_raw,  batch *off_raw, 
				 double *photom)
/*----------------------------------------------------------------------- */
/*   Compute average value of the photometric values, subtracting the     */
/*   average of the off source scans.                                     */
/*   Inputs:                                                              */
/*      on_raw    on source raw batch to be evaluated                     */
/*      on-batch  on source processed batch to be evaluated               */
/*      off_raw   off source raw batch to be evaluated                    */
/*   Outputs:                                                             */
/*      photom    pairs of scan average photometric values.               */
/*----------------------------------------------------------------------- */
{
  int i, j, On_nscans, Off_nscans, On_nframes, Off_nframes;
  double *data1, *data2, sum1, sum2;
  float  Off_mean1, Off_mean2;
  long  count;

  /* how many scans? */
  On_nscans  = on_raw->number_of_scans;
  Off_nscans = off_raw->number_of_scans;

  /* how many frames? */
  On_nframes = on_raw->fullscans[0]->PA->dimension;
  Off_nframes = off_raw->fullscans[0]->PA->dimension;

  /* average off source scans */
  sum1 = sum2 = 0.0;
  count = 0;
  for (i=0; i<Off_nscans; i++) {
    data1 = off_raw->fullscans[i]->PA->data;
    data2 = off_raw->fullscans[i]->PB->data;
    for (j=0; j<Off_nframes; j++) {
      count++;
      sum1 += data1[j];
      sum2 += data2[j];
    }
  }
  if (count>1) {
    /* Off_mean1 = sum1/count; */
	Off_mean1 = (float) (sum1/count); /* csabet 27.Jun.01 */
    /* Off_mean2 = sum2/count; */
	Off_mean2 = (float) (sum2/count); /* csabet 27.Jun.01 */
  } else {
    Off_mean1 = 0.0;
    Off_mean2 = 0.0;
  }

 /* average on source scans */
  for (i=0; i<On_nscans; i++) {
    sum1 = sum2 = 0.0;
    count = 0;
    data1 = on_raw->fullscans[i]->PA->data;
    data2 = on_raw->fullscans[i]->PB->data;
    for (j=0; j<On_nframes; j++) {
      count++;
      sum1 += data1[j];
      sum2 += data2[j];
    }
    if (count>1) {
      photom[2*i]   = sum1/count -  Off_mean1;
      photom[2*i+1] = sum2/count - Off_mean2;
    } else {
      photom[2*i]   = 0.0;
      photom[2*i+1] = 0.0;
    }
  }

} /* end vndrs_get_scan_photometry */
double vndrs_get_OPD (fourier_spectrum *Spectrum, double delta_f,
		      int hi, int lo, float centerp, float *snr)
/*----------------------------------------------------------------------- */
/*   Calculate the OPD residual from a spectrum                           */
/*   Method is a direct parameter search by computing a phase ramp for    */
/*   each test delay, multiplying the spectrum and integrating the        */
/*   amplitude over the expected region of the source response.           */
/*   Note: lag function in ascending order of OPD.                        */
/*   Inputs:                                                              */
/*      Spectrum     Complex spectrum                                     */
/*      delta_f      Spectral frequency step in Hz                        */
/*      hi           last channel in selected region                      */
/*      lo           first channel in selected region                     */
/*      centerp      Center pixel of the bandpass                         */
/*   Returns:                                                             */
/*      OPD residual in meters, MAGICBLANK if failed                      */
/*----------------------------------------------------------------------- */
{
#define NSEARCH 200  /* number of cells to search */
  double out = 0.0;
  int  nfreq;
  int count, countx;
  int i, j, m, imax;
  float delay;
  float delta, tdelay;
  float testsnr[NSEARCH], testamp[NSEARCH], testdel[NSEARCH];
  float maxv;
  float xre, xim, red, imd, sumre, sumim, sumxre, sumxim;

#ifdef DEBUG_SESSION	/* csabet 27.Jun.01 */
	float	amp;
#endif

  nfreq = Spectrum->dimension/2;

  /* debug */
  /* fprintf (stdout,"nfreq %d lo %d hi %d centerp %f\n", 
     nfreq, lo, hi, centerp);*/

  /* loop over test delays */
  m = NSEARCH;             /* number of attempts */
  /* delta = 0.05 * TWOPI / m; */ /* delay step in rad per cell */
	/* delay step in rad per cell */
	delta = (float) (0.05 * TWOPI / m); /* csabet 27.Jun.01 */

  for (j=0; j<m; j++) {
    tdelay = (j - m/2) * delta; /* test delay */
    testdel[j] = tdelay;

    /* zero accumulators */
    sumre  = 0.0; sumim  = 0.0; count = 0;
    sumxre = 0.0; sumxim = 0.0; countx = 0;
    for (i=lo/2; i<nfreq; i++) {
      /* compute phase ramp from tdelay */
      /* red = cos ((i-centerp) * tdelay); */
		red = (float) (cos ((i-centerp) * tdelay)); /* csabet 27.Jun.01 */
      /* imd = sin ((i-centerp) * tdelay); */
		imd = (float) (sin ((i-centerp) * tdelay)); /* csabet 27.Jun.01 */

      /* multiply spectrum */
      /* xre = red * Spectrum->data[i].re - imd * Spectrum->data[i].im; */
		xre = (float) (red * Spectrum->data[i].re - imd * Spectrum->data[i].im); /* csabet 27.Jun.01 */
      /* xim = red * Spectrum->data[i].im + imd * Spectrum->data[i].re; */
		xim = (float) (red * Spectrum->data[i].im + imd * Spectrum->data[i].re); /* csabet 27.Jun.01 */

#ifdef DEBUG_SESSION	/* csabet 27.Jun.01 */
	/* debug */
	amp = sqrt (xre*xre + xim*xim); 
	if (j==0) fprintf (stdout, "spectrum %d %g\n ",i,amp);
#endif

      /* sum in and out of band separately */
      if ((i>=lo) && (i<=hi)) { /* desired */
	sumre += xre;
	sumim += xim;
	count++;
      } else { /* out of band */
	sumxre += xre;
	sumxim += xim;
	countx++;
      }
    } /* end loop summing spectrum */

    /* amplitude */
    testamp[j] = (sumre*sumre + sumim*sumim);

    /* snr */
    sumre /= count;
    sumim /= count;
    sumxre /= countx;
    sumxim /= countx;
    testsnr[j] = (sumre*sumre + sumim*sumim) / 
      (sumxre*sumxre + sumxim*sumxim);
    /* debug */
    /*fprintf (stdout," %d %f %f %f %f %f %d %f %f %d \n", 
	     j,tdelay,testsnr[j], testamp[j],sumre,sumim,
	     count,sumxre,sumxim,countx);*/
  } /* end loop over trials */
  
  /* find peak in integrated amplitude */
  imax = -1; maxv = 0.0;
  for (i=0; i<m; i++) {
    if (maxv<testsnr[i]) {
      imax = i;
      maxv = testsnr[i];
    }
  }
  
  /* set final values */
  delay = testdel[imax]; /* radians per cell */
  /* *snr = sqrt (testsnr[imax]); */
	*snr = (float) (sqrt (testsnr[imax])); /* csabet 27.Jun.01 */

  /* debug */
  /*out = ((delay / TWOPI) / delta_f)*CLIGHT;
  fprintf (stdout,"%d delay %f snr %f out %f\n", 
  imax, delay,*snr, out);*/

  /* delay in meters */
  out = ((delay / TWOPI) / delta_f) * CLIGHT;

  /* make sure minimum SNR */
  if (*snr<2.0) out = MAGICBLANK;

  return out;
} /* end  vndrs_get_OPD */

double vndrs_get_moment(power_spectrum *Amp, int hi, int lo, 
			float centerp, float *snr)
/*----------------------------------------------------------------------- */
/*   Calculate the first moment an amplitude spectrum in a specified      */
/*   interval.  Makes bias correction from high end out of band region.   */
/*   Note: lag function in ascending order of OPD.                        */
/*   Inputs:                                                              */
/*      Amp          Amplitude of spectrum                                */
/*      hi           last channel in selected region                      */
/*      lo           first channel in selected region                     */
/*      centerp      Center pixel of the bandpass                         */
/*   Returns:                                                             */
/*      First moment in pixels, MAGICBLANK if failed                      */
/*----------------------------------------------------------------------- */
{
  double out = 0.0;
  int i, nfreq, maxi;
  float amp, x, bias;
  double sum0, sum1;

  nfreq = Amp->dimension/2;

  /* first measure amplitude bias */
  sum0 = sum1 = bias = 0.0;
  for (i=lo/2; i<lo-1; i++) {
    sum0++;
    sum1 += Amp->data[i];
  }

  maxi = hi + (nfreq-hi)/4;
  for (i=hi; i<maxi; i++) {
    sum0++;
    sum1 += Amp->data[i];
  }
  /* determine bias */
  /* if (sum0>0.0) bias = sum1 / sum0; */
	if (sum0>0.0) bias = (float) (sum1 / sum0); /* csabet 27.Jun.01 */

  /* get first moment in band about centerp */
  sum0 = sum1 = 0.0;
  for (i=lo; i<hi; i++) {
    /* amp = Amp->data[i] - bias; */
	amp = (float) (Amp->data[i] - bias); /* csabet 27.Jun.01 */

    x = (float)(i) - centerp;
    sum0 += amp;
    sum1 += amp * x;
  }

  /* get moment, must be bright enough */
  /* *snr = sum0 / (bias * (hi - lo + 1)); */
	*snr = (float) (sum0 / (bias * (hi - lo + 1)));	/* csabet 27.Jun.01 */

  if (*snr > 2.0) {
    out = centerp + sum1 / sum0;
  } else { /* bad */
    out =  MAGICBLANK;
  }

  /* debug */
  /*i = (lo+hi) / 2;
  fprintf (stderr,"get_mom: lo= %f bias=%f sum0=%f center=%f snr=%f\n"
  , Amp->data[lo], bias,sum0, Amp->data[i], *snr);*/

  return out;
} /* end  vndrs_get_OPD_derivative */
