/* $Id: vndrs_OPD.h,v 1.1 2001/07/24 09:42:17 vltip Exp $ */
#ifndef _VNDRS_OPD_H
#define _VNDRS_OPD_H
/*----------------------------------------------------------------------- */
/* VINCI OPD estimation routines                                          */
/* A call to vndrs_initialization is needed before calling these routines.*/
/* W. D. Cotton  March, 2001                                              */ 
/*----------------------------------------------------------------------- */
#include "vndrs.h"
#define CLIGHT 2.997924562e8 /* speed of light m/s */
#define TWOPI 6.283185308    /* two pi */
#define MAGICBLANK (-999.99)   /* Magic blanking value for bad values */

typedef struct
{
  long   number;       /* the number of elements */
  double *time;        /* MJD time of elements */
  double avg_time;     /* average time (days) */
  double time_interv;  /* time interval (days) */
  double *OPDresid;    /* scan estimates of OPD residual in m */
  double *OPDdot;      /* scan estimates of OPD derivatives in m/s */
  double *OPDdotdot;   /* scan estimates of OPD 2nd derivatives in m/s/s */
  double *weight;      /* weight for each scan */
  double *photom;      /* photometric scan averages, 2 element per entry */
  double *corpower;    /* scan correlated power moments, 3 element per entry*/
  double mean_OPD;     /* mean of OPD */
  double mean_OPDdot;  /* mean of OPD derivative*/
  double mean_OPDdotdot;/* mean of OPD 2nd derivative*/
  double mean_ph[2];   /* mean photometric value for batch */
  double mean_cp[3];   /* mean correlated power moments for batch */
  double sigma_OPD;    /* std. deviation of population of OPD */
  double sigma_OPDdot; /* std. deviation of population of OPD derivative*/
  double sigma_OPDdotdot; /* std. deviation of population of OPD 2nd derivative*/
  double sigma_ph[2];  /* std. deviation of photometric value for batch */
  double sigma_cp[3];  /* std. deviation of correlated power moments for batch */
  double tau_0;        /* estimate of coherence time for batch in s */
  double RA;           /* target ra in degrees */
  double Dec;          /* target dec in degrees */
  float  mu12;         /* average mu12 */
  float  mu22;         /* average mu22 */
  float  emu12;        /* std. error in mu12 */
  float  emu22;        /* std. error in mu22 */
  float  fringerate;   /* fringe rate in Hz */
  long   nscans;       /* number of scans includes */
  char   name[50];     /* source name */
  char   Date_Obs[50]; /* Observing date, time */
  char   Observer[50]; /* Observer's name */
  char   band[10];     /* observing band */
  char   station1[10]; /* first station name */
  char   station2[10]; /* first station name */
  char   telescope[20];/* telescope name */
  char   OBname[50];   /* Bob blob name */
  char   equinox[10];  /* Coordinate equinox */
  char   Catg[20];     /* observation category */
  char   ReadMode[20]; /* readout mode */
} opd_data;
/* An 'opd_data' stores the OPD residual estimates from a batch */

/* public functions */
opd_data* vndrs_calloc_opd_data (long number);
/*----------------------------------------------------------------------- */
/*   Allocate an opd_data structure                                       */
/*   Inputs:                                                              */
/*      number    number of elements in structure                         */
/*   Returns:                                                             */
/*      an opd_data structure pointer                                     */
/*----------------------------------------------------------------------- */

void vndrs_free_opd_data(opd_data* data);
/*----------------------------------------------------------------------- */
/*   Free an opd_data structure                                           */
/*   Inputs:                                                              */
/*      data    the structure to be deallocated                           */
/*----------------------------------------------------------------------- */

void vndrs_OPD_fit (char *infile, batch *on_raw,  batch *on_batch, 
		    batch *off_raw,  opd_data** opd, int *ierr);
/*----------------------------------------------------------------------- */
/*   Estimate OPD residual and first time derivative.                     */
/*   OPD residual is estimated from the slope of the phase across the     */
/*   bandpass, the time derivative is estimated from the centroid of the  */
/*   power spectrum.  In both cases, the results are adjusted that the    */
/*   batch average is zero.                                               */
/*   Inputs:                                                              */
/*      infile    input file with header keywords                         */
/*      on-raw    on source raw batch to be evaluated                     */
/*      on-batch  on source processed batch to be evaluated               */
/*      off-raw   off source raw batch to be evaluated                    */
/*   Output:                                                              */
/*      opd       opd_data structure giving the results                   */
/*                This structure is allocated and the memory must be      */
/*                deallocated using vndrs_free_opd_data.                  */
/*                OPDs in meters and rates in m/s                         */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*----------------------------------------------------------------------- */
#endif /* end VNDRS_OPD_H */
